/**
 *                      ==================
 *                      IITK OSD Simulator
 *                      ==================
 *   An Implementation of Simulator for Object based storage device(OSD)
 *
 *   Copyright (C) 2007-2008 CSE IIT KANPUR
 *
 *   Contact: Department of Computer Science and Engineering
 *            Indian Institute of Technology,Kanpur(IIT Kanpur)
 *            {moona,riyaz}@cse.iitk.ac.in
 *
 *   This file is part of IITK OSD Simulator Project.
 *
 *   IITK OSD Simulator Project is free software; you can redistribute it
 *   and/or modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   IITK OSD Simulator Project is distributed in the hope that it will be
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the  GNU General
 *   Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
# include "osd.h"
# include "osd_attr2.h"
# define be8_to_cpu(arg)  arg
extern int DEBUG;
static int dlevel=UPPER_FUNC;

/**
**********************************
 * File:  osd.c
 * General Functions Used by all OSD components
 * Encap/decap command specific parameters
 **/
//----------------------------------------------------------------------------------------
u32 handle_osd_cmnd_get_din_size(OSD_CDB_STRUCT *osd_cdb)
{
	OSD_CDB_BASIC_HDR *osd_cdb_basic_hdr;
	OSD_CDB_RW_COMMAND_HDR *osd_cdb_rw_hdr;
	OSD_CDB_ATTR_LIST_HDR *osd_cdb_list_hdr;
	u32 din_size=0,size;
	int osd_cdb_attr_fmt=0;

	osd_cdb_basic_hdr=(OSD_CDB_BASIC_HDR *)osd_cdb;
	osd_cdb_rw_hdr=(OSD_CDB_RW_COMMAND_HDR *)osd_cdb;
	osd_cdb_list_hdr=(OSD_CDB_ATTR_LIST_HDR *)osd_cdb;
	
	if(osd_cdb_basic_hdr->service_action== OSD_CMD_WRITE || osd_cdb_basic_hdr->service_action==OSD_CMD_APPEND )
	{
		osd_cdb_rw_hdr=(OSD_CDB_RW_COMMAND_HDR *)osd_cdb;
		din_size= osd_cdb_rw_hdr->length;
	}
    
	osd_cdb_attr_fmt=osd_cdb_basic_hdr->option_byte2 & 63;
	osd_cdb_attr_fmt =osd_cdb_attr_fmt >> 4;
		
	if(osd_cdb_attr_fmt == OSD_CDB_ATTR_FORMAT_LIST)
	{
		osd_cdb_list_hdr=(OSD_CDB_ATTR_LIST_HDR *)osd_cdb;
		
		if(osd_cdb_list_hdr->get_list_length != 0)
		{
			size=osd_cdb_list_hdr->get_list_dout_offset+osd_cdb_list_hdr->get_list_length;
			if(din_size < size)
			din_size=size;
		}
		
		
		if(osd_cdb_list_hdr->set_list_length!=0)
		{
			size=osd_cdb_list_hdr->set_list_length+osd_cdb_list_hdr->set_list_dout_offset;
			if(din_size < size)
			din_size=size;
		}
			
	}
	din_size+=255;
	din_size &= ~(255);
	
	return din_size;
}


//----------------------------------------------------------------------------------------
u32 handle_osd_cmnd_get_dout_size(OSD_CDB_STRUCT *osd_cdb)
{
	OSD_CDB_BASIC_HDR *osd_cdb_basic_hdr;
	OSD_CDB_RW_COMMAND_HDR *osd_cdb_rw_hdr;
	OSD_CDB_ATTR_LIST_HDR *osd_cdb_list_hdr;
	

	u32 dout_size=0,size;
	int osd_cdb_attr_fmt=0;

	osd_cdb_basic_hdr=(OSD_CDB_BASIC_HDR *)osd_cdb;
	osd_cdb_rw_hdr=(OSD_CDB_RW_COMMAND_HDR *)osd_cdb;
	osd_cdb_list_hdr=(OSD_CDB_ATTR_LIST_HDR *)osd_cdb;
	
	if(osd_cdb_basic_hdr->service_action== OSD_CMD_READ)
	{
		osd_cdb_rw_hdr=(OSD_CDB_RW_COMMAND_HDR *)osd_cdb;
		dout_size= osd_cdb_rw_hdr->length;
		
	}else if(osd_cdb_basic_hdr->service_action== OSD_CMD_LIST)
	{
		OSD_CDB_LIST_COMMAND_HDR *osd_cdb_list_command_hdr;
		osd_cdb_list_command_hdr=(OSD_CDB_LIST_COMMAND_HDR *)osd_cdb;
		dout_size= osd_cdb_list_command_hdr->allocation_length;
	
	}else if(osd_cdb_basic_hdr->service_action== OSD_CMD_LIST_COLLECTION)
	{
		OSD_CDB_LIST_COLLECTION_COMMAND_HDR *osd_cdb_list_coll_command_hdr;
		osd_cdb_list_coll_command_hdr=(OSD_CDB_LIST_COLLECTION_COMMAND_HDR *)osd_cdb;
		dout_size= osd_cdb_list_coll_command_hdr->allocation_length;
		
	}
    
	osd_cdb_attr_fmt=osd_cdb_basic_hdr->option_byte2 & 63;
	osd_cdb_attr_fmt =osd_cdb_attr_fmt >> 4;
		
	if(osd_cdb_attr_fmt == OSD_CDB_ATTR_FORMAT_LIST)
	{
		osd_cdb_list_hdr=(OSD_CDB_ATTR_LIST_HDR *)osd_cdb;
		
		if(osd_cdb_list_hdr->get_list_length != 0)
		{ 
		
			size=osd_cdb_list_hdr->retrieved_attr_din_offset+osd_cdb_list_hdr->get_allocation_length;
			if(dout_size < size)
			dout_size=size;
		}
	}
	dout_size+=255;
	dout_size &= ~(255);

	return dout_size;
}

//----------------------------------------------------------------------------------------
int handle_osd_cmnd_get_scsi_ahs_offset(u8 *ahs, u32 ahssize)
{
	int i,len=ahssize;
  	//u8 *ahs=(u8 *)cmnd->pdu.ahs;
	u16 ahslen;u8 ahstype,ahsspec;u32 expreaddatalen;

	for(i=0;;)
	{
		ahslen=be16_to_cpu(*(u16 *)((u8 *)ahs + i));
		ahstype=*((u8 *)((u8 *)(ahs)+i+2));
		ahsspec=*((u8 *)((u8 *)(ahs)+i+3));
		expreaddatalen=be32_to_cpu(*((u32 *)((u8 *)(ahs)+i+4)));
		
		if(ahstype==2)
		{
			print_msg(dlevel,"OSD:Got AHSType=0x%x  ActualAHSLen=0x%x AHSSpec=0x%x ExpReadDataLen=0x%x \n", ahstype,
			ahslen,ahsspec,expreaddatalen);
			i=i+8;
		}
		else if(ahstype ==1)
		{
			if(ahslen == 185)
			{
				print_msg(dlevel,"OSD:Got AHSType=0x%x  ActualAHSLen=0x%x AHSSpec=0x%x \nOSD: AHS Offset =  0x%x \n", ahstype,ahslen,ahsspec,i+4);
				return i+4;
			}
			else
			{
				print_msg(dlevel,"OSD:Got AHSType=0x%x  ActualAHSLen=0x%x AHSSpec=0x%x WRONG LENTH\n", ahstype,ahslen,ahsspec);
				return 0;
			}
		}
		else
		{
			print_msg(dlevel,"OSD:Got AHSType=0x%x  NOT HANDLED\n", ahstype);
			return 0;
		}
		if(i > len - 4)
		{
			print_msg(dlevel,"OSD:WRONG AHS HEADER  \n");
			return 0;
		
		}
	}

}
//---------------------------------------------------------------------------------
int handle_osd_cmnd_get_osd_cdb_struct(OSD_HEADER *kern_usr_meta, OSD_CDB_STRUCT * osd_cdb)
{
	OSD_CDB_BASIC_HDR *osd_cdb_basic_hdr;
	
	OSD_CDB_ANY_COMMAND_HDR *osd_cdb_any_hdr;
	OSD_CDB_ATTR_LIST_HDR *osd_cdb_list_hdr;
	
	int ahs_offset=0;
	int osd_cdb_attr_fmt=0;

	osd_cdb_basic_hdr=(OSD_CDB_BASIC_HDR *)osd_cdb;
	decap_osd_cdb_basic_hdr(osd_cdb_basic_hdr,kern_usr_meta->scb);
	print_osd_cdb_basic_hdr(osd_cdb_basic_hdr);

	ahs_offset=handle_osd_cmnd_get_scsi_ahs_offset(kern_usr_meta->ahs_data,kern_usr_meta->ahs_len);

	if(ahs_offset==0)
	{
		print_msg(ERROR_MSG,"OSD: problem in getting ahs offset for SCSI AHS in pdu.ahs\n");
		return 1;
	}
	print_msg(dlevel,"OSD:TARGET OSD_CDB  OSD CODE= 0x%x COMMAND is \n",osd_cdb_basic_hdr->service_action);
	switch(osd_cdb_basic_hdr->service_action)
	{
	case OSD_CMD_FORMAT:
		{
		OSD_CDB_FORMAT_COMMAND_HDR *osd_cdb_format_hdr=(OSD_CDB_FORMAT_COMMAND_HDR *)osd_cdb;
		print_msg(dlevel,"OSD_CMD_FORMAT");
		decap_osd_cdb_format_hdr(osd_cdb_format_hdr,(u8 *)(kern_usr_meta->ahs_data)+ahs_offset);
		//cmnd->pdu.scsiahs);
		print_osd_cdb_format_hdr(osd_cdb_format_hdr);
		}
		break;	
		
	case OSD_CMD_CREATE_PARTITION:
		{
		OSD_CDB_CREATE_PARTITION_COMMAND_HDR *osd_cdb_create_partition_hdr=(OSD_CDB_CREATE_PARTITION_COMMAND_HDR *)osd_cdb;
		print_msg(dlevel,"OSD_CMD_CREATE_PARTITION");
		decap_osd_cdb_create_partition_hdr(osd_cdb_create_partition_hdr,(u8 *)(kern_usr_meta->ahs_data)+ahs_offset);
		//cmnd->pdu.scsiahs);
		print_osd_cdb_create_partition_hdr(osd_cdb_create_partition_hdr);
		
		}
		break;	
	case OSD_CMD_CREATE_COLLECTION:
		{
		OSD_CDB_CREATE_COLLECTION_COMMAND_HDR *osd_cdb_create_collection_hdr=(OSD_CDB_CREATE_COLLECTION_COMMAND_HDR *)osd_cdb;
		print_msg(dlevel,"OSD_CMD_CREATE_COLLECTION");
		decap_osd_cdb_create_collection_hdr(osd_cdb_create_collection_hdr,(u8 *)(kern_usr_meta->ahs_data)+ahs_offset);
		//cmnd->pdu.scsiahs);
		print_osd_cdb_create_collection_hdr(osd_cdb_create_collection_hdr);
		
		}
		break;	

	case OSD_CMD_ADD_TO_COLLECTION:
		{
		OSD_CDB_ADD_TO_COLLECTION_COMMAND_HDR *osd_cdb_add_to_collection_hdr=(OSD_CDB_ADD_TO_COLLECTION_COMMAND_HDR *)osd_cdb;
		print_msg(dlevel,"OSD_CMD_ADD_TO_COLLECTION");
		decap_osd_cdb_add_to_collection_hdr(osd_cdb_add_to_collection_hdr,(u8 *)(kern_usr_meta->ahs_data)+ahs_offset);
		//cmnd->pdu.scsiahs);
		print_osd_cdb_add_to_collection_hdr(osd_cdb_add_to_collection_hdr);
		
		}
		break;	
	
	case OSD_CMD_REMOVE_FROM_COLLECTION:
		{
		OSD_CDB_REMOVE_FROM_COLLECTION_COMMAND_HDR *osd_cdb_remove_from_collection_hdr=(OSD_CDB_REMOVE_FROM_COLLECTION_COMMAND_HDR *)osd_cdb;
		print_msg(dlevel,"OSD_CMD_REMOVE_FROM_COLLECTION");
		decap_osd_cdb_remove_from_collection_hdr(osd_cdb_remove_from_collection_hdr,(u8 *)(kern_usr_meta->ahs_data)+ahs_offset);
		//cmnd->pdu.scsiahs);
		print_osd_cdb_remove_from_collection_hdr(osd_cdb_remove_from_collection_hdr);
		
		}
		break;	
	
	case OSD_CMD_CREATE:
		{
		OSD_CDB_CREATE_COMMAND_HDR *osd_cdb_create_hdr=(OSD_CDB_CREATE_COMMAND_HDR *)osd_cdb;
		print_msg(dlevel,"OSD_CMD_CREATE");
		decap_osd_cdb_create_hdr(osd_cdb_create_hdr,(u8 *)(kern_usr_meta->ahs_data)+ahs_offset);
		//cmnd->pdu.scsiahs);
		print_osd_cdb_create_hdr(osd_cdb_create_hdr);
		
		}
		break;	
			
	case OSD_CMD_REMOVE_COLLECTION:
		{
		OSD_CDB_REMOVE_COLLECTION_COMMAND_HDR *osd_cdb_remove_collection_hdr=(OSD_CDB_REMOVE_COLLECTION_COMMAND_HDR *)osd_cdb;
		print_msg(dlevel,"OSD_CMD_REMOVE_COLLECTION");
		decap_osd_cdb_remove_collection_hdr(osd_cdb_remove_collection_hdr,(u8 *)(kern_usr_meta->ahs_data)+ahs_offset);
		//cmnd->pdu.scsiahs);
		print_osd_cdb_remove_collection_hdr(osd_cdb_remove_collection_hdr);
		
		}
		break;	

	case OSD_CMD_REMOVE:
		{
		OSD_CDB_REMOVE_COMMAND_HDR *osd_cdb_remove_hdr=(OSD_CDB_REMOVE_COMMAND_HDR *)osd_cdb;
		print_msg(dlevel,"OSD_CMD_REMOVE");
		decap_osd_cdb_remove_hdr(osd_cdb_remove_hdr,(u8 *)(kern_usr_meta->ahs_data)+ahs_offset);
		//cmnd->pdu.scsiahs);
		print_osd_cdb_remove_hdr(osd_cdb_remove_hdr);
		}
	
		break;	

	case OSD_CMD_REMOVE_PARTITION:
		{
		OSD_CDB_REMOVE_PARTITION_COMMAND_HDR *osd_cdb_remove_part_hdr=(OSD_CDB_REMOVE_PARTITION_COMMAND_HDR *)osd_cdb;
		print_msg(dlevel,"OSD_CMD_REMOVE_PARTITION");
		decap_osd_cdb_remove_partition_hdr(osd_cdb_remove_part_hdr,(u8 *)(kern_usr_meta->ahs_data)+ahs_offset);
		//cmnd->pdu.scsiahs);
		print_osd_cdb_remove_partition_hdr(osd_cdb_remove_part_hdr);
		
		}
		break;	

	case OSD_CMD_WRITE:
	case OSD_CMD_READ:
		{
		OSD_CDB_RW_COMMAND_HDR *osd_cdb_rw_hdr;
		osd_cdb_rw_hdr=(OSD_CDB_RW_COMMAND_HDR *)osd_cdb;
		print_msg(dlevel,"OSD_CMD_WRITE/READ");
		decap_osd_cdb_rw_hdr(osd_cdb_rw_hdr,(u8 *)(kern_usr_meta->ahs_data)+ahs_offset);
		//cmnd->pdu.scsiahs);
		print_osd_cdb_rw_hdr(osd_cdb_rw_hdr);
		}
		break;	
	
	case OSD_CMD_LIST:
		{
		OSD_CDB_LIST_COMMAND_HDR *osd_cdb_list_hdr;
		print_msg(dlevel,"OSD_CMD_LIST");
		osd_cdb_list_hdr=(OSD_CDB_LIST_COMMAND_HDR *)osd_cdb;
		decap_osd_cdb_list_command_hdr(osd_cdb_list_hdr,(u8 *)(kern_usr_meta->ahs_data)+ahs_offset);
		//cmnd->pdu.scsiahs);
		print_osd_cdb_list_command_hdr(osd_cdb_list_hdr);
		}
		break;

	case OSD_CMD_LIST_COLLECTION:
		{
		OSD_CDB_LIST_COLLECTION_COMMAND_HDR *osd_cdb_list_coll_hdr;
		print_msg(dlevel,"OSD_CMD_LIST_COLLECTION");
		osd_cdb_list_coll_hdr=(OSD_CDB_LIST_COLLECTION_COMMAND_HDR *)osd_cdb;
		decap_osd_cdb_list_collection_command_hdr(osd_cdb_list_coll_hdr,(u8 *)(kern_usr_meta->ahs_data)+ahs_offset);
		//cmnd->pdu.scsiahs);
		print_osd_cdb_list_collection_command_hdr(osd_cdb_list_coll_hdr);
		}
		break;
		
	case OSD_CMD_APPEND:
		{
		OSD_CDB_APPEND_COMMAND_HDR *osd_cdb_append_hdr;
		print_msg(dlevel,"OSD_CMD_APPEND");
		 osd_cdb_append_hdr=(OSD_CDB_APPEND_COMMAND_HDR *)osd_cdb;
		decap_osd_cdb_append_hdr(osd_cdb_append_hdr,(u8 *)(kern_usr_meta->ahs_data)+ahs_offset);
		//cmnd->pdu.scsiahs);
		print_osd_cdb_append_hdr(osd_cdb_append_hdr);
		}
		break;	
		
	case OSD_CMD_CREATE_AND_WRITE:
		{
		OSD_CDB_CREATE_WRITE_COMMAND_HDR *osd_cdb_create_write_hdr;
		print_msg(dlevel,"OSD_CMD_CREATE_AND_WRITE");
		 osd_cdb_create_write_hdr=(OSD_CDB_CREATE_WRITE_COMMAND_HDR *)osd_cdb;
		decap_osd_cdb_create_write_hdr(osd_cdb_create_write_hdr,(u8 *)(kern_usr_meta->ahs_data)+ahs_offset);
		//cmnd->pdu.scsiahs);
		print_osd_cdb_create_write_hdr(osd_cdb_create_write_hdr);
		}
		break;
	
	case OSD_CMD_SET_ATTR:
        case OSD_CMD_GET_ATTR:
	case OSD_CMD_FLUSH:
        case OSD_CMD_FLUSH_PARTITION:
        case OSD_CMD_FLUSH_OSD:
	case OSD_CMD_FLUSH_COLLECTION:
		{
	        osd_cdb_any_hdr=(OSD_CDB_ANY_COMMAND_HDR *)osd_cdb;
        	print_msg(dlevel,"OSD_CMD_ANY_OTHER unsupported");
                decap_osd_cdb_any_hdr(osd_cdb_any_hdr,(u8 *)(kern_usr_meta->ahs_data)+ahs_offset);//cmnd->pdu.scsiahs);
                print_osd_cdb_any_hdr(osd_cdb_any_hdr);
		}
		break;					
	default:
		osd_cdb_any_hdr=(OSD_CDB_ANY_COMMAND_HDR *)osd_cdb;
		print_msg(dlevel,"OSD_CMD_ANY_OTHER unsupported");
		decap_osd_cdb_any_hdr(osd_cdb_any_hdr,(u8 *)(kern_usr_meta->ahs_data)+ahs_offset);//cmnd->pdu.scsiahs);
		print_osd_cdb_any_hdr(osd_cdb_any_hdr);
		return 1;
	}

	osd_cdb_attr_fmt=osd_cdb_basic_hdr->option_byte2 & 63;
	osd_cdb_attr_fmt =osd_cdb_attr_fmt >> 4;
		
	if(osd_cdb_attr_fmt == OSD_CDB_ATTR_FORMAT_LIST)
	{
		osd_cdb_list_hdr=(OSD_CDB_ATTR_LIST_HDR *)osd_cdb;
		decap_osd_cdb_list_hdr(osd_cdb_list_hdr,(u8 *)(kern_usr_meta->ahs_data)+ahs_offset);
		print_osd_cdb_list_hdr(osd_cdb_list_hdr);
	}	
	
	return 0;	

}
//----------------------------------------------------------------------------------------

void decap_osd_cdb_basic_hdr(OSD_CDB_BASIC_HDR *osd_cdb_basic_hdr,u8 *cmd)
{
	print_msg(dlevel,"\n\nOSD: Decapsulating OSD BASIC CDB \n");
       	osd_cdb_basic_hdr->opcode= be8_to_cpu(*(u8 *)&cmd[0]);//1
        osd_cdb_basic_hdr->control= be8_to_cpu(*(u8 *)&cmd[1]);//1
        //osd_cdb_basic_hdr->resvd[5]; //5
        osd_cdb_basic_hdr->add_len=be8_to_cpu(*(u8 *)&cmd[7]);; //1
        osd_cdb_basic_hdr->service_action=be16_to_cpu(*(u16 *)&cmd[8]);; //2
        osd_cdb_basic_hdr->option_byte1=be8_to_cpu(*(u8 *)&cmd[10]);; //1
        osd_cdb_basic_hdr->option_byte2=be8_to_cpu(*(u8 *)&cmd[11]);;//1
        osd_cdb_basic_hdr->timsetamp=be8_to_cpu(*(u8 *)&cmd[12]);; //1
        //osd_cdb_basic_hdr->resvd1[3];//3

}
//----------------------------------------------------------------------------------------

void print_osd_cdb_basic_hdr(OSD_CDB_BASIC_HDR *osd_cdb_basic_hdr)
{
	print_msg(dlevel,"\n\nOSD: Printing OSD CDB BASIC HDR \n");
	print_msg(dlevel,"OSD: Opcode = 0x%x  Control =0x%x  Add Len=0x%x \nOSD:Service Action=0x%x  Opt Byte1=0x%x Opt Byte2=0x%x  timestamp=0x%x\n",osd_cdb_basic_hdr->opcode,osd_cdb_basic_hdr->control,osd_cdb_basic_hdr->add_len,osd_cdb_basic_hdr->service_action,osd_cdb_basic_hdr->option_byte1,osd_cdb_basic_hdr->option_byte2,osd_cdb_basic_hdr->timsetamp); 
}
//----------------------------------------------------------------------------------------

void decap_osd_cdb_rw_hdr(OSD_CDB_RW_COMMAND_HDR *osd_cdb_rw_hdr,u8 *cmd )
{
 	//u8 resvd[16];
       osd_cdb_rw_hdr->part_id=be64_to_cpu(*((u64 *)&cmd[0]));  //8
       osd_cdb_rw_hdr->object_id=be64_to_cpu(*((u64 *)&cmd[8]));;   //8
        //u8 resevd1[4];  //4
       osd_cdb_rw_hdr->length=be64_to_cpu(*((u64 *)&cmd[20])); //8
       osd_cdb_rw_hdr->starting_byte_offset=be64_to_cpu(*((u64 *)&cmd[28]));; //8
}
//----------------------------------------------------------------------------------------
void print_osd_cdb_rw_hdr(OSD_CDB_RW_COMMAND_HDR *osd_cdb_rw_hdr)
{
	print_msg(dlevel,"\n\nOSD: Printing OSD CDB RW HDR \n");
	print_msg(dlevel,"OSD: Part-id= 0x%llx  Obj-id=0x%llx  Length=0x%llx  StartByteOffset=0x%llx\n",osd_cdb_rw_hdr->part_id, osd_cdb_rw_hdr->object_id,osd_cdb_rw_hdr->length,osd_cdb_rw_hdr->starting_byte_offset);
}
//----------------------------------------------------------------------------------------
void decap_osd_cdb_any_hdr(OSD_CDB_ANY_COMMAND_HDR *osd_cdb_any_hdr,u8 *cmd )
{
 	//u8 resvd[16];
       osd_cdb_any_hdr->part_id = be64_to_cpu(*((u64 *)&cmd[0]));  //8
       osd_cdb_any_hdr->object_id=be64_to_cpu(*((u64 *)&cmd[8]));;   //8
        //u8 resevd1[20];  //20
}
//----------------------------------------------------------------------------------------
void print_osd_cdb_any_hdr(OSD_CDB_ANY_COMMAND_HDR *osd_cdb_any_hdr)
{
	print_msg(dlevel,"\n\nOSD: Printing OSD CDB ANY HDR \n");
	print_msg(dlevel,"OSD: Part-id= 0x%llx  Obj-id=0x%llx \n",osd_cdb_any_hdr->part_id, osd_cdb_any_hdr->object_id);
}
//----------------------------------------------------------------------------------------
void decap_osd_cdb_list_hdr(OSD_CDB_ATTR_LIST_HDR *osd_cdb_list_hdr,u8 *cmd )
{
    //u8 resvd[52];
    osd_cdb_list_hdr->get_list_length=be32_to_cpu(*((u32 *)&cmd[36])) ;  //4
    osd_cdb_list_hdr->get_list_dout_offset=be32_to_cpu(*((u32 *)&cmd[40])); //4
    osd_cdb_list_hdr->get_allocation_length=be32_to_cpu(*((u32 *)&cmd[44]));//4
    osd_cdb_list_hdr->retrieved_attr_din_offset=be32_to_cpu(*((u32 *)&cmd[48]));//4
    osd_cdb_list_hdr->set_list_length=be32_to_cpu(*((u32 *)&cmd[52])); //4
    osd_cdb_list_hdr->set_list_dout_offset=be32_to_cpu(*((u32 *)&cmd[56]));//4
    //u8 reserved1[4]; 
}
//----------------------------------------------------------------------------------------
void print_osd_cdb_list_hdr(OSD_CDB_ATTR_LIST_HDR *osd_cdb_list_hdr)
{
	print_msg(dlevel,"OSD: Printing OSD CDB LIST HDR \n");
	print_msg(dlevel,"OSD: GetListLen=0x%x  GetListDoutOff=0x%x GetAllocLen=0x%x \n OSD: RetriAttrDinOf=0x%x  SetListLen =0x%x  SetListDoutOff=0x%x \n",osd_cdb_list_hdr->get_list_length, osd_cdb_list_hdr->get_list_dout_offset,
	 osd_cdb_list_hdr->get_allocation_length,osd_cdb_list_hdr->retrieved_attr_din_offset,osd_cdb_list_hdr->set_list_length,osd_cdb_list_hdr->set_list_dout_offset);
}
//----------------------------------------------------------------------------------------
void decap_osd_cdb_format_hdr(OSD_CDB_FORMAT_COMMAND_HDR *osd_cdb_format_hdr,u8 *cmd )
{
	osd_cdb_format_hdr->formatted_capacity=be64_to_cpu(*((u64 *)&cmd[20]));  //8
}
//----------------------------------------------------------------------------------------

void print_osd_cdb_format_hdr(OSD_CDB_FORMAT_COMMAND_HDR  *osd_cdb_format_hdr)
{
	print_msg(dlevel,"OSD: Printing OSD CDB FORMAT COMMAND HDR \n");
	print_msg(dlevel,"OSD: Formatted Capacity =0x%llx \n",osd_cdb_format_hdr->formatted_capacity);
}
//----------------------------------------------------------------------------------------

void decap_osd_cdb_create_partition_hdr (OSD_CDB_CREATE_PARTITION_COMMAND_HDR *osd_cdb_create_partition_hdr,u8 *cmd )
{
	osd_cdb_create_partition_hdr->req_part_id=be64_to_cpu(*((u64 *)&cmd[0])); 
}

//----------------------------------------------------------------------------------------

void  print_osd_cdb_create_partition_hdr (OSD_CDB_CREATE_PARTITION_COMMAND_HDR *osd_cdb_create_partition_hdr)
{
	print_msg(dlevel,"OSD: Printing OSD CDB CREATE PARTITION COMMAND HDR \n");
	print_msg(dlevel,"OSD: Request partition ID =0x%llx \n",osd_cdb_create_partition_hdr->req_part_id);
}
//----------------------------------------------------------------------------------------
void decap_osd_cdb_create_hdr (OSD_CDB_CREATE_COMMAND_HDR *osd_cdb_create_hdr,u8 *cmd )
{
    //u8 resvd[16];
    osd_cdb_create_hdr->part_id=be64_to_cpu(*((u64 *)&cmd[0])); //8
    osd_cdb_create_hdr->req_obj_id=be64_to_cpu(*((u64 *)&cmd[8])); //8
    //u8 resvd1[4];//4
    osd_cdb_create_hdr->no_of_objects=be16_to_cpu(*((u16 *)&cmd[20]));
    //u8 rsvd2[14];
}
//----------------------------------------------------------------------------------------
void  print_osd_cdb_create_hdr (OSD_CDB_CREATE_COMMAND_HDR *osd_cdb_create_hdr)
{
	print_msg(dlevel,"OSD: Printing OSD CDB CREATE COMMAND HDR \n");
	print_msg(dlevel,"OSD: Partition ID =0x%llx Req Obj ID= 0x%llx No Of Objects=0x%x \n",osd_cdb_create_hdr->part_id,osd_cdb_create_hdr->req_obj_id,osd_cdb_create_hdr->no_of_objects);
}
//----------------------------------------------------------------------------------------
void decap_osd_cdb_create_collection_hdr (OSD_CDB_CREATE_COLLECTION_COMMAND_HDR *osd_cdb_create_collection_hdr,u8 *cmd )
{
  //u8 resvd[16];
	osd_cdb_create_collection_hdr->part_id=be64_to_cpu(*((u64 *)&cmd[0])); //8
	osd_cdb_create_collection_hdr->req_coll_obj_id=be64_to_cpu(*((u64 *)&cmd[8])); //8 
    //u8 resvd1[20];
}


//----------------------------------------------------------------------------------------
void  print_osd_cdb_create_collection_hdr (OSD_CDB_CREATE_COLLECTION_COMMAND_HDR *osd_cdb_create_collection_hdr)
{
	print_msg(dlevel,"OSD: Printing OSD CDB REMOVE COLLECTION COMMAND HDR \n");
	print_msg(dlevel,"OSD: Partition ID =0x%llx Req Collection Obj ID= 0x%llx \n",osd_cdb_create_collection_hdr->part_id,osd_cdb_create_collection_hdr->req_coll_obj_id);
}
//----------------------------------------------------------------------------------------
void decap_osd_cdb_add_to_collection_hdr (OSD_CDB_ADD_TO_COLLECTION_COMMAND_HDR *osd_cdb_add_to_coll_hdr,u8 *cmd )
{
       osd_cdb_add_to_coll_hdr->part_id=be64_to_cpu(*((u64 *)&cmd[0]));  //8
       osd_cdb_add_to_coll_hdr->coll_obj_id=be64_to_cpu(*((u64 *)&cmd[8]));;   //8
        //u8 resevd1[4];  //4
       osd_cdb_add_to_coll_hdr->obj_id=be64_to_cpu(*((u64 *)&cmd[20])); //8
}
//----------------------------------------------------------------------------------------
void  print_osd_cdb_add_to_collection_hdr (OSD_CDB_ADD_TO_COLLECTION_COMMAND_HDR *osd_cdb_add_to_coll_hdr)
{
	print_msg(dlevel,"OSD: Printing OSD CDB ADD TO COLLECTION COMMAND HDR \n");
	print_msg(dlevel,"OSD: Request partition ID =0x%llx Collection id=0x%llx obj-id=0x%llx \n",osd_cdb_add_to_coll_hdr->part_id,osd_cdb_add_to_coll_hdr->coll_obj_id,osd_cdb_add_to_coll_hdr->obj_id);
}
//----------------------------------------------------------------------------------------
void decap_osd_cdb_remove_from_collection_hdr(OSD_CDB_REMOVE_FROM_COLLECTION_COMMAND_HDR *osd_cdb_remove_from_coll_hdr,u8 *cmd )
{
       osd_cdb_remove_from_coll_hdr->part_id=be64_to_cpu(*((u64 *)&cmd[0]));  //8
       osd_cdb_remove_from_coll_hdr->coll_obj_id=be64_to_cpu(*((u64 *)&cmd[8]));;   //8
        //u8 resevd1[4];  //4
       osd_cdb_remove_from_coll_hdr->obj_id=be64_to_cpu(*((u64 *)&cmd[20])); //8
}
//----------------------------------------------------------------------------------------
void  print_osd_cdb_remove_from_collection_hdr (OSD_CDB_REMOVE_FROM_COLLECTION_COMMAND_HDR *osd_cdb_remove_from_coll_hdr)
{
	print_msg(dlevel,"OSD: Printing OSD CDB REMOVE FROM COLLECTION COMMAND HDR \n");
	print_msg(dlevel,"OSD: Request partition ID =0x%llx Collection id=0x%llx obj-id=0x%llx \n",osd_cdb_remove_from_coll_hdr->part_id,osd_cdb_remove_from_coll_hdr->coll_obj_id,osd_cdb_remove_from_coll_hdr->obj_id);
}

//----------------------------------------------------------------------------------------
void decap_osd_cdb_remove_partition_hdr (OSD_CDB_REMOVE_PARTITION_COMMAND_HDR *osd_cdb_remove_partition_hdr,u8 *cmd )
{
   osd_cdb_remove_partition_hdr->rem_part_id=be64_to_cpu(*((u64 *)&cmd[0])); //8
}
//----------------------------------------------------------------------------------------
void  print_osd_cdb_remove_partition_hdr (OSD_CDB_REMOVE_PARTITION_COMMAND_HDR *osd_cdb_remove_partition_hdr)
{
	print_msg(dlevel,"OSD: Printing OSD CDB REMOVE PARTITION COMMAND HDR \n");
	print_msg(dlevel,"OSD: Request remove partition ID =0x%llx \n",osd_cdb_remove_partition_hdr->rem_part_id);
}
//----------------------------------------------------------------------------------------
void decap_osd_cdb_remove_hdr (OSD_CDB_REMOVE_COMMAND_HDR *osd_cdb_remove_hdr,u8 *cmd )
{
    //u8 resvd[16];
    osd_cdb_remove_hdr->part_id=be64_to_cpu(*((u64 *)&cmd[0])); //8
    osd_cdb_remove_hdr->rem_obj_id=be64_to_cpu(*((u64 *)&cmd[8])); //8
}
//----------------------------------------------------------------------------------------
void  print_osd_cdb_remove_hdr (OSD_CDB_REMOVE_COMMAND_HDR *osd_cdb_remove_hdr)
{
	print_msg(dlevel,"OSD: Printing OSD CDB REMOVE COMMAND HDR \n");
	print_msg(dlevel,"OSD: Partition ID =0x%llx Rem Obj ID= 0x%llx  \n",osd_cdb_remove_hdr->part_id,osd_cdb_remove_hdr->rem_obj_id);
}
//----------------------------------------------------------------------------------------
void decap_osd_cdb_remove_collection_hdr (OSD_CDB_REMOVE_COLLECTION_COMMAND_HDR *osd_cdb_remove_collection_hdr,u8 *cmd )
{
    osd_cdb_remove_collection_hdr->part_id=be64_to_cpu(*((u64 *)&cmd[0])); //8
    osd_cdb_remove_collection_hdr->rem_coll_obj_id=be64_to_cpu(*((u64 *)&cmd[8])); //8
}
//----------------------------------------------------------------------------------------
void  print_osd_cdb_remove_collection_hdr (OSD_CDB_REMOVE_COLLECTION_COMMAND_HDR *osd_cdb_remove_collection_hdr)
{
	print_msg(dlevel,"OSD: Printing OSD CDB REMOVE COLLECTION COMMAND HDR \n");
	print_msg(dlevel,"OSD: Partition ID =0x%llx Rem Coll Obj ID= 0x%llx  \n",osd_cdb_remove_collection_hdr->part_id,osd_cdb_remove_collection_hdr->rem_coll_obj_id);
}

void decap_osd_cdb_list_command_hdr (OSD_CDB_LIST_COMMAND_HDR *osd_cdb_list_hdr,u8 *cmd )
{
	//u8 resvd[16];
	osd_cdb_list_hdr->part_id=be64_to_cpu(*((u64 *)&cmd[0])); //8
	//u8  resvd1[8];//8
	osd_cdb_list_hdr->list_identifier=be32_to_cpu(*((u32 *)&cmd[16]));//4
	osd_cdb_list_hdr->allocation_length=be64_to_cpu(*((u64 *)&cmd[20]));//8
	osd_cdb_list_hdr->initial_obj_id=be64_to_cpu(*((u64 *)&cmd[28]));//8
}
//--------------------------------------------------------------------------------
void  print_osd_cdb_list_command_hdr (OSD_CDB_LIST_COMMAND_HDR *osd_cdb_list_hdr)
{
	print_msg(dlevel,"OSD: Printing OSD CDB LIST COMMAND HDR \n");
	print_msg(dlevel,"OSD: Partition ID =0x%llx list id =0x%lx Allocation len=0x%llx Initial Obj ID= 0x%llx ",osd_cdb_list_hdr->part_id,osd_cdb_list_hdr->list_identifier,osd_cdb_list_hdr->allocation_length,osd_cdb_list_hdr->initial_obj_id); 
}
//---------------------------------------------------------------------------------
void decap_osd_cdb_list_collection_command_hdr (OSD_CDB_LIST_COLLECTION_COMMAND_HDR *osd_cdb_list_coll_hdr,u8 *cmd )
{
	//u8 resvd[16];
	osd_cdb_list_coll_hdr->part_id=be64_to_cpu(*((u64 *)&cmd[0])); //8
	osd_cdb_list_coll_hdr->coll_id=be64_to_cpu(*((u64 *)&cmd[8])); //8
	osd_cdb_list_coll_hdr->list_identifier=be32_to_cpu(*((u32 *)&cmd[16]));//4
	osd_cdb_list_coll_hdr->allocation_length=be64_to_cpu(*((u64 *)&cmd[20]));//8
	osd_cdb_list_coll_hdr->initial_obj_id=be64_to_cpu(*((u64 *)&cmd[28]));//8
}
//---------------------------------------------------------------------------------
void  print_osd_cdb_list_collection_command_hdr (OSD_CDB_LIST_COLLECTION_COMMAND_HDR *osd_cdb_list_coll_hdr)
{
	print_msg(dlevel,"OSD: Printing OSD CDB LIST COLLECTION COMMAND HDR \n");
	print_msg(dlevel,"OSD: Partition ID =0x%llx Collection ID=0x%llx  List id =0x%lx Allocation len=0x%llx Initial Obj ID= 0x%llx ",osd_cdb_list_coll_hdr->part_id,osd_cdb_list_coll_hdr->coll_id,osd_cdb_list_coll_hdr->list_identifier,osd_cdb_list_coll_hdr->allocation_length,osd_cdb_list_coll_hdr->initial_obj_id); 
}
//---------------------------------------------------------------------------------
void decap_osd_cdb_append_hdr(OSD_CDB_APPEND_COMMAND_HDR *osd_cdb_append_hdr,u8 *cmd )
{
       osd_cdb_append_hdr->part_id=be64_to_cpu(*((u64 *)&cmd[0]));  //8
       osd_cdb_append_hdr->object_id=be64_to_cpu(*((u64 *)&cmd[8]));;   //8
        //u8 resevd1[4];  //4
       osd_cdb_append_hdr->length=be64_to_cpu(*((u64 *)&cmd[20])); //8
}
//---------------------------------------------------------------------------------
void print_osd_cdb_append_hdr(OSD_CDB_APPEND_COMMAND_HDR *osd_cdb_append_hdr)
{
	print_msg(dlevel,"\n\nOSD: Printing OSD CDB APPEND HDR \n");
	print_msg(dlevel,"OSD: Part-id= 0x%llx  Obj-id=0x%llx  Length=0x%llx  \n",osd_cdb_append_hdr->part_id, osd_cdb_append_hdr->object_id,osd_cdb_append_hdr->length);
}
//---------------------------------------------------------------------------------

void decap_osd_cdb_create_write_hdr(OSD_CDB_CREATE_WRITE_COMMAND_HDR *osd_cdb_create_write_hdr,u8 *cmd )
{
	//u8 resvd[16];
       osd_cdb_create_write_hdr->part_id=be64_to_cpu(*((u64 *)&cmd[0]));  //8
       osd_cdb_create_write_hdr->req_obj_id=be64_to_cpu(*((u64 *)&cmd[8]));;   //8
        //u8 resevd1[4];  //4
       osd_cdb_create_write_hdr->length=be64_to_cpu(*((u64 *)&cmd[20])); //8
       osd_cdb_create_write_hdr->starting_byte_offset=be64_to_cpu(*((u64 *)&cmd[28]));; 
}
//---------------------------------------------------------------------------

void print_osd_cdb_create_write_hdr(OSD_CDB_CREATE_WRITE_COMMAND_HDR *osd_cdb_create_write_hdr)
{
	print_msg(dlevel,"\n\nOSD: Printing OSD CDB CREATE AND WRITE  HDR \n");
	print_msg(dlevel,"OSD: Part-id= 0x%llx  Obj-id=0x%llx  Length=0x%llx  StartByteOffset=0x%llx\n",osd_cdb_create_write_hdr->part_id, osd_cdb_create_write_hdr->req_obj_id,osd_cdb_create_write_hdr->length,osd_cdb_create_write_hdr->starting_byte_offset);
}
//---------------------------------------------------------------------------
