/**
 *                      ==================
 *                      IITK OSD Simulator
 *                      ==================
 *   An Implementation of Simulator for Object based storage device(OSD)
 *
 *   Copyright (C) 2007-2008 CSE IIT KANPUR
 *
 *   Contact: Department of Computer Science and Engineering
 *            Indian Institute of Technology,Kanpur(IIT Kanpur)
 *            {moona,riyaz}@cse.iitk.ac.in
 *
 *   This file is part of IITK OSD Simulator Project.
 *
 *   IITK OSD Simulator Project is free software; you can redistribute it
 *   and/or modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   IITK OSD Simulator Project is distributed in the hope that it will be
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the  GNU General
 *   Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/*************************************************************
** Copyright 2004, 2005. IBM Corp.** All Rights Reserved.***
**************************************************************
** Redistribution and use in source and binary forms, with or without
** modifications, are permitted provided that the following conditions 
** are met:
** 
** i.  Redistribution of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**
** ii. Redistribution in binary form must reproduce the above copyright 
**     notice, this list of conditions and the following disclaimer in the 
**     documentation and/or other materials provided with the distribution.
**
** iii.Neither the name(s) of IBM Corp.** nor the names of its/their
**     contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
*******************************************************************
** THIS SOFTWARE IS PROVIDED BY IBM CORP. AND CONTRIBUTORS "AS IS" 
** AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING , BUT NOT LIMITED
** TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
** PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE IBM
** CORP. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
** INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
** OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
** HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
** STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
** OF THE POSSIBILITY OF SUCH DAMAGE.
** 
*****************************************************************************
*****************************************************************************/

/**************
 * osd.h
 * definitions used by all osd components
 * Author: Allon Shafrir
 *******************************************************************/
#ifndef OSD_H
#define OSD_H

/******************************************************************
 * includes
 */

#ifdef __KERNEL__
#include <linux/types.h>
#else
#include <stdint.h>
# include "osd_types.h"
# define u8 uint8_t 
# define u16 uint16_t 
# define u32 uint32_t 
# define u64 uint64_t 
# define u48 uint48_t
# define PAGE_CACHE_SIZE 4096

#ifndef _BIG_ENDIAN
#include <byteswap.h>
	# define be16_to_cpu(src)   bswap_16(src)
	# define be32_to_cpu(src)   bswap_32(src)
	# define be64_to_cpu(src)   bswap_64(src)
	# define cpu_to_be16(src)   bswap_16(src)
	# define cpu_to_be32(src)   bswap_32(src)
	# define cpu_to_be64(src)   bswap_64(src)
#else
	# define be16_to_cpu(src)   (src)
	# define be32_to_cpu(src)   (src)
	# define be64_to_cpu(src)   (src)
	# define cpu_to_be16(src)   (src)
	# define cpu_to_be32(src)   (src)
	# define cpu_to_be64(src)   (src)
#endif /* ifndef __BIG_ENDIAN */

#endif /* ifdef _KERNEL_*/

# include "osd_socket.h"
/******************************************************************
 * #defines
 */

#define OSD_CAPABILITY_SIZE 80
//size of cryptographic values used (keys, key-seeds, integrity check values)
#define OSD_CRYPTO_VAL_SIZE 20

#define OSD_CREDENTIAL_SIZE 120

/*
//modifier for packed structs in gcc. used in define because w/o it confuses tags
#define OSD_PACKED __attribute__((packed))
#define OSD_PACKED_STRUCT_END //nothing is needed
*/
//service actions - according to table1 48 in section 6 of OSD standard rev 10
#define OSD_CMD_CREATE            0x8802
#define OSD_CMD_CREATE_PARTITION  0x880B
#define OSD_CMD_GET_ATTR          0x880E
#define OSD_CMD_LIST              0x8803
#define OSD_CMD_REMOVE            0x880A
#define OSD_CMD_REMOVE_PARTITION  0x880C
#define OSD_CMD_READ              0x8805
#define OSD_CMD_SET_ATTR          0x880F
#define OSD_CMD_WRITE             0x8806
#define OSD_CMD_FORMAT            0x8801
#define OSD_CMD_SET_KEY           0x8818
#define OSD_CMD_INQUIRY           0x1200
#define OSD_CMD_APPEND		  0x8807	
//non standard service action for debugging - for internal use only
#define OSD_CMD_DEBUG             0x8F80 

//non standard service action for CLEAR command as agreed
#define OSD_CMD_CLEAR             0x8FA0

//unsupported commands
#define OSD_CMD_FLUSH             0x8808
#define OSD_CMD_FLUSH_PARTITION   0x881B
#define OSD_CMD_FLUSH_OSD         0x881C
#define OSD_CMD_APPEND            0x8807
#define OSD_CMD_CREATE_AND_WRITE  0x8812
#define OSD_CMD_CREATE_COLLECTION 0x8815
#define OSD_CMD_REMOVE_COLLECTION 0x8816
#define OSD_CMD_FLUSH_COLLECTION  0x881A
#define OSD_CMD_LIST_COLLECTION   0x8817
#define OSD_CMD_PERFORM_SCSI_CMD  0x8F7E
#define OSD_CMD_PERFORM_TSK_MNGN  0x8F7F
#define OSD_CMD_SET_MASTER_KEY    0x8819

//"virtual" service action codes only used internally by initiator
#define OSD_CMD_INQUIRY_83        0x1283
#define OSD_CMD_INQUIRY_B0        0x12B0
#define OSD_CMD_INQUIRY_B1        0x12B1

# define OSD_CMD_ADD_TO_COLLECTION 0x8820
# define OSD_CMD_REMOVE_FROM_COLLECTION 0x8821

# define OSD_CDB_ATTR_FORMAT_LIST 3
# define OSD_CDB_ATTR_FORMAT_PAGE 2

# define SOCK_ERR 255

/** Strcture to define OSD SCSI CDB of 16 bytes
*/
typedef struct 
{
        u8 opcode; //1
        u8 control; //1
        u8 resvd[5]; //5
        u8 add_len; //1
        u16 service_action; //2
        u8 option_byte1; //1
        u8 option_byte2;//1
        u8 timsetamp; //1
        u8 resvd1[3];//3
}OSD_CDB_BASIC_HDR ; //scsi cdb 16 bytes

/**
 *
 *    Function:  decap_osd_cdb_basic_hdr
 *		 print_osd_cdb_basic_hdr
 *
 *    Description: Functions decapsulate and prints OSD Basic CDB
 *
 *    Argument:  @osd_cdb_basic_hdr - store decapsulated CDB
 *		 @cmd - OSD CBD from network
 * 
 *    Return codes: nothing  
 *
 **/
void decap_osd_cdb_basic_hdr(OSD_CDB_BASIC_HDR *osd_cdb_basic_hdr,u8 *cmd);
void print_osd_cdb_basic_hdr(OSD_CDB_BASIC_HDR *osd_cdb_basic_hdr);



/** Structure for OSD CDB get-attribute List according to OSD-2 T10 draft section 5.2.2.3 
*/
typedef struct 
{
    u8 resvd[52];
    u32 get_list_length;
    u32 get_list_dout_offset;
    u32 get_allocation_length;
    u32 retrieved_attr_din_offset;
    u32 set_list_length;
    u32 set_list_dout_offset;
    u8 reserved1[4];
} OSD_CDB_ATTR_LIST_HDR;


/**
 *
 *    Functions:  decap_osd_cdb_list_hdr
 *		 print_osd_cdb_list_hdr
 *
 *    Description: Functions decapsulates and print OSD GET ATTRIBUTE LIST CDB 
 *
 *    Arguments:  @osd_cdb_list_hdr - store decapsulated GET ATTR LIST CDB
 *		 @cmd - OSD CBD from network
 * 
 *    Return code: nothing
 *
 **/

void decap_osd_cdb_list_hdr(OSD_CDB_ATTR_LIST_HDR *osd_cdb_list_hdr,u8 *cmd );
void print_osd_cdb_list_hdr(OSD_CDB_ATTR_LIST_HDR *osd_cdb_list_hdr);


/** Structure for OSD CDB get-attribute Page according to OSD-2 T10 draft section 5.2.2.2 
*/
typedef struct 
{
        u8 resvd[52];
        u32 get_page_no;
        u32 get_alloc_length;
        u32 retrieved_attr_din_offset;
        u32 set_page_no;
        u32 set_attr_no;
        u32 set_attr_length;
        u32  set_attr_dout_offset;


}OSD_CDB_ATTR_PAGE_HDR;


/** Structure for OSD CDB security parameters according to OSD-2 T10 draft section 5.2.6
*/

typedef struct 
{
        u8 resvd[80];
        u8 capability[80];
        u8 req_int_check_value;
        u8 req_nonce[12];
        u32 din_int_chk_ofs;
        u32 dout_int_chk_ofs;
}OSD_CDB_SECURITY_HDR;


/**
 Structure of  OSD CDB for OSD_FORMAT command according to OSD-2 T10 draft section 6.11
*/ 
typedef struct {
    u8 resvd[16];
    u8 resvd1[20];
    u64 formatted_capacity;
    u8 rsvd2[8];
 }OSD_CDB_FORMAT_COMMAND_HDR;

/**
 *
 *    Function:  decap_osd_cdb_format_hdr
 *		 print_osd_cdb_format_hdr
 *
 *    Description: Functions decapsulate and prints OSD FORMAT CDB
 *
 *    Argument:  @osd_cdb_format_hdr - store decapsulated CDB
 *		 @cmd - OSD CBD from network
 * 
 *    Return codes: nothing
 *
 **/

void decap_osd_cdb_format_hdr(OSD_CDB_FORMAT_COMMAND_HDR  *osd_cdb_format_hdr,u8 *cmd );
void print_osd_cdb_format_hdr(OSD_CDB_FORMAT_COMMAND_HDR  *osd_cdb_format_hdr);


/**
 Structure of  OSD CDB for CREATE_PARTITION command according to OSD-2 T10 draft section
 6.6
*/ 
 
typedef struct {
    u8 rsvd[16];
    u64 req_part_id;
    u8 rsvd1[28];
  
} OSD_CDB_CREATE_PARTITION_COMMAND_HDR;

/**
 *
 *    Function:  decap_osd_cdb_create_partition_hdr
 *		 print_osd_cdb_create_partition_hdr
 *
 *    Description: Functions decapsulate and prints OSD CREATE PART CDB
 *
 *    Argument:  @osd_cdb_create_partition_hdr - store decapsulated CDB
 *		 @cmd - OSD CBD from network
 * 
 *    Return codes: nothing
 *
 **/

void decap_osd_cdb_create_partition_hdr (OSD_CDB_CREATE_PARTITION_COMMAND_HDR *osd_cdb_create_partition_hdr,u8 *cmd );
void  print_osd_cdb_create_partition_hdr (OSD_CDB_CREATE_PARTITION_COMMAND_HDR *osd_cdb_create_partition_hdr);

/**
 Structure of  OSD CDB for CREATE USER OBJECT command according to OSD-2 T10 draft section
 6.3
*/ 

typedef struct {
    u8 resvd[16];
    u64 part_id;
    u64 req_obj_id;
    u8 resvd1[4];
    u16 no_of_objects;
    u8 rsvd2[14];
}OSD_CDB_CREATE_COMMAND_HDR;

/**
 *
 *    Function:  decap_osd_cdb_create_hdr
 *		 print_osd_cdb_create_hdr
 *
 *    Description: Functions decapsulate and prints OSD CREATE CDB
 *
 *    Argument:  @osd_cdb_create_hdr - store decapsulated CDB
 *		 @cmd - OSD CBD from network
 * 
 *    Return code: nothing
 *
 **/

void decap_osd_cdb_create_hdr (OSD_CDB_CREATE_COMMAND_HDR *osd_cdb_create_hdr,u8 *cmd );
void  print_osd_cdb_create_hdr (OSD_CDB_CREATE_COMMAND_HDR *osd_cdb_create_hdr);

/**
 Structure of  OSD CDB for CREATE COLLECTION command according to OSD-2 T10 draft section
 6.5
*/ 

typedef struct {
    u8 resvd[16];
    u64 part_id;
    u64 req_coll_obj_id;
    u8 resvd1[20];
}OSD_CDB_CREATE_COLLECTION_COMMAND_HDR;

/**
 *
 *    Function:  decap_osd_cdb_create_collection_hdr
 *		 print_osd_cdb_create_collection_hdr
 *
 *    Description: Functions decapsulate and prints OSD CREATE COLLECTION CDB
 *
 *    Argument:  @osd_cdb_create_collection_hdr - store decapsulated CDB
 *		 @cmd - OSD CBD from network
 * 
 *    Return codes: nothing
 *
 **/

void decap_osd_cdb_create_collection_hdr (OSD_CDB_CREATE_COLLECTION_COMMAND_HDR *osd_cdb_create_coll_hdr,u8 *cmd );
void  print_osd_cdb_create_collection_hdr (OSD_CDB_CREATE_COLLECTION_COMMAND_HDR *osd_cdb_create_coll_hdr);


/**
 Structure of  OSD CDB for ADD TO COLELCTION command (command currently not T10 compilant)
*/ 

typedef struct {
    u8 resvd[16];
    u64 part_id;
    u64 coll_obj_id;
    u8 rsvd1[4];
    u64 obj_id;
    u8 rsvd2[8];
} OSD_CDB_ADD_TO_COLLECTION_COMMAND_HDR;

/**
 *
 *    Function:  decap_osd_cdb_add_to_collection_hdr
 *		 print_osd_cdb_add_to_collection_hdr
 *
 *    Description: Functions decapsulate and prints OSD ADD TO COLLECTION CDB
 *
 *    Argument:  @osd_cdb_create_add_to_hdr - store decapsulated CDB
 *		 @cmd - OSD CBD from network
 * 
 *    Return codes: nothing
 *
 **/

void decap_osd_cdb_add_to_collection_hdr(OSD_CDB_ADD_TO_COLLECTION_COMMAND_HDR *osd_cdb_add_to_coll_hdr,u8 *cmd );
void  print_osd_cdb_add_to_collection_hdr (OSD_CDB_ADD_TO_COLLECTION_COMMAND_HDR *osd_cdb_add_to_coll_hdr);


/**
 Structure of  OSD CDB for REMOVE FROM COLELCTION command (command currently not T10 compilant)
*/ 
typedef struct {
    u8 resvd[16];
    u64 part_id;
    u64 coll_obj_id;
    u8 rsvd1[4];
    u64 obj_id;
    u8 rsvd2[8];
} OSD_CDB_REMOVE_FROM_COLLECTION_COMMAND_HDR;

/**
 *
 *    Function:  decap_osd_cdb_remove_from_collection_hdr
 *		 print_osd_cdb_remove_from_collection_hdr
 *
 *    Description: Functions decapsulate and prints OSD REMOVE FROM COLLECTION 
 * 		   CDB
 *
 *    Argument:  @osd_cdb_remove_from_coll_hdr - store decapsulated CDB
 *		 @cmd - OSD CBD from network
 * 
 *    Return codes: nothing
 *
 **/
void decap_osd_cdb_remove_from_collection_hdr(OSD_CDB_REMOVE_FROM_COLLECTION_COMMAND_HDR *osd_cdb_remove_from_coll_hdr,u8 *cmd );
void  print_osd_cdb_remove_from_collection_hdr (OSD_CDB_REMOVE_FROM_COLLECTION_COMMAND_HDR *osd_cdb_remove_from_coll_hdr);

/**
 Structure of  OSD CDB for CREATE_N_WR  command according to OSD-2 T10 draft section
 6.4
*/ 

typedef struct 
{
        u8 resvd[16];
        u64 part_id;
        u64 req_obj_id;
        u8 resevd1[4];
        u64 length;
        u64 starting_byte_offset;
}OSD_CDB_CREATE_WRITE_COMMAND_HDR;

/**
 *
 *    Function:  decap_osd_cdb_create_write_hdr
 *		 print_osd_cdb_create_write_hdr
 *
 *    Description: Functions decapsulate and prints OSD CREATE N WR CDB
 *
 *    Argument:  @osd_cdb_create_write_hdr - store decapsulated CDB
 *		 @cmd - OSD CBD from network
 * 
 *    Return codes: nothing  
 *
 **/
void decap_osd_cdb_create_write_hdr(OSD_CDB_CREATE_WRITE_COMMAND_HDR *osd_cdb_create_write_hdr,u8 *cmd );
void print_osd_cdb_create_write_hdr(OSD_CDB_CREATE_WRITE_COMMAND_HDR *osd_cdb_create_write_hdr);

/**
 Structure of  OSD CDB for REMOVE OBJECT command according to OSD-2 T10 draft section
 6.18
*/ 
typedef struct {
    u8 resvd[16];
    u64 part_id;
    u64 rem_obj_id;
    u8 resvd1[20];
}OSD_CDB_REMOVE_COMMAND_HDR;

/**
 *
 *    Function:  decap_osd_cdb_remove_hdr
 *		 print_osd_cdb_remove_hdr
 *
 *    Description: Functions decapsulate and prints OSD REMOVE CDB
 *
 *    Argument:  @osd_cdb_remove_hdr - store decapsulated CDB
 *		 @cmd - OSD CBD from network
 * 
 *    Return codes: nothing
 *
 **/

void decap_osd_cdb_remove_hdr (OSD_CDB_REMOVE_COMMAND_HDR *osd_cdb_remove_hdr,u8 *cmd );
void  print_osd_cdb_remove_hdr (OSD_CDB_REMOVE_COMMAND_HDR *osd_cdb_remove_hdr);

/**
 Structure of  OSD CDB for REMOVE COLLECTION command according to OSD-2 T10 draft section
 6.19
*/ 
typedef struct {
    u8 resvd[16];
    u64 part_id;
    u64 rem_coll_obj_id;
    u8 resvd1[20];
}OSD_CDB_REMOVE_COLLECTION_COMMAND_HDR;

/**
 *
 *    Function:  decap_osd_cdb_remove_collection_hdr
 *		 print_osd_cdb_remove_collection_hdr
 *
 *    Description: Functions decapsulate and prints OSD REMOVE COLLECTION CDB
 *
 *    Argument:  @osd_cdb_remove_collection_hdr - store decapsulated CDB
 *		 @cmd - OSD CBD from network
 * 
 *    Return codes: nothing
 *
 **/
void decap_osd_cdb_remove_collection_hdr (OSD_CDB_REMOVE_COLLECTION_COMMAND_HDR *osd_cdb_remove_collection_hdr,u8 *cmd );
void  print_osd_cdb_remove_collection_hdr (OSD_CDB_REMOVE_COLLECTION_COMMAND_HDR *osd_cdb_remove_collection_hdr);


/**
 Structure of  OSD CDB for REMOVE PARTITION command according to OSD-2 T10 draft section
 6.20
*/ 
typedef struct {
    u8 resvd[16];
    u64 rem_part_id;
    u8 resvd1[28];
}OSD_CDB_REMOVE_PARTITION_COMMAND_HDR;

/**
 *
 *    Function:  decap_osd_cdb_remove_partition_hdr
 *		 print_osd_cdb_remove_partition_hdr
 *
 *    Description: Functions decapsulate and prints OSD REMOVE PART CDB
 *
 *    Argument:  @osd_cdb_remove_partition_hdr - store decapsulated CDB
 *		 @cmd - OSD CBD from network
 * 
 *    Return codes: nothing
 *
 **/
void decap_osd_cdb_remove_partition_hdr (OSD_CDB_REMOVE_PARTITION_COMMAND_HDR *osd_cdb_remove_partition_hdr,u8 *cmd );
void  print_osd_cdb_remove_partition_hdr (OSD_CDB_REMOVE_PARTITION_COMMAND_HDR *osd_cdb_remove_partition_hdr);


/**
 Structure of  OSD CDB for READ/WRITE User object command according to OSD-2 T10 draft section 6.17 and 6.24
*/ 


typedef struct 
{
        u8 resvd[16];
        u64 part_id;
        u64 object_id;
        u8 resevd1[4];
        u64 length;
        u64 starting_byte_offset;
}OSD_CDB_RW_COMMAND_HDR;


/**
 *
 *    Functions:  decap_osd_cdb_rw_hdr
 *		 print_osd_cdb_rw_hdr
 *
 *    Description: Functions decapsulates and print OSD RW CDB and 
 *
 *    Argument:  @osd_cdb_rw_hdr - store decapsulated RW CDB
 *		 @cmd - OSD CBD from network
 * 
 *    Return codes: nothing  
 *
 **/
void decap_osd_cdb_rw_hdr(OSD_CDB_RW_COMMAND_HDR *osd_cdb_rw_hdr,u8 *cmd );
void print_osd_cdb_rw_hdr(OSD_CDB_RW_COMMAND_HDR *osd_cdb_rw_hdr);


/**
 Structure of  OSD CDB for APPEND User object command according to OSD-2 T10 draft section 6.2
*/ 

typedef struct 
{
        u8 resvd[16];
        u64 part_id;
        u64 object_id;
        u8 resevd1[4];
        u64 length;
        u64 resevd2;
}OSD_CDB_APPEND_COMMAND_HDR;

/**
 *
 *    Functions:  decap_osd_cdb_append_hdr
 *		 print_osd_cdb_append_hdr
 *
 *    Description: Functions decapsulates and print OSD APPEND CDB and 
 *
 *    Argument:  @osd_cdb_append_hdr - store decapsulated APPNED CDB
 *		 @cmd - OSD CBD from network
 * 
 *    Return codes: 0 if success,1 otherwise  
 *
 **/


void decap_osd_cdb_append_hdr(OSD_CDB_APPEND_COMMAND_HDR *osd_cdb_append_hdr,u8 *cmd );
void print_osd_cdb_append_hdr(OSD_CDB_APPEND_COMMAND_HDR *osd_cdb_append_hdr);

/**
 Structure of  OSD CDB for List command according to OSD-2 T10 draft section 6.13
*/ 

typedef struct {
    u8 resvd[16];
    u64 part_id;
    u8  resvd1[8];
    u32 list_identifier;
    u64 allocation_length;
    u64 initial_obj_id;
}OSD_CDB_LIST_COMMAND_HDR;


/**
 *
 *    Functions:  decap_osd_cdb_list_hdr
 *		 print_osd_cdb_list_hdr
 *
 *    Description: Functions decapsulates and print OSD LIST CDB 
 *
 *    Argument:  @osd_cdb_list_hdr - store decapsulated CDB
 *		 @cmd - OSD CBD from network
 * 
 *    Return codes: 0 if success,1 otherwise  
 *
 **/

void decap_osd_cdb_list_command_hdr (OSD_CDB_LIST_COMMAND_HDR *osd_cdb_list_hdr,u8 *cmd );
void  print_osd_cdb_list_command_hdr (OSD_CDB_LIST_COMMAND_HDR *osd_cdb_list_hdr);


/**
 Structure of  OSD CDB for List Collection command according to OSD-2 T10 draft section 6.14
*/ 

typedef struct {
    u8 resvd[16];
    u64 part_id;
    u64 coll_id;
    u32 list_identifier;
    u64 allocation_length;
    u64 initial_obj_id;
}OSD_CDB_LIST_COLLECTION_COMMAND_HDR;


/**
 *
 *    Functions:  decap_osd_cdb_list_collection_hdr
 *		 print_osd_cdb_list_collection_hdr
 *
 *    Description: Functions decapsulates and print OSD LIST COLLECTION CDB 
 *
 *    Argument:  @osd_cdb_list_coll_hdr - store decapsulated CDB
 *		 @cmd - OSD CBD from network
 * 
 *    Return codes: 0 if success,1 otherwise  
 *
 **/
void decap_osd_cdb_list_collection_command_hdr (OSD_CDB_LIST_COLLECTION_COMMAND_HDR *osd_cdb_list_coll_hdr,u8 *cmd );
void  print_osd_cdb_list_collection_command_hdr (OSD_CDB_LIST_COLLECTION_COMMAND_HDR *osd_cdb_list_coll_hdr);

/**
 Structure of  OSD CDB for commands other than READ/WRITE User object command according to OSD-2 T10 draft section 5.2.1
*/ 

typedef struct 
{
	u8 resvd[16];
	u64 part_id;
        u64 object_id;
        u8 resvd1[20];
}OSD_CDB_ANY_COMMAND_HDR;

/**
 *
 *    Functions:  decap_osd_cdb_any_hdr
 *		 print_osd_cdb_any_hdr
 *
 *    Description: Functions decapsulates and print OSD ANY CDB 
 *
 *    Argument:  @osd_cdb_any_hdr - store decapsulated CDB
 *		 @cmd - OSD CBD from network
 * 
 *    Return codes: 0 if success,1 otherwise  
 *
 **/

void decap_osd_cdb_any_hdr(OSD_CDB_ANY_COMMAND_HDR *osd_cdb_any_hdr,u8 *cmd );
void print_osd_cdb_any_hdr(OSD_CDB_ANY_COMMAND_HDR *osd_cdb_any_hdr);

# define OSD_CDB_STRUCT  OSD_CDB_SECURITY_HDR


/**
 *
 *    Functions:  handle_osd_cmnd_get_scsi_ahs_offset
 *
 *    Description: Print OSD SCSI Additional Header segment 
 *
 *    Arguments:  @ahs - OSD SCSI CDB with AHS 
 *		 @ahssize - AHS size
 * 
 *    Return codes: 0 if success,1 otherwise  
 *
 **/

int handle_osd_cmnd_get_scsi_ahs_offset(u8 *ahs, u32 ahssize);

/**
 *
 *    Functions:  handle_osd_cmnd_get_osd_cdb_struct
 *
 *    Description: Function decapsulate OSD CDB according to OSD command 
 * 		  specific parameters from network to host 
 *
 *    Arguments:  @kern_usr_meta - OSD CDB  fields used are
 *				scb - SCSI CDB of 16 bytes
 *				ahs_data - AHS data of 184 bytes
 *		 @osd_cdb - to store decapsulated OSD CDB of 200 bytes 
 * 
 *    Return codes: 0 if success,1 otherwise  
 *
 **/

int handle_osd_cmnd_get_osd_cdb_struct(OSD_HEADER  *kern_usr_meta,OSD_CDB_STRUCT * osd_cdb);

/**
 *
 *    Functions:  handle_osd_cmnd_get_din_size
 *		 handle_osd_cmnd_get_dout_size
 *
 *    Description: Functions Calculate data in/out buffer size for OSD command
 *
 *    Arguments:  @osd_cdb - OSD CDB 
 * 
 *    Return codes: data buffer size
 *
 **/

u32 handle_osd_cmnd_get_din_size(OSD_CDB_STRUCT *osd_cdb);
u32 handle_osd_cmnd_get_dout_size(OSD_CDB_STRUCT *osd_cdb);

#endif /*OSD_H*/

