/**
 *                      ==================
 *                      IITK OSD Simulator
 *                      ==================
 *   An Implementation of Simulator for Object based storage device(OSD)
 *
 *   Copyright (C) 2007-2008 CSE IIT KANPUR
 *
 *   Contact: Department of Computer Science and Engineering
 *            Indian Institute of Technology,Kanpur(IIT Kanpur)
 *            {moona,riyaz}@cse.iitk.ac.in
 *
 *   This file is part of IITK OSD Simulator Project.
 *
 *   IITK OSD Simulator Project is free software; you can redistribute it
 *   and/or modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   IITK OSD Simulator Project is distributed in the hope that it will be
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the  GNU General
 *   Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
# include "osd_attr.h"


/**
**********************************
* File:  osd_attr.c
* Functions to initialize structure variable associated with each page of each object type 
**/
//-------------------------------------------------------------------------------------------------

void  initialize_all_pages_struct()
{
	 initialize_root_directory_page_struct();
	 initialize_root_info_page_struct();
	 initialize_root_quotas_page_struct();
	 initialize_root_timestamps_page_struct();
	 initialize_root_security_page_struct();
	
	 initialize_partition_directory_page_struct();
	 initialize_partition_info_page_struct();
	 initialize_partition_quotas_page_struct();
	 initialize_partition_timestamps_page_struct();
	 initialize_partition_security_page_struct();
	
	 initialize_uobject_directory_page_struct();
	 initialize_uobject_info_page_struct();
	 initialize_uobject_quotas_page_struct();
	 initialize_uobject_timestamps_page_struct();
	 initialize_uobject_security_page_struct();
	 initialize_uobject_collections_page_struct();

	 initialize_collection_directory_page_struct();
	 initialize_collection_info_page_struct();
	 initialize_collection_timestamps_page_struct();
	 initialize_collection_security_page_struct();

	 initialize_current_command_pages_struct();	
	
}
//--------------------------------------------------------------------------------------------------

void initialize_root_directory_page_struct()
{
    OSD_ROOT_DIRECTORY_PAGE.attr_count=    OSD_PAGE_NUM_ROOT_DIRECTORY_ATTR_COUNT;
     OSD_ROOT_DIRECTORY_PAGE.attributes=(OSD_ATTR_INFO *)allocate(sizeof(OSD_ATTR_INFO)*OSD_PAGE_NUM_ROOT_DIRECTORY_ATTR_COUNT);
	
     	OSD_ROOT_DIRECTORY_PAGE.attributes[0].attr_no=OSD_ATTR_NUM_ROOT_DIRECTORY_page_id;          
	OSD_ROOT_DIRECTORY_PAGE.attributes[0].attr_size=OSD_ATTR_NUM_ROOT_DIRECTORY_page_id_size;
	OSD_ROOT_DIRECTORY_PAGE.attributes[0].attr_offset=OSD_ATTR_NUM_ROOT_DIRECTORY_offset;
	OSD_ROOT_DIRECTORY_PAGE.attributes[0].attr_set=OSD_ATTR_NUM_ROOT_DIRECTORY_set;
	
	OSD_ROOT_DIRECTORY_PAGE.attributes[1].attr_no=OSD_ATTR_NUM_ROOT_DIRECTORY_A1;
	OSD_ROOT_DIRECTORY_PAGE.attributes[1].attr_size=OSD_ATTR_NUM_ROOT_DIRECTORY_A1_size;
	OSD_ROOT_DIRECTORY_PAGE.attributes[1].attr_offset=OSD_ATTR_NUM_ROOT_DIRECTORY_A1_offset;
	OSD_ROOT_DIRECTORY_PAGE.attributes[1].attr_set=OSD_ATTR_NUM_ROOT_DIRECTORY_A1_set;
	
	OSD_ROOT_DIRECTORY_PAGE.attributes[2].attr_no=OSD_ATTR_NUM_ROOT_DIRECTORY_A2;
	OSD_ROOT_DIRECTORY_PAGE.attributes[2].attr_size=OSD_ATTR_NUM_ROOT_DIRECTORY_A2_size;
	OSD_ROOT_DIRECTORY_PAGE.attributes[2].attr_offset=OSD_ATTR_NUM_ROOT_DIRECTORY_A2_offset;
	OSD_ROOT_DIRECTORY_PAGE.attributes[2].attr_set=OSD_ATTR_NUM_ROOT_DIRECTORY_A2_set;

	OSD_ROOT_DIRECTORY_PAGE.attributes[3].attr_no=OSD_ATTR_NUM_ROOT_DIRECTORY_A3;
	OSD_ROOT_DIRECTORY_PAGE.attributes[3].attr_size=OSD_ATTR_NUM_ROOT_DIRECTORY_A3_size;
	OSD_ROOT_DIRECTORY_PAGE.attributes[3].attr_offset=OSD_ATTR_NUM_ROOT_DIRECTORY_A3_offset;
	OSD_ROOT_DIRECTORY_PAGE.attributes[3].attr_set=OSD_ATTR_NUM_ROOT_DIRECTORY_A3_set;

	OSD_ROOT_DIRECTORY_PAGE.attributes[4].attr_no=OSD_ATTR_NUM_ROOT_DIRECTORY_A5;
	OSD_ROOT_DIRECTORY_PAGE.attributes[4].attr_size=OSD_ATTR_NUM_ROOT_DIRECTORY_A5_size;
	OSD_ROOT_DIRECTORY_PAGE.attributes[4].attr_offset=OSD_ATTR_NUM_ROOT_DIRECTORY_A5_offset;
	OSD_ROOT_DIRECTORY_PAGE.attributes[4].attr_set=OSD_ATTR_NUM_ROOT_DIRECTORY_A5_set;

}
//--------------------------------------------------------------------------------------------------
void initialize_root_info_page_struct()
{
	OSD_ROOT_INFO_PAGE.attr_count=OSD_PAGE_NUM_ROOT_INFO_ATTR_COUNT;
	OSD_ROOT_INFO_PAGE.attributes=(OSD_ATTR_INFO *)allocate(sizeof(OSD_ATTR_INFO)*OSD_PAGE_NUM_ROOT_INFO_ATTR_COUNT);
	
	OSD_ROOT_INFO_PAGE.attributes[0].attr_no=OSD_ATTR_NUM_ROOT_INFO_page_id;
	OSD_ROOT_INFO_PAGE.attributes[0].attr_size=PAGE_ID_ATTR_SIZE;
	OSD_ROOT_INFO_PAGE.attributes[0].attr_offset=PAGE_ID_ATTR_OFFSET;
	OSD_ROOT_INFO_PAGE.attributes[0].attr_set=0;
		
	OSD_ROOT_INFO_PAGE.attributes[1].attr_no=OSD_ATTR_NUM_ROOT_INFO_OSD_SYSTEM_ID;
	OSD_ROOT_INFO_PAGE.attributes[1].attr_size=OSD_ATTR_NUM_ROOT_INFO_OSD_SYSTEM_ID_size;
	OSD_ROOT_INFO_PAGE.attributes[1].attr_offset=OSD_ATTR_NUM_ROOT_INFO_OSD_SYSTEM_ID_offset;
	OSD_ROOT_INFO_PAGE.attributes[1].attr_set=OSD_ATTR_NUM_ROOT_INFO_OSD_SYSTEM_ID_set;
	
	OSD_ROOT_INFO_PAGE.attributes[2].attr_no=OSD_ATTR_NUM_ROOT_INFO_VENDOR_ID;
	OSD_ROOT_INFO_PAGE.attributes[2].attr_size=OSD_ATTR_NUM_ROOT_INFO_VENDOR_ID_size;
	OSD_ROOT_INFO_PAGE.attributes[2].attr_offset=OSD_ATTR_NUM_ROOT_INFO_VENDOR_ID_offset;
	OSD_ROOT_INFO_PAGE.attributes[2].attr_set=OSD_ATTR_NUM_ROOT_INFO_VENDOR_ID_set;
	
	OSD_ROOT_INFO_PAGE.attributes[3].attr_no=OSD_ATTR_NUM_ROOT_INFO_PRODUCT_ID;
	OSD_ROOT_INFO_PAGE.attributes[3].attr_size=OSD_ATTR_NUM_ROOT_INFO_PRODUCT_ID_size;
	OSD_ROOT_INFO_PAGE.attributes[3].attr_offset=OSD_ATTR_NUM_ROOT_INFO_PRODUCT_ID_offset;
	OSD_ROOT_INFO_PAGE.attributes[3].attr_set=OSD_ATTR_NUM_ROOT_INFO_PRODUCT_ID_set;
	
	OSD_ROOT_INFO_PAGE.attributes[4].attr_no=OSD_ATTR_NUM_ROOT_INFO_PRODUCT_MODEL;
	OSD_ROOT_INFO_PAGE.attributes[4].attr_size=OSD_ATTR_NUM_ROOT_INFO_PRODUCT_MODEL_size;
	OSD_ROOT_INFO_PAGE.attributes[4].attr_offset=OSD_ATTR_NUM_ROOT_INFO_PRODUCT_MODEL_offset;
	OSD_ROOT_INFO_PAGE.attributes[4].attr_set=OSD_ATTR_NUM_ROOT_INFO_PRODUCT_MODEL_set;

	
	OSD_ROOT_INFO_PAGE.attributes[5].attr_no=OSD_ATTR_NUM_ROOT_INFO_PRODUCT_REVISION_LEVEL;
	OSD_ROOT_INFO_PAGE.attributes[5].attr_size=OSD_ATTR_NUM_ROOT_INFO_PRODUCT_REVISION_LEVEL_size;
	OSD_ROOT_INFO_PAGE.attributes[5].attr_offset=OSD_ATTR_NUM_ROOT_INFO_PRODUCT_REVISION_LEVEL_offset;
	OSD_ROOT_INFO_PAGE.attributes[5].attr_set=OSD_ATTR_NUM_ROOT_INFO_PRODUCT_REVISION_LEVEL_set;
	
	OSD_ROOT_INFO_PAGE.attributes[6].attr_no=OSD_ATTR_NUM_ROOT_INFO_SERIAL_NUMBER;
	OSD_ROOT_INFO_PAGE.attributes[6].attr_size=OSD_ATTR_NUM_ROOT_INFO_SERIAL_NUMBER_size;
	OSD_ROOT_INFO_PAGE.attributes[6].attr_offset=OSD_ATTR_NUM_ROOT_INFO_SERIAL_NUMBER_offset;
	OSD_ROOT_INFO_PAGE.attributes[6].attr_set=OSD_ATTR_NUM_ROOT_INFO_SERIAL_NUMBER_set;
	
	OSD_ROOT_INFO_PAGE.attributes[7].attr_no=OSD_ATTR_NUM_ROOT_INFO_OSD_NAME;
	OSD_ROOT_INFO_PAGE.attributes[7].attr_size=OSD_ATTR_NUM_ROOT_INFO_OSD_NAME_size;
	OSD_ROOT_INFO_PAGE.attributes[7].attr_offset=OSD_ATTR_NUM_ROOT_INFO_OSD_NAME_offset;
	OSD_ROOT_INFO_PAGE.attributes[7].attr_set=OSD_ATTR_NUM_ROOT_INFO_OSD_NAME_set;
	
	OSD_ROOT_INFO_PAGE.attributes[8].attr_no=OSD_ATTR_NUM_ROOT_INFO_TOTAL_CAPACITY;
	OSD_ROOT_INFO_PAGE.attributes[8].attr_size=OSD_ATTR_NUM_ROOT_INFO_TOTAL_CAPACITY_size;
	OSD_ROOT_INFO_PAGE.attributes[8].attr_offset=OSD_ATTR_NUM_ROOT_INFO_TOTAL_CAPACITY_offset;
	OSD_ROOT_INFO_PAGE.attributes[8].attr_set=OSD_ATTR_NUM_ROOT_INFO_TOTAL_CAPACITY_set;
	
	OSD_ROOT_INFO_PAGE.attributes[9].attr_no=OSD_ATTR_NUM_ROOT_INFO_USED_CAPACITY;
	OSD_ROOT_INFO_PAGE.attributes[9].attr_size=OSD_ATTR_NUM_ROOT_INFO_USED_CAPACITY_size;
	OSD_ROOT_INFO_PAGE.attributes[9].attr_offset=OSD_ATTR_NUM_ROOT_INFO_USED_CAPACITY_offset;
	OSD_ROOT_INFO_PAGE.attributes[9].attr_set=OSD_ATTR_NUM_ROOT_INFO_USED_CAPACITY_set;
	
	OSD_ROOT_INFO_PAGE.attributes[10].attr_no=OSD_ATTR_NUM_ROOT_INFO_NUM_OF_PARTITIONS;
	OSD_ROOT_INFO_PAGE.attributes[10].attr_size=OSD_ATTR_NUM_ROOT_INFO_NUM_OF_PARTITIONS_size;
	OSD_ROOT_INFO_PAGE.attributes[10].attr_offset=OSD_ATTR_NUM_ROOT_INFO_NUM_OF_PARTITIONS_offset;
	OSD_ROOT_INFO_PAGE.attributes[10].attr_set=OSD_ATTR_NUM_ROOT_INFO_NUM_OF_PARTITIONS_set;

	OSD_ROOT_INFO_PAGE.attributes[11].attr_no=OSD_ATTR_NUM_ROOT_INFO_CLOCK;
	OSD_ROOT_INFO_PAGE.attributes[11].attr_size=OSD_ATTR_NUM_ROOT_INFO_CLOCK_size;
	OSD_ROOT_INFO_PAGE.attributes[11].attr_offset=OSD_ATTR_NUM_ROOT_INFO_CLOCK_offset;	
	OSD_ROOT_INFO_PAGE.attributes[11].attr_set=OSD_ATTR_NUM_ROOT_INFO_CLOCK_set;	

}
//--------------------------------------------------------------------------------------------------
void initialize_root_quotas_page_struct()
{
	OSD_ROOT_QUOTAS_PAGE.attr_count=OSD_PAGE_NUM_ROOT_QUOTAS_ATTR_COUNT;
	OSD_ROOT_QUOTAS_PAGE.attributes=(OSD_ATTR_INFO *)allocate(sizeof(OSD_ATTR_INFO)*OSD_PAGE_NUM_ROOT_QUOTAS_ATTR_COUNT);
	
	OSD_ROOT_QUOTAS_PAGE.attributes[0].attr_no=OSD_ATTR_NUM_ROOT_QUOTAS_page_id;
	OSD_ROOT_QUOTAS_PAGE.attributes[0].attr_size=PAGE_ID_ATTR_SIZE;
	OSD_ROOT_QUOTAS_PAGE.attributes[0].attr_offset=PAGE_ID_ATTR_OFFSET;
	OSD_ROOT_QUOTAS_PAGE.attributes[0].attr_set=0;
	
	OSD_ROOT_QUOTAS_PAGE.attributes[1].attr_no=OSD_ATTR_NUM_ROOT_QUOTAS_DEFAULT_MAX_OBJ_LEN;
	OSD_ROOT_QUOTAS_PAGE.attributes[1].attr_size=OSD_ATTR_NUM_ROOT_QUOTAS_DEFAULT_MAX_OBJ_LEN_size;
	OSD_ROOT_QUOTAS_PAGE.attributes[1].attr_offset=OSD_ATTR_NUM_ROOT_QUOTAS_DEFAULT_MAX_OBJ_LEN_offset;
	OSD_ROOT_QUOTAS_PAGE.attributes[1].attr_set=OSD_ATTR_NUM_ROOT_QUOTAS_DEFAULT_MAX_OBJ_LEN_set;

	OSD_ROOT_QUOTAS_PAGE.attributes[2].attr_no=OSD_ATTR_NUM_ROOT_QUOTAS_PARTITION_CAPACITY_QUOTA;
	OSD_ROOT_QUOTAS_PAGE.attributes[2].attr_size=OSD_ATTR_NUM_ROOT_QUOTAS_PARTITION_CAPACITY_QUOTA_size;
	OSD_ROOT_QUOTAS_PAGE.attributes[2].attr_offset=OSD_ATTR_NUM_ROOT_QUOTAS_PARTITION_CAPACITY_QUOTA_offset;
	OSD_ROOT_QUOTAS_PAGE.attributes[2].attr_set=OSD_ATTR_NUM_ROOT_QUOTAS_PARTITION_CAPACITY_QUOTA_set;
	
	OSD_ROOT_QUOTAS_PAGE.attributes[3].attr_no=OSD_ATTR_NUM_ROOT_QUOTAS_PARTITION_OBJ_COUNT;
	OSD_ROOT_QUOTAS_PAGE.attributes[3].attr_size=OSD_ATTR_NUM_ROOT_QUOTAS_PARTITION_OBJ_COUNT_size;
	OSD_ROOT_QUOTAS_PAGE.attributes[3].attr_offset=OSD_ATTR_NUM_ROOT_QUOTAS_PARTITION_OBJ_COUNT_offset;
	OSD_ROOT_QUOTAS_PAGE.attributes[3].attr_set=OSD_ATTR_NUM_ROOT_QUOTAS_PARTITION_OBJ_COUNT_set;

	OSD_ROOT_QUOTAS_PAGE.attributes[4].attr_no=OSD_ATTR_NUM_ROOT_QUOTAS_PARTITION_COLLECTIONS_PER_OBJ;
	OSD_ROOT_QUOTAS_PAGE.attributes[4].attr_size=OSD_ATTR_NUM_ROOT_QUOTAS_PARTITION_COLLECTIONS_PER_OBJ_size;
	OSD_ROOT_QUOTAS_PAGE.attributes[4].attr_offset=OSD_ATTR_NUM_ROOT_QUOTAS_PARTITION_COLLECTIONS_PER_OBJ_offset;
	OSD_ROOT_QUOTAS_PAGE.attributes[4].attr_set=OSD_ATTR_NUM_ROOT_QUOTAS_PARTITION_COLLECTIONS_PER_OBJ_set;
	
	OSD_ROOT_QUOTAS_PAGE.attributes[5].attr_no=OSD_ATTR_NUM_ROOT_QUOTAS_PARTITION_COUNT;
	OSD_ROOT_QUOTAS_PAGE.attributes[5].attr_size=OSD_ATTR_NUM_ROOT_QUOTAS_PARTITION_COUNT_size;
	OSD_ROOT_QUOTAS_PAGE.attributes[5].attr_offset=OSD_ATTR_NUM_ROOT_QUOTAS_PARTITION_COUNT_offset;
	OSD_ROOT_QUOTAS_PAGE.attributes[5].attr_set=OSD_ATTR_NUM_ROOT_QUOTAS_PARTITION_COUNT_set;
	
}
	
void initialize_root_timestamps_page_struct()
{
	OSD_ROOT_TIMESTAMPS_PAGE.attr_count=OSD_PAGE_NUM_ROOT_TIMESTAMPS_ATTR_COUNT;
	OSD_ROOT_TIMESTAMPS_PAGE.attributes=(OSD_ATTR_INFO *)allocate(sizeof(OSD_ATTR_INFO)*OSD_PAGE_NUM_ROOT_TIMESTAMPS_ATTR_COUNT);
	

	OSD_ROOT_TIMESTAMPS_PAGE.attributes[0].attr_no=OSD_ATTR_NUM_ROOT_TIMESTAMPS_page_id;
	OSD_ROOT_TIMESTAMPS_PAGE.attributes[0].attr_size=PAGE_ID_ATTR_SIZE;
	OSD_ROOT_TIMESTAMPS_PAGE.attributes[0].attr_offset=PAGE_ID_ATTR_OFFSET;
	OSD_ROOT_TIMESTAMPS_PAGE.attributes[0].attr_set=0;
	
	OSD_ROOT_TIMESTAMPS_PAGE.attributes[1].attr_no=OSD_ATTR_NUM_ROOT_TIMESTAMPS_ATTR_ACCESSED_TIME;
	OSD_ROOT_TIMESTAMPS_PAGE.attributes[1].attr_size=OSD_ATTR_NUM_ROOT_TIMESTAMPS_ATTR_ACCESSED_TIME_size;
	OSD_ROOT_TIMESTAMPS_PAGE.attributes[1].attr_offset=OSD_ATTR_NUM_ROOT_TIMESTAMPS_ATTR_ACCESSED_TIME_offset;
	OSD_ROOT_TIMESTAMPS_PAGE.attributes[1].attr_set=OSD_ATTR_NUM_ROOT_TIMESTAMPS_ATTR_ACCESSED_TIME_set;

	OSD_ROOT_TIMESTAMPS_PAGE.attributes[2].attr_no=OSD_ATTR_NUM_ROOT_TIMESTAMPS_ATTR_MODIFIED_TIME;
	OSD_ROOT_TIMESTAMPS_PAGE.attributes[2].attr_size=OSD_ATTR_NUM_ROOT_TIMESTAMPS_ATTR_MODIFIED_TIME_size;
	OSD_ROOT_TIMESTAMPS_PAGE.attributes[2].attr_offset=OSD_ATTR_NUM_ROOT_TIMESTAMPS_ATTR_MODIFIED_TIME_offset;
	OSD_ROOT_TIMESTAMPS_PAGE.attributes[2].attr_set=OSD_ATTR_NUM_ROOT_TIMESTAMPS_ATTR_MODIFIED_TIME_set;
	
	OSD_ROOT_TIMESTAMPS_PAGE.attributes[3].attr_no=OSD_ATTR_NUM_ROOT_TIMESTAMPS_TIMESTAMP_BYPASS;
	OSD_ROOT_TIMESTAMPS_PAGE.attributes[3].attr_size=OSD_ATTR_NUM_ROOT_TIMESTAMPS_TIMESTAMP_BYPASS_size;
	OSD_ROOT_TIMESTAMPS_PAGE.attributes[3].attr_offset=OSD_ATTR_NUM_ROOT_TIMESTAMPS_TIMESTAMP_BYPASS_offset;
	OSD_ROOT_TIMESTAMPS_PAGE.attributes[3].attr_set=OSD_ATTR_NUM_ROOT_TIMESTAMPS_TIMESTAMP_BYPASS_set;
	

}
//--------------------------------------------------------------------------------------------------
void initialize_root_security_page_struct()
{
	int i;

	OSD_ROOT_SECURITY_PAGE.attr_count=OSD_PAGE_NUM_ROOT_SECURITY_ATTR_COUNT;
	OSD_ROOT_SECURITY_PAGE.attributes=(OSD_ATTR_INFO  *)allocate(sizeof(OSD_ATTR_INFO)*OSD_PAGE_NUM_ROOT_SECURITY_ATTR_COUNT);

		
	OSD_ROOT_SECURITY_PAGE.attributes[0].attr_no=OSD_ATTR_NUM_ROOT_SECURITY_page_id;
	OSD_ROOT_SECURITY_PAGE.attributes[0].attr_size=PAGE_ID_ATTR_SIZE;
	OSD_ROOT_SECURITY_PAGE.attributes[0].attr_offset=PAGE_ID_ATTR_OFFSET;
	OSD_ROOT_SECURITY_PAGE.attributes[0].attr_set=0;
	
	
	OSD_ROOT_SECURITY_PAGE.attributes[1].attr_no=OSD_ATTR_NUM_ROOT_SECURITY_DEFAULT_SEC_METHOD;
	OSD_ROOT_SECURITY_PAGE.attributes[1].attr_size=OSD_ATTR_NUM_ROOT_SECURITY_DEFAULT_SEC_METHOD_size;
	OSD_ROOT_SECURITY_PAGE.attributes[1].attr_offset=OSD_ATTR_NUM_ROOT_SECURITY_DEFAULT_SEC_METHOD_offset;
	OSD_ROOT_SECURITY_PAGE.attributes[1].attr_set=OSD_ATTR_NUM_ROOT_SECURITY_DEFAULT_SEC_METHOD_set;
	
	OSD_ROOT_SECURITY_PAGE.attributes[2].attr_no=OSD_ATTR_NUM_ROOT_SECURITY_OLDEST_VALID_NONCE;
	OSD_ROOT_SECURITY_PAGE.attributes[2].attr_size=OSD_ATTR_NUM_ROOT_SECURITY_OLDEST_VALID_NONCE_size;
	OSD_ROOT_SECURITY_PAGE.attributes[2].attr_offset=OSD_ATTR_NUM_ROOT_SECURITY_OLDEST_VALID_NONCE_offset;
	OSD_ROOT_SECURITY_PAGE.attributes[2].attr_set=OSD_ATTR_NUM_ROOT_SECURITY_OLDEST_VALID_NONCE_set;
	
	OSD_ROOT_SECURITY_PAGE.attributes[3].attr_no=OSD_ATTR_NUM_ROOT_SECURITY_NEWEST_VALID_NONCE;
	OSD_ROOT_SECURITY_PAGE.attributes[3].attr_size=OSD_ATTR_NUM_ROOT_SECURITY_NEWEST_VALID_NONCE_size;
	OSD_ROOT_SECURITY_PAGE.attributes[3].attr_offset=OSD_ATTR_NUM_ROOT_SECURITY_NEWEST_VALID_NONCE_offset;
	OSD_ROOT_SECURITY_PAGE.attributes[3].attr_set=OSD_ATTR_NUM_ROOT_SECURITY_NEWEST_VALID_NONCE_set;

	OSD_ROOT_SECURITY_PAGE.attributes[4].attr_no=OSD_ATTR_NUM_ROOT_SECURITY_PARTITION_DEFAULT_SEC_METHOD;
	OSD_ROOT_SECURITY_PAGE.attributes[4].attr_size=OSD_ATTR_NUM_ROOT_SECURITY_PARTITION_DEFAULT_SEC_METHOD_size;
	OSD_ROOT_SECURITY_PAGE.attributes[4].attr_offset=OSD_ATTR_NUM_ROOT_SECURITY_PARTITION_DEFAULT_SEC_METHOD_offset;
	OSD_ROOT_SECURITY_PAGE.attributes[4].attr_set=OSD_ATTR_NUM_ROOT_SECURITY_PARTITION_DEFAULT_SEC_METHOD_set;
	
	OSD_ROOT_SECURITY_PAGE.attributes[5].attr_no=OSD_ATTR_NUM_ROOT_SECURITY_SUPPORTED_SEC_METHODS;
	OSD_ROOT_SECURITY_PAGE.attributes[5].attr_size=OSD_ATTR_NUM_ROOT_SECURITY_SUPPORTED_SEC_METHODS_size;
	OSD_ROOT_SECURITY_PAGE.attributes[5].attr_offset=OSD_ATTR_NUM_ROOT_SECURITY_SUPPORTED_SEC_METHODS_offset;
	OSD_ROOT_SECURITY_PAGE.attributes[5].attr_set=OSD_ATTR_NUM_ROOT_SECURITY_SUPPORTED_SEC_METHODS_set;
	
	OSD_ROOT_SECURITY_PAGE.attributes[6].attr_no=OSD_ATTR_NUM_ROOT_SECURITY_ADJUSTABLE_CLOCK;
	OSD_ROOT_SECURITY_PAGE.attributes[6].attr_size=OSD_ATTR_NUM_ROOT_SECURITY_ADJUSTABLE_CLOCK_size;
	OSD_ROOT_SECURITY_PAGE.attributes[6].attr_offset=OSD_ATTR_NUM_ROOT_SECURITY_ADJUSTABLE_CLOCK_offset;
	OSD_ROOT_SECURITY_PAGE.attributes[6].attr_set=OSD_ATTR_NUM_ROOT_SECURITY_ADJUSTABLE_CLOCK_set;
	
	OSD_ROOT_SECURITY_PAGE.attributes[7].attr_no=OSD_ATTR_NUM_ROOT_SECURITY_MASTER_KEY_ID;
	OSD_ROOT_SECURITY_PAGE.attributes[7].attr_size=OSD_ATTR_NUM_ROOT_SECURITY_MASTER_KEY_ID_size;
	OSD_ROOT_SECURITY_PAGE.attributes[7].attr_offset=OSD_ATTR_NUM_ROOT_SECURITY_MASTER_KEY_ID_offset;
	OSD_ROOT_SECURITY_PAGE.attributes[7].attr_set=OSD_ATTR_NUM_ROOT_SECURITY_MASTER_KEY_ID_set;
	
	OSD_ROOT_SECURITY_PAGE.attributes[8].attr_no=OSD_ATTR_NUM_ROOT_SECURITY_ROOT_KEY_ID;
	OSD_ROOT_SECURITY_PAGE.attributes[8].attr_size=OSD_ATTR_NUM_ROOT_SECURITY_ROOT_KEY_ID_size;
	OSD_ROOT_SECURITY_PAGE.attributes[8].attr_offset=OSD_ATTR_NUM_ROOT_SECURITY_ROOT_KEY_ID_offset;
	OSD_ROOT_SECURITY_PAGE.attributes[8].attr_set=OSD_ATTR_NUM_ROOT_SECURITY_ROOT_KEY_ID_set;
	
	
	for (i=0;i<16;i++)
	{
		OSD_ROOT_SECURITY_PAGE.attributes[i+9].attr_no=OSD_ATTR_NUM_ROOT_SECURITY_ALG_1+i;
		OSD_ROOT_SECURITY_PAGE.attributes[i+9].attr_size=1;
		OSD_ROOT_SECURITY_PAGE.attributes[i+9].attr_offset=OSD_ATTR_NUM_ROOT_SECURITY_ALG_1_offset+i;
		OSD_ROOT_SECURITY_PAGE.attributes[i+9].attr_set=OSD_ATTR_NUM_ROOT_SECURITY_ALG_1_set;
		
	}	
	
	for (i=0;i<16;i++)
	{
	
	OSD_ROOT_SECURITY_PAGE.attributes[i+24].attr_no=
	OSD_ATTR_NUM_ROOT_SECURITY_DH_GROUP_1+i;
	OSD_ROOT_SECURITY_PAGE.attributes[i+24].attr_size=1;
	OSD_ROOT_SECURITY_PAGE.attributes[i+24].attr_offset=OSD_ATTR_NUM_ROOT_SECURITY_DH_GROUP_1_offset+i;
	OSD_ROOT_SECURITY_PAGE.attributes[i+24].attr_set=OSD_ATTR_NUM_ROOT_SECURITY_DH_GROUP_1_set;
	}
}
//--------------------------------------------------------------------------------------------------
void initialize_partition_directory_page_struct()
{

   OSD_PART_DIRECTORY_PAGE.attr_count= OSD_PAGE_NUM_PART_DIRECTORY_ATTR_COUNT;
     OSD_PART_DIRECTORY_PAGE.attributes=(OSD_ATTR_INFO *)allocate(sizeof(OSD_ATTR_INFO)*OSD_PAGE_NUM_PART_DIRECTORY_ATTR_COUNT);
 
	OSD_PART_DIRECTORY_PAGE.attributes[0].attr_no= OSD_ATTR_NUM_PART_DIRECTORY_page_id;
	OSD_PART_DIRECTORY_PAGE.attributes[0].attr_size=OSD_ATTR_NUM_PART_DIRECTORY_page_id_size;
	OSD_PART_DIRECTORY_PAGE.attributes[0].attr_offset=OSD_ATTR_NUM_PART_DIRECTORY_page_id_offset;
	OSD_PART_DIRECTORY_PAGE.attributes[0].attr_set=OSD_ATTR_NUM_PART_DIRECTORY_page_id_set;

	OSD_PART_DIRECTORY_PAGE.attributes[1].attr_no=OSD_ATTR_NUM_PART_DIRECTORY_A1;
	OSD_PART_DIRECTORY_PAGE.attributes[1].attr_size=OSD_ATTR_NUM_PART_DIRECTORY_A1_size;
	OSD_PART_DIRECTORY_PAGE.attributes[1].attr_offset=OSD_ATTR_NUM_PART_DIRECTORY_A1_offset;
	OSD_PART_DIRECTORY_PAGE.attributes[1].attr_set=OSD_ATTR_NUM_PART_DIRECTORY_A1_set;

	OSD_PART_DIRECTORY_PAGE.attributes[2].attr_no=OSD_ATTR_NUM_PART_DIRECTORY_A2;
	OSD_PART_DIRECTORY_PAGE.attributes[2].attr_size=OSD_ATTR_NUM_PART_DIRECTORY_A2_size;
	OSD_PART_DIRECTORY_PAGE.attributes[2].attr_offset=OSD_ATTR_NUM_PART_DIRECTORY_A2_offset;
	OSD_PART_DIRECTORY_PAGE.attributes[2].attr_set=OSD_ATTR_NUM_PART_DIRECTORY_A2_set;
	
	OSD_PART_DIRECTORY_PAGE.attributes[3].attr_no=OSD_ATTR_NUM_PART_DIRECTORY_A3;
	OSD_PART_DIRECTORY_PAGE.attributes[3].attr_size=OSD_ATTR_NUM_PART_DIRECTORY_A3_size;
	OSD_PART_DIRECTORY_PAGE.attributes[3].attr_offset=OSD_ATTR_NUM_PART_DIRECTORY_A3_offset;
	OSD_PART_DIRECTORY_PAGE.attributes[3].attr_set=OSD_ATTR_NUM_PART_DIRECTORY_A3_set;
	
	OSD_PART_DIRECTORY_PAGE.attributes[4].attr_no=OSD_ATTR_NUM_PART_DIRECTORY_A5;
	OSD_PART_DIRECTORY_PAGE.attributes[4].attr_size=OSD_ATTR_NUM_PART_DIRECTORY_A5_size;
	OSD_PART_DIRECTORY_PAGE.attributes[4].attr_offset=OSD_ATTR_NUM_PART_DIRECTORY_A5_offset;
	OSD_PART_DIRECTORY_PAGE.attributes[4].attr_set=OSD_ATTR_NUM_PART_DIRECTORY_A5_set;

}
//--------------------------------------------------------------------------------------------------

void initialize_partition_info_page_struct()
{
	OSD_PART_INFO_PAGE.attr_count= OSD_PAGE_NUM_PART_INFO_ATTR_COUNT;
	OSD_PART_INFO_PAGE.attributes=(OSD_ATTR_INFO *)allocate(sizeof(OSD_ATTR_INFO)*OSD_PAGE_NUM_PART_INFO_ATTR_COUNT);
	
	OSD_PART_INFO_PAGE.attributes[0].attr_no=OSD_ATTR_NUM_PART_INFO_page_id;
	OSD_PART_INFO_PAGE.attributes[0].attr_size=PAGE_ID_ATTR_SIZE;
	OSD_PART_INFO_PAGE.attributes[0].attr_offset=	PAGE_ID_ATTR_OFFSET;OSD_PART_INFO_PAGE.attributes[0].attr_set=	0;
	
	
	OSD_PART_INFO_PAGE.attributes[1].attr_no=OSD_ATTR_NUM_PART_INFO_PARTITION_ID;
	OSD_PART_INFO_PAGE.attributes[1].attr_size=OSD_ATTR_NUM_PART_INFO_PARTITION_ID_size;
	OSD_PART_INFO_PAGE.attributes[1].attr_offset=	OSD_ATTR_NUM_PART_INFO_PARTITION_ID_offset;
	OSD_PART_INFO_PAGE.attributes[1].attr_set=	OSD_ATTR_NUM_PART_INFO_PARTITION_ID_set;
	
	OSD_PART_INFO_PAGE.attributes[2].attr_no=OSD_ATTR_NUM_PART_INFO_USERNAME;
	OSD_PART_INFO_PAGE.attributes[2].attr_size=OSD_ATTR_NUM_PART_INFO_USERNAME_size;
	OSD_PART_INFO_PAGE.attributes[2].attr_offset=	OSD_ATTR_NUM_PART_INFO_USERNAME_offset;
	OSD_PART_INFO_PAGE.attributes[2].attr_set=	OSD_ATTR_NUM_PART_INFO_USERNAME_set;
	
	OSD_PART_INFO_PAGE.attributes[3].attr_no=OSD_ATTR_NUM_PART_INFO_USED_CAPACITY;
	OSD_PART_INFO_PAGE.attributes[3].attr_size=OSD_ATTR_NUM_PART_INFO_USED_CAPACITY_size;
	OSD_PART_INFO_PAGE.attributes[3].attr_offset=	OSD_ATTR_NUM_PART_INFO_USED_CAPACITY_offset;
	OSD_PART_INFO_PAGE.attributes[3].attr_set=	OSD_ATTR_NUM_PART_INFO_USED_CAPACITY_set;
	
	
	OSD_PART_INFO_PAGE.attributes[4].attr_no=OSD_ATTR_NUM_PART_INFO_NUM_OF_OBJECTS;
	OSD_PART_INFO_PAGE.attributes[4].attr_size=OSD_ATTR_NUM_PART_INFO_NUM_OF_OBJECTS_size;
	OSD_PART_INFO_PAGE.attributes[4].attr_offset=	OSD_ATTR_NUM_PART_INFO_NUM_OF_OBJECTS_offset;
	OSD_PART_INFO_PAGE.attributes[4].attr_set=	OSD_ATTR_NUM_PART_INFO_NUM_OF_OBJECTS_set;
	
			
}
//--------------------------------------------------------------------------------------------------


void initialize_partition_quotas_page_struct()
{

	OSD_PART_QUOTAS_PAGE.attr_count= OSD_PAGE_NUM_PART_QUOTAS_ATTR_COUNT;
	OSD_PART_QUOTAS_PAGE.attributes=(OSD_ATTR_INFO *)allocate(sizeof(OSD_ATTR_INFO)*OSD_PAGE_NUM_PART_QUOTAS_ATTR_COUNT);


	OSD_PART_QUOTAS_PAGE.attributes[0].attr_no=OSD_ATTR_NUM_PART_QUOTAS_page_id;
	OSD_PART_QUOTAS_PAGE.attributes[0].attr_size=PAGE_ID_ATTR_SIZE;
	OSD_PART_QUOTAS_PAGE.attributes[0].attr_offset=PAGE_ID_ATTR_OFFSET;OSD_PART_QUOTAS_PAGE.attributes[0].attr_set=0;
	
	OSD_PART_QUOTAS_PAGE.attributes[1].attr_no=OSD_ATTR_NUM_PART_QUOTAS_DEFAULT_MAX_OBJ_LEN;
	OSD_PART_QUOTAS_PAGE.attributes[1].attr_size=OSD_ATTR_NUM_PART_QUOTAS_DEFAULT_MAX_OBJ_LEN_size;
	OSD_PART_QUOTAS_PAGE.attributes[1].attr_offset=OSD_ATTR_NUM_PART_QUOTAS_DEFAULT_MAX_OBJ_LEN_offset;
	OSD_PART_QUOTAS_PAGE.attributes[1].attr_set=OSD_ATTR_NUM_PART_QUOTAS_DEFAULT_MAX_OBJ_LEN_set;

	OSD_PART_QUOTAS_PAGE.attributes[2].attr_no=OSD_ATTR_NUM_PART_QUOTAS_CAPACITY_QUOTA;
	OSD_PART_QUOTAS_PAGE.attributes[2].attr_size=OSD_ATTR_NUM_PART_QUOTAS_CAPACITY_QUOTA_size;
	OSD_PART_QUOTAS_PAGE.attributes[2].attr_offset=OSD_ATTR_NUM_PART_QUOTAS_CAPACITY_QUOTA_offset;
	OSD_PART_QUOTAS_PAGE.attributes[2].attr_set=OSD_ATTR_NUM_PART_QUOTAS_CAPACITY_QUOTA_set;

	OSD_PART_QUOTAS_PAGE.attributes[3].attr_no=OSD_ATTR_NUM_PART_QUOTAS_OBJ_COUNT;
	OSD_PART_QUOTAS_PAGE.attributes[3].attr_size=OSD_ATTR_NUM_PART_QUOTAS_OBJ_COUNT_size;
	OSD_PART_QUOTAS_PAGE.attributes[3].attr_offset=OSD_ATTR_NUM_PART_QUOTAS_OBJ_COUNT_offset;
	OSD_PART_QUOTAS_PAGE.attributes[3].attr_set=OSD_ATTR_NUM_PART_QUOTAS_OBJ_COUNT_set;

	OSD_PART_QUOTAS_PAGE.attributes[4].attr_no=OSD_ATTR_NUM_PART_QUOTAS_COLLECTIONS_PER_OBJ;
	OSD_PART_QUOTAS_PAGE.attributes[4].attr_size=OSD_ATTR_NUM_PART_QUOTAS_COLLECTIONS_PER_OBJ_size;	OSD_PART_QUOTAS_PAGE.attributes[4].attr_offset=OSD_ATTR_NUM_PART_QUOTAS_COLLECTIONS_PER_OBJ_offset;
	OSD_PART_QUOTAS_PAGE.attributes[4].attr_set=OSD_ATTR_NUM_PART_QUOTAS_COLLECTIONS_PER_OBJ_set;
	
}

//--------------------------------------------------------------------------------------------------
void initialize_partition_timestamps_page_struct()
{

	OSD_PART_TIMESTAMPS_PAGE.attr_count= OSD_PAGE_NUM_PART_TIMESTAMPS_ATTR_COUNT;
	OSD_PART_TIMESTAMPS_PAGE.attributes=(OSD_ATTR_INFO *)allocate(sizeof(OSD_ATTR_INFO)*OSD_PAGE_NUM_PART_TIMESTAMPS_ATTR_COUNT);

	OSD_PART_TIMESTAMPS_PAGE.attributes[0].attr_no= OSD_ATTR_NUM_PART_TIMESTAMPS_page_id;
	OSD_PART_TIMESTAMPS_PAGE.attributes[0].attr_size=PAGE_ID_ATTR_SIZE;
	OSD_PART_TIMESTAMPS_PAGE.attributes[0].attr_offset=PAGE_ID_ATTR_OFFSET;OSD_PART_TIMESTAMPS_PAGE.attributes[0].attr_set=0;
	
	OSD_PART_TIMESTAMPS_PAGE.attributes[1].attr_no=OSD_ATTR_NUM_PART_TIMESTAMPS_CREATED_TIME;
	OSD_PART_TIMESTAMPS_PAGE.attributes[1].attr_size=OSD_ATTR_NUM_PART_TIMESTAMPS_CREATED_TIME_size;
	OSD_PART_TIMESTAMPS_PAGE.attributes[1].attr_offset=OSD_ATTR_NUM_PART_TIMESTAMPS_CREATED_TIME_offset;
	OSD_PART_TIMESTAMPS_PAGE.attributes[1].attr_set=OSD_ATTR_NUM_PART_TIMESTAMPS_CREATED_TIME_set;
	
	OSD_PART_TIMESTAMPS_PAGE.attributes[2].attr_no=OSD_ATTR_NUM_PART_TIMESTAMPS_ATTR_ACCESSED_TIME;
	OSD_PART_TIMESTAMPS_PAGE.attributes[2].attr_size=OSD_ATTR_NUM_PART_TIMESTAMPS_ATTR_ACCESSED_TIME_size;
	OSD_PART_TIMESTAMPS_PAGE.attributes[2].attr_offset=OSD_ATTR_NUM_PART_TIMESTAMPS_ATTR_ACCESSED_TIME_offset;
	OSD_PART_TIMESTAMPS_PAGE.attributes[2].attr_set=OSD_ATTR_NUM_PART_TIMESTAMPS_ATTR_ACCESSED_TIME_set;

		
	OSD_PART_TIMESTAMPS_PAGE.attributes[3].attr_no=OSD_ATTR_NUM_PART_TIMESTAMPS_ATTR_MODIFIED_TIME;
	OSD_PART_TIMESTAMPS_PAGE.attributes[3].attr_size=OSD_ATTR_NUM_PART_TIMESTAMPS_ATTR_MODIFIED_TIME_size;
	OSD_PART_TIMESTAMPS_PAGE.attributes[3].attr_offset=OSD_ATTR_NUM_PART_TIMESTAMPS_ATTR_MODIFIED_TIME_offset;
	OSD_PART_TIMESTAMPS_PAGE.attributes[3].attr_set=OSD_ATTR_NUM_PART_TIMESTAMPS_ATTR_MODIFIED_TIME_set;
	
	OSD_PART_TIMESTAMPS_PAGE.attributes[4].attr_no=OSD_ATTR_NUM_PART_TIMESTAMPS_DATA_ACCESSED_TIME;
	OSD_PART_TIMESTAMPS_PAGE.attributes[4].attr_size=OSD_ATTR_NUM_PART_TIMESTAMPS_DATA_ACCESSED_TIME_size;
	OSD_PART_TIMESTAMPS_PAGE.attributes[4].attr_offset=OSD_ATTR_NUM_PART_TIMESTAMPS_DATA_ACCESSED_TIME_offset;
	OSD_PART_TIMESTAMPS_PAGE.attributes[4].attr_set=OSD_ATTR_NUM_PART_TIMESTAMPS_DATA_ACCESSED_TIME_set;
	
	OSD_PART_TIMESTAMPS_PAGE.attributes[5].attr_no=OSD_ATTR_NUM_PART_TIMESTAMPS_DATA_MODIFIED_TIME;
	OSD_PART_TIMESTAMPS_PAGE.attributes[5].attr_size=OSD_ATTR_NUM_PART_TIMESTAMPS_DATA_MODIFIED_TIME_size;
	OSD_PART_TIMESTAMPS_PAGE.attributes[5].attr_offset=OSD_ATTR_NUM_PART_TIMESTAMPS_DATA_MODIFIED_TIME_offset;
	OSD_PART_TIMESTAMPS_PAGE.attributes[5].attr_set=OSD_ATTR_NUM_PART_TIMESTAMPS_DATA_MODIFIED_TIME_set;
	
	OSD_PART_TIMESTAMPS_PAGE.attributes[6].attr_no=OSD_ATTR_NUM_PART_TIMESTAMPS_TIMESTAMP_BYPASS;
	OSD_PART_TIMESTAMPS_PAGE.attributes[6].attr_size=OSD_ATTR_NUM_PART_TIMESTAMPS_TIMESTAMP_BYPASS_size;
	OSD_PART_TIMESTAMPS_PAGE.attributes[6].attr_offset=OSD_ATTR_NUM_PART_TIMESTAMPS_TIMESTAMP_BYPASS_offset;
	OSD_PART_TIMESTAMPS_PAGE.attributes[6].attr_set=OSD_ATTR_NUM_PART_TIMESTAMPS_TIMESTAMP_BYPASS_set;
	
	
	
	

		
}

//--------------------------------------------------------------------------------------------------

void initialize_partition_security_page_struct()
{
 int i;

	OSD_PART_SECURITY_PAGE.attr_count= OSD_PAGE_NUM_PART_SECURITY_ATTR_COUNT;
	OSD_PART_SECURITY_PAGE.attributes=(OSD_ATTR_INFO *)allocate(sizeof(OSD_ATTR_INFO)*OSD_PAGE_NUM_PART_SECURITY_ATTR_COUNT);

	
	OSD_PART_SECURITY_PAGE.attributes[0].attr_no=OSD_ATTR_NUM_PART_SECURITY_page_id;
	OSD_PART_SECURITY_PAGE.attributes[0].attr_size=PAGE_ID_ATTR_SIZE;
	OSD_PART_SECURITY_PAGE.attributes[0].attr_offset=PAGE_ID_ATTR_OFFSET	;
	OSD_PART_SECURITY_PAGE.attributes[0].attr_set=0	;
	
	OSD_PART_SECURITY_PAGE.attributes[1].attr_no=OSD_ATTR_NUM_PART_SECURITY_DEFAULT_SEC_METHOD;
	OSD_PART_SECURITY_PAGE.attributes[1].attr_size=OSD_ATTR_NUM_PART_SECURITY_DEFAULT_SEC_METHOD_size;
	OSD_PART_SECURITY_PAGE.attributes[1].attr_offset=OSD_ATTR_NUM_PART_SECURITY_DEFAULT_SEC_METHOD_offset	;
	OSD_PART_SECURITY_PAGE.attributes[1].attr_set=OSD_ATTR_NUM_PART_SECURITY_DEFAULT_SEC_METHOD_set	;
	
	OSD_PART_SECURITY_PAGE.attributes[2].attr_no=OSD_ATTR_NUM_PART_SECURITY_OLDEST_VALID_NONCE;
	OSD_PART_SECURITY_PAGE.attributes[2].attr_size=OSD_ATTR_NUM_PART_SECURITY_OLDEST_VALID_NONCE_size;
	OSD_PART_SECURITY_PAGE.attributes[2].attr_offset=OSD_ATTR_NUM_PART_SECURITY_OLDEST_VALID_NONCE_offset	;
	OSD_PART_SECURITY_PAGE.attributes[2].attr_set=OSD_ATTR_NUM_PART_SECURITY_OLDEST_VALID_NONCE_set	;
	
	
	
	OSD_PART_SECURITY_PAGE.attributes[3].attr_no=OSD_ATTR_NUM_PART_SECURITY_NEWEST_VALID_NONCE;
	OSD_PART_SECURITY_PAGE.attributes[3].attr_size=OSD_ATTR_NUM_PART_SECURITY_NEWEST_VALID_NONCE_size;
	OSD_PART_SECURITY_PAGE.attributes[3].attr_offset=OSD_ATTR_NUM_PART_SECURITY_NEWEST_VALID_NONCE_offset	;
	OSD_PART_SECURITY_PAGE.attributes[3].attr_set=OSD_ATTR_NUM_PART_SECURITY_NEWEST_VALID_NONCE_set	;
	
	
	
	OSD_PART_SECURITY_PAGE.attributes[4].attr_no=OSD_ATTR_NUM_PART_SECURITY_REQ_NONCE_LIST_DEPTH;
	OSD_PART_SECURITY_PAGE.attributes[4].attr_size=OSD_ATTR_NUM_PART_SECURITY_REQ_NONCE_LIST_DEPTH_size;
	OSD_PART_SECURITY_PAGE.attributes[4].attr_offset=OSD_ATTR_NUM_PART_SECURITY_REQ_NONCE_LIST_DEPTH_offset;
	OSD_PART_SECURITY_PAGE.attributes[4].attr_set=OSD_ATTR_NUM_PART_SECURITY_REQ_NONCE_LIST_DEPTH_set;
	
	
	
	OSD_PART_SECURITY_PAGE.attributes[5].attr_no=OSD_ATTR_NUM_PART_SECURITY_FROZEN_KEYS_MASK;
	OSD_PART_SECURITY_PAGE.attributes[5].attr_size=OSD_ATTR_NUM_PART_SECURITY_FROZEN_KEYS_MASK_size;
	OSD_PART_SECURITY_PAGE.attributes[5].attr_offset=OSD_ATTR_NUM_PART_SECURITY_FROZEN_KEYS_MASK_offset;	
	OSD_PART_SECURITY_PAGE.attributes[5].attr_set=OSD_ATTR_NUM_PART_SECURITY_FROZEN_KEYS_MASK_set;	
	
	
	
	OSD_PART_SECURITY_PAGE.attributes[6].attr_no=OSD_ATTR_NUM_PART_SECURITY_PARTITION_KEY_ID;
	OSD_PART_SECURITY_PAGE.attributes[6].attr_size=OSD_ATTR_NUM_PART_SECURITY_PARTITION_KEY_ID_size;
	OSD_PART_SECURITY_PAGE.attributes[6].attr_offset=OSD_ATTR_NUM_PART_SECURITY_PARTITION_KEY_ID_offset;	
	OSD_PART_SECURITY_PAGE.attributes[6].attr_set=OSD_ATTR_NUM_PART_SECURITY_PARTITION_KEY_ID_set;	

	
	for(i=0;i<16;i++)
	{	
	OSD_PART_SECURITY_PAGE.attributes[i+7].attr_no=OSD_ATTR_NUM_PART_SECURITY_WORKING_KEY_ID_0+i;
	OSD_PART_SECURITY_PAGE.attributes[i+7].attr_size=7;
	OSD_PART_SECURITY_PAGE.attributes[i+7].attr_offset=OSD_ATTR_NUM_PART_SECURITY_WORKING_KEY_ID_0_offset+i*7;	
	OSD_PART_SECURITY_PAGE.attributes[i+7].attr_set=
	OSD_ATTR_NUM_PART_SECURITY_WORKING_KEY_ID_0_set;
	}
		
	
	OSD_PART_SECURITY_PAGE.attributes[23].attr_no=OSD_ATTR_NUM_PART_SECURITY_POLICY_ACCESS_TAG;
	OSD_PART_SECURITY_PAGE.attributes[23].attr_size=OSD_ATTR_NUM_PART_SECURITY_POLICY_ACCESS_TAG_size;
	OSD_PART_SECURITY_PAGE.attributes[23].attr_offset=OSD_ATTR_NUM_PART_SECURITY_POLICY_ACCESS_TAG_offset;
	OSD_PART_SECURITY_PAGE.attributes[23].attr_set=OSD_ATTR_NUM_PART_SECURITY_POLICY_ACCESS_TAG_set;

	
		
	OSD_PART_SECURITY_PAGE.attributes[24].attr_no=OSD_ATTR_NUM_PART_SECURITY_USER_OBJ_POLICY_ACCESS_TAG;
	OSD_PART_SECURITY_PAGE.attributes[24].attr_size=OSD_ATTR_NUM_PART_SECURITY_USER_OBJ_POLICY_ACCESS_TAG_size;
	OSD_PART_SECURITY_PAGE.attributes[24].attr_offset=OSD_ATTR_NUM_PART_SECURITY_USER_OBJ_POLICY_ACCESS_TAG_offset;
	OSD_PART_SECURITY_PAGE.attributes[24].attr_set=OSD_ATTR_NUM_PART_SECURITY_USER_OBJ_POLICY_ACCESS_TAG_set;

}
//------------------------------------------------------------------------------------------
	
void initialize_uobject_directory_page_struct()
{
	OSD_UOBJ_DIRECTORY_PAGE.attr_count= OSD_PAGE_NUM_UOBJ_DIRECTORY_ATTR_COUNT;
	OSD_UOBJ_DIRECTORY_PAGE.attributes=(OSD_ATTR_INFO *)allocate(sizeof(OSD_ATTR_INFO)*OSD_PAGE_NUM_UOBJ_DIRECTORY_ATTR_COUNT);

	OSD_UOBJ_DIRECTORY_PAGE.attributes[0].attr_no=OSD_ATTR_NUM_UOBJ_DIRECTORY_page_id;
	OSD_UOBJ_DIRECTORY_PAGE.attributes[0].attr_size=OSD_ATTR_NUM_UOBJ_DIRECTORY_page_id_size;
	OSD_UOBJ_DIRECTORY_PAGE.attributes[0].attr_offset=OSD_ATTR_NUM_UOBJ_DIRECTORY_page_id_offset;
	OSD_UOBJ_DIRECTORY_PAGE.attributes[0].attr_set=OSD_ATTR_NUM_UOBJ_DIRECTORY_page_id_set;

	OSD_UOBJ_DIRECTORY_PAGE.attributes[1].attr_no=OSD_ATTR_NUM_UOBJ_DIRECTORY_A1;
	OSD_UOBJ_DIRECTORY_PAGE.attributes[1].attr_size=OSD_ATTR_NUM_UOBJ_DIRECTORY_A1_size;
	OSD_UOBJ_DIRECTORY_PAGE.attributes[1].attr_offset=OSD_ATTR_NUM_UOBJ_DIRECTORY_A1_offset;
	OSD_UOBJ_DIRECTORY_PAGE.attributes[1].attr_set=OSD_ATTR_NUM_UOBJ_DIRECTORY_A1_set;

		
	OSD_UOBJ_DIRECTORY_PAGE.attributes[2].attr_no=OSD_ATTR_NUM_UOBJ_DIRECTORY_A2;
	OSD_UOBJ_DIRECTORY_PAGE.attributes[2].attr_size=OSD_ATTR_NUM_UOBJ_DIRECTORY_A2_size;
	OSD_UOBJ_DIRECTORY_PAGE.attributes[2].attr_offset=OSD_ATTR_NUM_UOBJ_DIRECTORY_A2_offset;
	OSD_UOBJ_DIRECTORY_PAGE.attributes[2].attr_set=OSD_ATTR_NUM_UOBJ_DIRECTORY_A2_set;
	
	
	
	OSD_UOBJ_DIRECTORY_PAGE.attributes[3].attr_no=OSD_ATTR_NUM_UOBJ_DIRECTORY_A3;
	OSD_UOBJ_DIRECTORY_PAGE.attributes[3].attr_size=OSD_ATTR_NUM_UOBJ_DIRECTORY_A3_size;
	OSD_UOBJ_DIRECTORY_PAGE.attributes[3].attr_offset=OSD_ATTR_NUM_UOBJ_DIRECTORY_A3_offset;
	OSD_UOBJ_DIRECTORY_PAGE.attributes[3].attr_set=OSD_ATTR_NUM_UOBJ_DIRECTORY_A3_set;
	
	
	
	OSD_UOBJ_DIRECTORY_PAGE.attributes[4].attr_no=OSD_ATTR_NUM_UOBJ_DIRECTORY_A4;
	OSD_UOBJ_DIRECTORY_PAGE.attributes[4].attr_size=OSD_ATTR_NUM_UOBJ_DIRECTORY_A4_size;
	OSD_UOBJ_DIRECTORY_PAGE.attributes[4].attr_offset=OSD_ATTR_NUM_UOBJ_DIRECTORY_A4_offset;
	OSD_UOBJ_DIRECTORY_PAGE.attributes[4].attr_set=OSD_ATTR_NUM_UOBJ_DIRECTORY_A4_set;
	
	
	OSD_UOBJ_DIRECTORY_PAGE.attributes[5].attr_no=OSD_ATTR_NUM_UOBJ_DIRECTORY_A5;
	OSD_UOBJ_DIRECTORY_PAGE.attributes[5].attr_size=OSD_ATTR_NUM_UOBJ_DIRECTORY_A5_size;
	OSD_UOBJ_DIRECTORY_PAGE.attributes[5].attr_offset=OSD_ATTR_NUM_UOBJ_DIRECTORY_A5_offset;
	OSD_UOBJ_DIRECTORY_PAGE.attributes[5].attr_set=OSD_ATTR_NUM_UOBJ_DIRECTORY_A5_set;
}
//-----------------------------------------------------------------------------------------
void initialize_uobject_info_page_struct()
{
	
	OSD_UOBJ_INFO_PAGE.attr_count= OSD_PAGE_NUM_UOBJ_INFO_ATTR_COUNT;
	OSD_UOBJ_INFO_PAGE.attributes=(OSD_ATTR_INFO *)allocate(sizeof(OSD_ATTR_INFO)*OSD_PAGE_NUM_UOBJ_INFO_ATTR_COUNT);

	OSD_UOBJ_INFO_PAGE.attributes[0].attr_no= OSD_ATTR_NUM_UOBJ_INFO_page_id;
	OSD_UOBJ_INFO_PAGE.attributes[0].attr_size= PAGE_ID_ATTR_SIZE;
	OSD_UOBJ_INFO_PAGE.attributes[0].attr_offset= PAGE_ID_ATTR_OFFSET;
	OSD_UOBJ_INFO_PAGE.attributes[0].attr_set= 0;
	
	
	OSD_UOBJ_INFO_PAGE.attributes[1].attr_no=OSD_ATTR_NUM_UOBJ_INFO_PARTITION_ID ;
	OSD_UOBJ_INFO_PAGE.attributes[1].attr_size=OSD_ATTR_NUM_UOBJ_INFO_PARTITION_ID_size ;
	OSD_UOBJ_INFO_PAGE.attributes[1].attr_offset=OSD_ATTR_NUM_UOBJ_INFO_PARTITION_ID_offset ;
	OSD_UOBJ_INFO_PAGE.attributes[1].attr_set=OSD_ATTR_NUM_UOBJ_INFO_PARTITION_ID_set ;

	
	OSD_UOBJ_INFO_PAGE.attributes[2].attr_no= OSD_ATTR_NUM_UOBJ_INFO_OBJECT_ID;
	OSD_UOBJ_INFO_PAGE.attributes[2].attr_size= OSD_ATTR_NUM_UOBJ_INFO_OBJECT_ID_size;
	OSD_UOBJ_INFO_PAGE.attributes[2].attr_offset= OSD_ATTR_NUM_UOBJ_INFO_OBJECT_ID_offset;
	OSD_UOBJ_INFO_PAGE.attributes[2].attr_set= OSD_ATTR_NUM_UOBJ_INFO_OBJECT_ID_set;
	
	

	OSD_UOBJ_INFO_PAGE.attributes[3].attr_no= OSD_ATTR_NUM_UOBJ_INFO_USERNAME;
	OSD_UOBJ_INFO_PAGE.attributes[3].attr_size= OSD_ATTR_NUM_UOBJ_INFO_USERNAME_size;
	OSD_UOBJ_INFO_PAGE.attributes[3].attr_offset=OSD_ATTR_NUM_UOBJ_INFO_USERNAME_offset ;
	OSD_UOBJ_INFO_PAGE.attributes[3].attr_set=OSD_ATTR_NUM_UOBJ_INFO_USERNAME_set ;

	
	OSD_UOBJ_INFO_PAGE.attributes[4].attr_no= OSD_ATTR_NUM_UOBJ_INFO_USED_CAPACITY;
	OSD_UOBJ_INFO_PAGE.attributes[4].attr_size= OSD_ATTR_NUM_UOBJ_INFO_USED_CAPACITY_size;
	OSD_UOBJ_INFO_PAGE.attributes[4].attr_offset= OSD_ATTR_NUM_UOBJ_INFO_USED_CAPACITY_offset;
	OSD_UOBJ_INFO_PAGE.attributes[4].attr_set= OSD_ATTR_NUM_UOBJ_INFO_USED_CAPACITY_set;

	
	OSD_UOBJ_INFO_PAGE.attributes[5].attr_no=OSD_ATTR_NUM_UOBJ_INFO_OBJ_LENGTH ;
	OSD_UOBJ_INFO_PAGE.attributes[5].attr_size=OSD_ATTR_NUM_UOBJ_INFO_OBJ_LENGTH_size ;
	OSD_UOBJ_INFO_PAGE.attributes[5].attr_offset=OSD_ATTR_NUM_UOBJ_INFO_OBJ_LENGTH_offset ;
	OSD_UOBJ_INFO_PAGE.attributes[5].attr_set=OSD_ATTR_NUM_UOBJ_INFO_OBJ_LENGTH_set ;
	
	



}
//-----------------------------------------------------------------------------------------
	
void initialize_uobject_quotas_page_struct()
{
	
	
	OSD_UOBJ_QUOTAS_PAGE.attr_count= OSD_PAGE_NUM_UOBJ_QUOTAS_ATTR_COUNT;
	OSD_UOBJ_QUOTAS_PAGE.attributes=(OSD_ATTR_INFO *)allocate(sizeof(OSD_ATTR_INFO)*OSD_PAGE_NUM_UOBJ_QUOTAS_ATTR_COUNT);

	OSD_UOBJ_QUOTAS_PAGE.attributes[0].attr_no= OSD_ATTR_NUM_UOBJ_QUOTAS_page_id;
	OSD_UOBJ_QUOTAS_PAGE.attributes[0].attr_size=PAGE_ID_ATTR_SIZE;
	OSD_UOBJ_QUOTAS_PAGE.attributes[0].attr_offset=PAGE_ID_ATTR_OFFSET;
	OSD_UOBJ_QUOTAS_PAGE.attributes[0].attr_set=0;
	
	OSD_UOBJ_QUOTAS_PAGE.attributes[1].attr_no= OSD_ATTR_NUM_UOBJ_QUOTAS_MAX_OBJ_LEN;
	OSD_UOBJ_QUOTAS_PAGE.attributes[1].attr_size= OSD_ATTR_NUM_UOBJ_QUOTAS_MAX_OBJ_LEN_size;
	OSD_UOBJ_QUOTAS_PAGE.attributes[1].attr_offset=OSD_ATTR_NUM_UOBJ_QUOTAS_MAX_OBJ_LEN_offset ;
	OSD_UOBJ_QUOTAS_PAGE.attributes[1].attr_set=OSD_ATTR_NUM_UOBJ_QUOTAS_MAX_OBJ_LEN_set ;
	
	
	
	
	
}
//------------------------------------------------------------------------------------------	
void initialize_uobject_timestamps_page_struct()
{


	OSD_UOBJ_TIMESTAMPS_PAGE.attr_count= OSD_PAGE_NUM_UOBJ_TIMESTAMPS_ATTR_COUNT;
	OSD_UOBJ_TIMESTAMPS_PAGE.attributes=(OSD_ATTR_INFO *)allocate(sizeof(OSD_ATTR_INFO)*OSD_PAGE_NUM_UOBJ_TIMESTAMPS_ATTR_COUNT);

	OSD_UOBJ_TIMESTAMPS_PAGE.attributes[0].attr_no= OSD_ATTR_NUM_UOBJ_TIMESTAMPS_page_id;
	OSD_UOBJ_TIMESTAMPS_PAGE.attributes[0].attr_size= PAGE_ID_ATTR_SIZE;
	OSD_UOBJ_TIMESTAMPS_PAGE.attributes[0].attr_offset=PAGE_ID_ATTR_OFFSET ;
	OSD_UOBJ_TIMESTAMPS_PAGE.attributes[0].attr_set=0;

	
	OSD_UOBJ_TIMESTAMPS_PAGE.attributes[1].attr_no= OSD_ATTR_NUM_UOBJ_TIMESTAMPS_CREATED_TIME;
	OSD_UOBJ_TIMESTAMPS_PAGE.attributes[1].attr_size= OSD_ATTR_NUM_UOBJ_TIMESTAMPS_CREATED_TIME_size;
	OSD_UOBJ_TIMESTAMPS_PAGE.attributes[1].attr_offset=OSD_ATTR_NUM_UOBJ_TIMESTAMPS_CREATED_TIME_offset;
	OSD_UOBJ_TIMESTAMPS_PAGE.attributes[1].attr_set=OSD_ATTR_NUM_UOBJ_TIMESTAMPS_CREATED_TIME_set;

	
	OSD_UOBJ_TIMESTAMPS_PAGE.attributes[2].attr_no= OSD_ATTR_NUM_UOBJ_TIMESTAMPS_ATTR_ACCESSED_TIME;
	OSD_UOBJ_TIMESTAMPS_PAGE.attributes[2].attr_size= OSD_ATTR_NUM_UOBJ_TIMESTAMPS_ATTR_ACCESSED_TIME_size;
	OSD_UOBJ_TIMESTAMPS_PAGE.attributes[2].attr_offset= OSD_ATTR_NUM_UOBJ_TIMESTAMPS_ATTR_ACCESSED_TIME_offset;
	OSD_UOBJ_TIMESTAMPS_PAGE.attributes[2].attr_set= OSD_ATTR_NUM_UOBJ_TIMESTAMPS_ATTR_ACCESSED_TIME_set;
	

	OSD_UOBJ_TIMESTAMPS_PAGE.attributes[3].attr_no= OSD_ATTR_NUM_UOBJ_TIMESTAMPS_ATTR_MODIFIED_TIME;
	OSD_UOBJ_TIMESTAMPS_PAGE.attributes[3].attr_size=OSD_ATTR_NUM_UOBJ_TIMESTAMPS_ATTR_MODIFIED_TIME_size ;
	OSD_UOBJ_TIMESTAMPS_PAGE.attributes[3].attr_offset= OSD_ATTR_NUM_UOBJ_TIMESTAMPS_ATTR_MODIFIED_TIME_offset;		
	OSD_UOBJ_TIMESTAMPS_PAGE.attributes[3].attr_set= OSD_ATTR_NUM_UOBJ_TIMESTAMPS_ATTR_MODIFIED_TIME_set;		
	
	
	
	OSD_UOBJ_TIMESTAMPS_PAGE.attributes[4].attr_no= OSD_ATTR_NUM_UOBJ_TIMESTAMPS_DATA_ACCESSED_TIME;
	OSD_UOBJ_TIMESTAMPS_PAGE.attributes[4].attr_size= OSD_ATTR_NUM_UOBJ_TIMESTAMPS_DATA_ACCESSED_TIME_size;
	OSD_UOBJ_TIMESTAMPS_PAGE.attributes[4].attr_offset= OSD_ATTR_NUM_UOBJ_TIMESTAMPS_DATA_ACCESSED_TIME_offset;
	OSD_UOBJ_TIMESTAMPS_PAGE.attributes[4].attr_set= OSD_ATTR_NUM_UOBJ_TIMESTAMPS_DATA_ACCESSED_TIME_set;

		
	OSD_UOBJ_TIMESTAMPS_PAGE.attributes[5].attr_no=OSD_ATTR_NUM_UOBJ_TIMESTAMPS_DATA_MODIFIED_TIME ;
	OSD_UOBJ_TIMESTAMPS_PAGE.attributes[5].attr_size= OSD_ATTR_NUM_UOBJ_TIMESTAMPS_DATA_MODIFIED_TIME_size;
	OSD_UOBJ_TIMESTAMPS_PAGE.attributes[5].attr_offset=OSD_ATTR_NUM_UOBJ_TIMESTAMPS_DATA_MODIFIED_TIME_offset ;
	OSD_UOBJ_TIMESTAMPS_PAGE.attributes[5].attr_set=OSD_ATTR_NUM_UOBJ_TIMESTAMPS_DATA_MODIFIED_TIME_set ;
	
	
	

}
//------------------------------------------------------------------------------------------
		
void initialize_uobject_security_page_struct()
{
	OSD_UOBJ_SECURITY_PAGE.attr_count= OSD_PAGE_NUM_UOBJ_SECURITY_ATTR_COUNT;
	OSD_UOBJ_SECURITY_PAGE.attributes=(OSD_ATTR_INFO *)allocate(sizeof(OSD_ATTR_INFO)*OSD_PAGE_NUM_UOBJ_SECURITY_ATTR_COUNT);


	OSD_UOBJ_SECURITY_PAGE.attributes[0].attr_no= OSD_ATTR_NUM_UOBJ_SECURITY_page_id;
	OSD_UOBJ_SECURITY_PAGE.attributes[0].attr_size= PAGE_ID_ATTR_SIZE;
	OSD_UOBJ_SECURITY_PAGE.attributes[0].attr_offset= PAGE_ID_ATTR_OFFSET;
	OSD_UOBJ_SECURITY_PAGE.attributes[0].attr_set= 0;

	OSD_UOBJ_SECURITY_PAGE.attributes[1].attr_no= OSD_ATTR_NUM_UOBJ_SECURITY_POLICY_ACCESS_TAG;
	OSD_UOBJ_SECURITY_PAGE.attributes[1].attr_size=OSD_ATTR_NUM_UOBJ_SECURITY_POLICY_ACCESS_TAG_size ;
	OSD_UOBJ_SECURITY_PAGE.attributes[1].attr_offset=OSD_ATTR_NUM_UOBJ_SECURITY_POLICY_ACCESS_TAG_offset ;
	OSD_UOBJ_SECURITY_PAGE.attributes[1].attr_set=OSD_ATTR_NUM_UOBJ_SECURITY_POLICY_ACCESS_TAG_set ;
	

}

void initialize_uobject_collections_page_struct()
{


	OSD_UOBJ_COLLECTIONS_PAGE.attr_count= OSD_PAGE_NUM_UOBJ_COLLECTIONS_ATTR_COUNT;
	OSD_UOBJ_COLLECTIONS_PAGE.attributes=(OSD_ATTR_INFO *)allocate(sizeof(OSD_ATTR_INFO)*OSD_PAGE_NUM_UOBJ_COLLECTIONS_ATTR_COUNT);


	OSD_UOBJ_COLLECTIONS_PAGE.attributes[0].attr_no=OSD_ATTR_NUM_T10_COLLECTIONS_page_id;
	OSD_UOBJ_COLLECTIONS_PAGE.attributes[0].attr_size= PAGE_ID_ATTR_SIZE;
	OSD_UOBJ_COLLECTIONS_PAGE.attributes[0].attr_offset= PAGE_ID_ATTR_OFFSET;
	OSD_UOBJ_COLLECTIONS_PAGE.attributes[0].attr_set=0; 

}

//--------------------------------------------------------------------------------------------------

void initialize_collection_directory_page_struct()
{
	
	OSD_COLL_DIRECTORY_PAGE.attr_count= OSD_PAGE_NUM_COLL_DIRECTORY_ATTR_COUNT;
	OSD_COLL_DIRECTORY_PAGE.attributes=(OSD_ATTR_INFO *)allocate(sizeof(OSD_ATTR_INFO)*OSD_PAGE_NUM_COLL_DIRECTORY_ATTR_COUNT);

	OSD_COLL_DIRECTORY_PAGE.attributes[0].attr_no=OSD_ATTR_NUM_COLL_DIRECTORY_page_id;
	OSD_COLL_DIRECTORY_PAGE.attributes[0].attr_size=OSD_ATTR_NUM_COLL_DIRECTORY_page_id_size;
	OSD_COLL_DIRECTORY_PAGE.attributes[0].attr_offset=OSD_ATTR_NUM_COLL_DIRECTORY_page_id_offset;
	OSD_COLL_DIRECTORY_PAGE.attributes[0].attr_set=OSD_ATTR_NUM_COLL_DIRECTORY_page_id_set;
	
	
	
	OSD_COLL_DIRECTORY_PAGE.attributes[1].attr_no=OSD_ATTR_NUM_COLL_DIRECTORY_A1;
	OSD_COLL_DIRECTORY_PAGE.attributes[1].attr_size=OSD_ATTR_NUM_COLL_DIRECTORY_A1_size;
	OSD_COLL_DIRECTORY_PAGE.attributes[1].attr_offset=OSD_ATTR_NUM_COLL_DIRECTORY_A1_offset;
	OSD_COLL_DIRECTORY_PAGE.attributes[1].attr_set=OSD_ATTR_NUM_COLL_DIRECTORY_A1_set;

	
	OSD_COLL_DIRECTORY_PAGE.attributes[2].attr_no=OSD_ATTR_NUM_COLL_DIRECTORY_A3;
	OSD_COLL_DIRECTORY_PAGE.attributes[2].attr_size=OSD_ATTR_NUM_COLL_DIRECTORY_A3_size;
	OSD_COLL_DIRECTORY_PAGE.attributes[2].attr_offset=OSD_ATTR_NUM_COLL_DIRECTORY_A3_offset;
	OSD_COLL_DIRECTORY_PAGE.attributes[2].attr_set=OSD_ATTR_NUM_COLL_DIRECTORY_A3_set;

		
	OSD_COLL_DIRECTORY_PAGE.attributes[3].attr_no=OSD_ATTR_NUM_COLL_DIRECTORY_A5;
	OSD_COLL_DIRECTORY_PAGE.attributes[3].attr_size=OSD_ATTR_NUM_COLL_DIRECTORY_A5_size;
	OSD_COLL_DIRECTORY_PAGE.attributes[3].attr_offset=OSD_ATTR_NUM_COLL_DIRECTORY_A5_offset;
	OSD_COLL_DIRECTORY_PAGE.attributes[3].attr_set=OSD_ATTR_NUM_COLL_DIRECTORY_A5_set;

	
}
//--------------------------------------------------------------------------------------------------
void initialize_collection_info_page_struct()
{
	OSD_COLL_INFO_PAGE.attr_count= OSD_PAGE_NUM_COLL_INFO_ATTR_COUNT;
	OSD_COLL_INFO_PAGE.attributes=(OSD_ATTR_INFO *)allocate(sizeof(OSD_ATTR_INFO)*OSD_PAGE_NUM_COLL_INFO_ATTR_COUNT);

	OSD_COLL_INFO_PAGE.attributes[0].attr_no= OSD_ATTR_NUM_COLL_INFO_page_id;
	OSD_COLL_INFO_PAGE.attributes[0].attr_size= PAGE_ID_ATTR_SIZE;
	OSD_COLL_INFO_PAGE.attributes[0].attr_offset= PAGE_ID_ATTR_OFFSET;
	OSD_COLL_INFO_PAGE.attributes[0].attr_set= 0;
	

	OSD_COLL_INFO_PAGE.attributes[1].attr_no= OSD_ATTR_NUM_COLL_INFO_PARTITION_ID;
	OSD_COLL_INFO_PAGE.attributes[1].attr_size= OSD_ATTR_NUM_COLL_INFO_PARTITION_ID_size;
	OSD_COLL_INFO_PAGE.attributes[1].attr_offset=OSD_ATTR_NUM_COLL_INFO_PARTITION_ID_offset ;
	OSD_COLL_INFO_PAGE.attributes[1].attr_set=OSD_ATTR_NUM_COLL_INFO_PARTITION_ID_set ;

	
	OSD_COLL_INFO_PAGE.attributes[2].attr_no= OSD_ATTR_NUM_COLL_INFO_OBJECT_ID;
	OSD_COLL_INFO_PAGE.attributes[2].attr_size= OSD_ATTR_NUM_COLL_INFO_OBJECT_ID_size;
	OSD_COLL_INFO_PAGE.attributes[2].attr_offset= OSD_ATTR_NUM_COLL_INFO_OBJECT_ID_offset;
	OSD_COLL_INFO_PAGE.attributes[2].attr_set= OSD_ATTR_NUM_COLL_INFO_OBJECT_ID_set;

	
	OSD_COLL_INFO_PAGE.attributes[3].attr_no= OSD_ATTR_NUM_COLL_INFO_USERNAME;
	OSD_COLL_INFO_PAGE.attributes[3].attr_size= OSD_ATTR_NUM_COLL_INFO_USERNAME_size;
	OSD_COLL_INFO_PAGE.attributes[3].attr_offset= OSD_ATTR_NUM_COLL_INFO_USERNAME_offset;
	OSD_COLL_INFO_PAGE.attributes[3].attr_set= OSD_ATTR_NUM_COLL_INFO_USERNAME_set;

	
	OSD_COLL_INFO_PAGE.attributes[4].attr_no= OSD_ATTR_NUM_COLL_INFO_USED_CAPACITY;
	OSD_COLL_INFO_PAGE.attributes[4].attr_size= OSD_ATTR_NUM_COLL_INFO_USED_CAPACITY_size;
	OSD_COLL_INFO_PAGE.attributes[4].attr_offset= OSD_ATTR_NUM_COLL_INFO_USED_CAPACITY_offset;
	OSD_COLL_INFO_PAGE.attributes[4].attr_set= OSD_ATTR_NUM_COLL_INFO_USED_CAPACITY_set;

	


}
//--------------------------------------------------------------------------------------------------
void initialize_collection_timestamps_page_struct()
{

	OSD_COLL_TIMESTAMPS_PAGE.attr_count= OSD_PAGE_NUM_COLL_TIMESTAMPS_ATTR_COUNT;
	OSD_COLL_TIMESTAMPS_PAGE.attributes=(OSD_ATTR_INFO *)allocate(sizeof(OSD_ATTR_INFO)*OSD_PAGE_NUM_COLL_TIMESTAMPS_ATTR_COUNT);

	OSD_COLL_TIMESTAMPS_PAGE.attributes[0].attr_no= OSD_ATTR_NUM_COLL_TIMESTAMPS_page_id;
	OSD_COLL_TIMESTAMPS_PAGE.attributes[0].attr_size= PAGE_ID_ATTR_SIZE;
	OSD_COLL_TIMESTAMPS_PAGE.attributes[0].attr_offset= PAGE_ID_ATTR_OFFSET;
	OSD_COLL_TIMESTAMPS_PAGE.attributes[0].attr_set= 0;


	OSD_COLL_TIMESTAMPS_PAGE.attributes[1].attr_no= OSD_ATTR_NUM_COLL_TIMESTAMPS_CREATED_TIME;
	OSD_COLL_TIMESTAMPS_PAGE.attributes[1].attr_size= OSD_ATTR_NUM_COLL_TIMESTAMPS_CREATED_TIME_size;
	OSD_COLL_TIMESTAMPS_PAGE.attributes[1].attr_offset=OSD_ATTR_NUM_COLL_TIMESTAMPS_CREATED_TIME_offset ;
	OSD_COLL_TIMESTAMPS_PAGE.attributes[1].attr_set=OSD_ATTR_NUM_COLL_TIMESTAMPS_CREATED_TIME_set ;


	
	OSD_COLL_TIMESTAMPS_PAGE.attributes[2].attr_no= OSD_ATTR_NUM_COLL_TIMESTAMPS_ATTR_ACCESSED_TIME;
	OSD_COLL_TIMESTAMPS_PAGE.attributes[2].attr_size= OSD_ATTR_NUM_COLL_TIMESTAMPS_ATTR_ACCESSED_TIME_size;
	OSD_COLL_TIMESTAMPS_PAGE.attributes[2].attr_offset= OSD_ATTR_NUM_COLL_TIMESTAMPS_ATTR_ACCESSED_TIME_offset;
	OSD_COLL_TIMESTAMPS_PAGE.attributes[2].attr_set= OSD_ATTR_NUM_COLL_TIMESTAMPS_ATTR_ACCESSED_TIME_set;

	

	OSD_COLL_TIMESTAMPS_PAGE.attributes[3].attr_no= OSD_ATTR_NUM_COLL_TIMESTAMPS_ATTR_MODIFIED_TIME;
	OSD_COLL_TIMESTAMPS_PAGE.attributes[3].attr_size= OSD_ATTR_NUM_COLL_TIMESTAMPS_ATTR_MODIFIED_TIME_size;
	OSD_COLL_TIMESTAMPS_PAGE.attributes[3].attr_offset= OSD_ATTR_NUM_COLL_TIMESTAMPS_ATTR_MODIFIED_TIME_offset;
	OSD_COLL_TIMESTAMPS_PAGE.attributes[3].attr_set= OSD_ATTR_NUM_COLL_TIMESTAMPS_ATTR_MODIFIED_TIME_set;

	

	OSD_COLL_TIMESTAMPS_PAGE.attributes[4].attr_no= OSD_ATTR_NUM_COLL_TIMESTAMPS_DATA_ACCESSED_TIME;
	OSD_COLL_TIMESTAMPS_PAGE.attributes[4].attr_size= OSD_ATTR_NUM_COLL_TIMESTAMPS_DATA_ACCESSED_TIME_size;
	OSD_COLL_TIMESTAMPS_PAGE.attributes[4].attr_offset=OSD_ATTR_NUM_COLL_TIMESTAMPS_DATA_ACCESSED_TIME_offset ;
	OSD_COLL_TIMESTAMPS_PAGE.attributes[4].attr_set=OSD_ATTR_NUM_COLL_TIMESTAMPS_DATA_ACCESSED_TIME_set ;

	

	OSD_COLL_TIMESTAMPS_PAGE.attributes[5].attr_no= OSD_ATTR_NUM_COLL_TIMESTAMPS_DATA_MODIFIED_TIME;
	OSD_COLL_TIMESTAMPS_PAGE.attributes[5].attr_size= OSD_ATTR_NUM_COLL_TIMESTAMPS_DATA_MODIFIED_TIME_size;
	OSD_COLL_TIMESTAMPS_PAGE.attributes[5].attr_offset= OSD_ATTR_NUM_COLL_TIMESTAMPS_DATA_MODIFIED_TIME_offset;
	OSD_COLL_TIMESTAMPS_PAGE.attributes[5].attr_set= OSD_ATTR_NUM_COLL_TIMESTAMPS_DATA_MODIFIED_TIME_set;



}
//--------------------------------------------------------------------------------------------------

void initialize_collection_security_page_struct()
{
	OSD_COLL_SECURITY_PAGE.attr_count= OSD_PAGE_NUM_COLL_SECURITY_ATTR_COUNT;
	OSD_COLL_SECURITY_PAGE.attributes=(OSD_ATTR_INFO *)allocate(sizeof(OSD_ATTR_INFO)*OSD_PAGE_NUM_COLL_SECURITY_ATTR_COUNT);

	OSD_COLL_SECURITY_PAGE.attributes[0].attr_no= OSD_ATTR_NUM_COLL_SECURITY_page_id;
	OSD_COLL_SECURITY_PAGE.attributes[0].attr_size= PAGE_ID_ATTR_SIZE;
	OSD_COLL_SECURITY_PAGE.attributes[0].attr_offset= PAGE_ID_ATTR_OFFSET;
	OSD_COLL_SECURITY_PAGE.attributes[0].attr_set= 0;

	
	OSD_COLL_SECURITY_PAGE.attributes[1].attr_no= OSD_ATTR_NUM_COLL_SECURITY_POLICY_ACCESS_TAG;
	OSD_COLL_SECURITY_PAGE.attributes[1].attr_size= OSD_ATTR_NUM_COLL_SECURITY_POLICY_ACCESS_TAG_size;
	OSD_COLL_SECURITY_PAGE.attributes[1].attr_offset= OSD_ATTR_NUM_COLL_SECURITY_POLICY_ACCESS_TAG_offset;
	OSD_COLL_SECURITY_PAGE.attributes[1].attr_set= OSD_ATTR_NUM_COLL_SECURITY_POLICY_ACCESS_TAG_set;

	

}
//--------------------------------------------------------------------------------------------------
void initialize_current_command_pages_struct()
{
	OSD_CUR_CMD_PAGE.attr_count= OSD_PAGE_NUM_CUR_CMD_ATTR_COUNT;
	OSD_CUR_CMD_PAGE.attributes=(OSD_ATTR_INFO *)allocate(sizeof(OSD_ATTR_INFO)*OSD_PAGE_NUM_CUR_CMD_ATTR_COUNT);
	
	OSD_CUR_CMD_PAGE.attributes[0].attr_no=OSD_ATTR_NUM_CUR_CMD_page_id;
	OSD_CUR_CMD_PAGE.attributes[0].attr_size=PAGE_ID_ATTR_SIZE;
	OSD_CUR_CMD_PAGE.attributes[0].attr_offset=PAGE_ID_ATTR_OFFSET;
	OSD_CUR_CMD_PAGE.attributes[0].attr_set=0;
	
	OSD_CUR_CMD_PAGE.attributes[1].attr_no=OSD_ATTR_NUM_CUR_CMD_RESPONSE_INT_CHECK_VAL;
	OSD_CUR_CMD_PAGE.attributes[1].attr_size=OSD_ATTR_NUM_CUR_CMD_RESPONSE_INT_CHECK_VAL_size;
	OSD_CUR_CMD_PAGE.attributes[1].attr_offset=OSD_ATTR_NUM_CUR_CMD_RESPONSE_INT_CHECK_VAL_offset;
	OSD_CUR_CMD_PAGE.attributes[1].attr_set=OSD_ATTR_NUM_CUR_CMD_RESPONSE_INT_CHECK_VAL_set;
	
	OSD_CUR_CMD_PAGE.attributes[2].attr_no=OSD_ATTR_NUM_CUR_CMD_OBJ_TYPE;
	OSD_CUR_CMD_PAGE.attributes[2].attr_size=OSD_ATTR_NUM_CUR_CMD_OBJ_TYPE_size;
	OSD_CUR_CMD_PAGE.attributes[2].attr_offset=OSD_ATTR_NUM_CUR_CMD_OBJ_TYPE_offset;
	OSD_CUR_CMD_PAGE.attributes[2].attr_set=OSD_ATTR_NUM_CUR_CMD_OBJ_TYPE_set;
	
	OSD_CUR_CMD_PAGE.attributes[3].attr_no=OSD_ATTR_NUM_CUR_CMD_PARTITION_ID;
	OSD_CUR_CMD_PAGE.attributes[3].attr_size=OSD_ATTR_NUM_CUR_CMD_PARTITION_ID_size;
	OSD_CUR_CMD_PAGE.attributes[3].attr_offset=OSD_ATTR_NUM_CUR_CMD_PARTITION_ID_offset;
	OSD_CUR_CMD_PAGE.attributes[3].attr_set=OSD_ATTR_NUM_CUR_CMD_PARTITION_ID_set;
	
	
	OSD_CUR_CMD_PAGE.attributes[4].attr_no=OSD_ATTR_NUM_CUR_CMD_OBJ_ID;
	OSD_CUR_CMD_PAGE.attributes[4].attr_size=OSD_ATTR_NUM_CUR_CMD_OBJ_ID_size;
	OSD_CUR_CMD_PAGE.attributes[4].attr_offset=OSD_ATTR_NUM_CUR_CMD_OBJ_ID_offset;
	OSD_CUR_CMD_PAGE.attributes[4].attr_set=OSD_ATTR_NUM_CUR_CMD_OBJ_ID_set;
	
	
	
	OSD_CUR_CMD_PAGE.attributes[5].attr_no=OSD_ATTR_NUM_CUR_CMD_APPEND_STARTING_BYTE;
	OSD_CUR_CMD_PAGE.attributes[5].attr_size=OSD_ATTR_NUM_CUR_CMD_APPEND_STARTING_BYTE_size;
	OSD_CUR_CMD_PAGE.attributes[5].attr_offset=OSD_ATTR_NUM_CUR_CMD_APPEND_STARTING_BYTE_offset;
	OSD_CUR_CMD_PAGE.attributes[5].attr_set=OSD_ATTR_NUM_CUR_CMD_APPEND_STARTING_BYTE_set;
	
}
//-----------------------------------------------------------------------------------------
