/**
 *                      ==================
 *                      IITK OSD Simulator
 *                      ==================
 *   An Implementation of Simulator for Object based storage device(OSD)
 *
 *   Copyright (C) 2007-2008 CSE IIT KANPUR
 *
 *   Contact: Department of Computer Science and Engineering
 *            Indian Institute of Technology,Kanpur(IIT Kanpur)
 *            {moona,riyaz}@cse.iitk.ac.in
 *
 *   This file is part of IITK OSD Simulator Project.
 *
 *   IITK OSD Simulator Project is free software; you can redistribute it
 *   and/or modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   IITK OSD Simulator Project is distributed in the hope that it will be
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the  GNU General
 *   Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#ifndef OSD_ATTR_H
#define OSD_ATTR_H

# include <stdint.h>
# include <string.h>
# include "osd_file_manip.h"
# include "osd_attr_numbers.h"
# include "osd_types.h"

/**
**********************************
 * File:  osd_attr.h
 * Declaration of Structure for Page storage 
 * Definition of  structure variable for each page
 * Functions to initialize structure variable associated with each page of each object type
 **/

# define MAX_ATTR_SIZE 100

/** Structure for information about give OSD attribute
*/
typedef struct 
{
 uint32_t attr_no;
 uint16_t attr_size;
 uint16_t attr_offset;
 uint8_t attr_set;
  
}OSD_ATTR_INFO;

/** Structure for maintaining attributes of given attribute page
*/
typedef struct 
{
  uint16_t attr_count;//
  OSD_ATTR_INFO *attributes;

}OSD_PAGE_ATTR_STORAGE_STRUCT;


/** Macros for Root attribute page  total no of attributes per page*/
# define OSD_PAGE_NUM_ROOT_DIRECTORY_ATTR_COUNT 5
# define OSD_PAGE_NUM_ROOT_INFO_ATTR_COUNT 12
# define OSD_PAGE_NUM_ROOT_QUOTAS_ATTR_COUNT 6
# define OSD_PAGE_NUM_ROOT_TIMESTAMPS_ATTR_COUNT 4
# define OSD_PAGE_NUM_ROOT_SECURITY_ATTR_COUNT 41 

/** Variables to intialize Root attribute pages */
OSD_PAGE_ATTR_STORAGE_STRUCT  OSD_ROOT_DIRECTORY_PAGE;
OSD_PAGE_ATTR_STORAGE_STRUCT  OSD_ROOT_INFO_PAGE;
OSD_PAGE_ATTR_STORAGE_STRUCT  OSD_ROOT_QUOTAS_PAGE ;
OSD_PAGE_ATTR_STORAGE_STRUCT  OSD_ROOT_TIMESTAMPS_PAGE ;
OSD_PAGE_ATTR_STORAGE_STRUCT  OSD_ROOT_SECURITY_PAGE;

/** Macros for partition attribute page total no of attributes per page*/
# define OSD_PAGE_NUM_PART_DIRECTORY_ATTR_COUNT 5
# define OSD_PAGE_NUM_PART_INFO_ATTR_COUNT 5
# define OSD_PAGE_NUM_PART_QUOTAS_ATTR_COUNT 5
# define OSD_PAGE_NUM_PART_TIMESTAMPS_ATTR_COUNT 7
# define OSD_PAGE_NUM_PART_SECURITY_ATTR_COUNT 25
/** Variables to intialize Partition attribute pages */
OSD_PAGE_ATTR_STORAGE_STRUCT  OSD_PART_DIRECTORY_PAGE;
OSD_PAGE_ATTR_STORAGE_STRUCT  OSD_PART_INFO_PAGE;
OSD_PAGE_ATTR_STORAGE_STRUCT  OSD_PART_QUOTAS_PAGE ;
OSD_PAGE_ATTR_STORAGE_STRUCT  OSD_PART_TIMESTAMPS_PAGE ;
OSD_PAGE_ATTR_STORAGE_STRUCT  OSD_PART_SECURITY_PAGE;

/** Macros for user object attribute page  total no of attributes per page*/
# define OSD_PAGE_NUM_UOBJ_DIRECTORY_ATTR_COUNT 6
# define OSD_PAGE_NUM_UOBJ_INFO_ATTR_COUNT 6
# define OSD_PAGE_NUM_UOBJ_QUOTAS_ATTR_COUNT 3
# define OSD_PAGE_NUM_UOBJ_TIMESTAMPS_ATTR_COUNT 6
# define OSD_PAGE_NUM_UOBJ_SECURITY_ATTR_COUNT 2
# define  OSD_PAGE_NUM_UOBJ_COLLECTIONS_ATTR_COUNT 1
/** Variables to intialize User object attribute pages */
OSD_PAGE_ATTR_STORAGE_STRUCT  OSD_UOBJ_DIRECTORY_PAGE;
OSD_PAGE_ATTR_STORAGE_STRUCT  OSD_UOBJ_INFO_PAGE;
OSD_PAGE_ATTR_STORAGE_STRUCT  OSD_UOBJ_QUOTAS_PAGE ;
OSD_PAGE_ATTR_STORAGE_STRUCT  OSD_UOBJ_TIMESTAMPS_PAGE ;
OSD_PAGE_ATTR_STORAGE_STRUCT  OSD_UOBJ_SECURITY_PAGE;
OSD_PAGE_ATTR_STORAGE_STRUCT  OSD_UOBJ_COLLECTIONS_PAGE;

/** Macros for collection attribute page  total no of attributes per page*/
# define OSD_PAGE_NUM_COLL_DIRECTORY_ATTR_COUNT 4
# define OSD_PAGE_NUM_COLL_INFO_ATTR_COUNT 5
# define OSD_PAGE_NUM_COLL_TIMESTAMPS_ATTR_COUNT 6
# define OSD_PAGE_NUM_COLL_SECURITY_ATTR_COUNT 2
/** Variables to intialize Collection object attribute pages */
OSD_PAGE_ATTR_STORAGE_STRUCT  OSD_COLL_DIRECTORY_PAGE;
OSD_PAGE_ATTR_STORAGE_STRUCT  OSD_COLL_INFO_PAGE;
OSD_PAGE_ATTR_STORAGE_STRUCT  OSD_COLL_TIMESTAMPS_PAGE ;
OSD_PAGE_ATTR_STORAGE_STRUCT  OSD_COLL_SECURITY_PAGE;

/** Macros for command attribute page */
# define OSD_PAGE_NUM_CUR_CMD_ATTR_COUNT 6
/** Variable to initialize Command attribute page */
OSD_PAGE_ATTR_STORAGE_STRUCT  OSD_CUR_CMD_PAGE;


/**
 *
 *    Functions:     initialize_root_directory_page_struct
 *		     initialize_root_info_page_struct
 *		     initialize_root_quotas_page_struct
 *		     initialize_root_timestamps_page_struct
 *		     initialize_root_security_page_struct()	
 *
 *    Description:  Initialize following root structure variables respectively 	
 *                  OSD_ROOT_DIRECTORY_PAGE - for Root Directory Page
 *		    OSD_ROOT_INFO_PAGE	- for Root Information Page
 *		    OSD_ROOT_QUOTAS_PAGE - for Root Quotas page
 *		    OSD_ROOT_TIMESTAMPS_PAGE - for Root Timestamps page	
 *		    OSD_ROOT_SECURITY_PAGE  - for Root Security Page		 *			
 **/
	void initialize_root_directory_page_struct();
	void initialize_root_info_page_struct();
	void initialize_root_quotas_page_struct();
	void initialize_root_timestamps_page_struct();
	void initialize_root_security_page_struct();
	


/**
 *
 *    Functions:     initialize_partition_directory_page_struct();
 *		     initialize_partition_info_page_struct();
 *		     initialize_partition_quotas_page_struct();
 *		     initialize_partition_timestamps_page_struct();
 *		     initialize_partition_security_page_struct();
 *
 *    Description:  Initialize following partition structure variables respectively 
 *	
 *                  OSD_PART_DIRECTORY_PAGE - for Part Directory Page
 *		    OSD_PART_INFO_PAGE	- for Part Information Page
 *		    OSD_PART_QUOTAS_PAGE - for Part Quotas page
 *		    OSD_PART_TIMESTAMPS_PAGE - for Part Timestamps page	
 *		    OSD_PART_SECURITY_PAGE  - for Part Security Page		 *			
 **/
	void initialize_partition_directory_page_struct();
	void initialize_partition_info_page_struct();
	void initialize_partition_quotas_page_struct();
	void initialize_partition_timestamps_page_struct();
	void initialize_partition_security_page_struct();

/**
 *
 *    Functions:     initialize_uobject_directory_page_struct();
 *		     initialize_uobject_info_page_struct();
 *		     initialize_uobject_quotas_page_struct();
 *		     initialize_uobject_timestamps_page_struct();
 *		     initialize_uobject_security_page_struct();
 *		     initialize_uobject_collections_page_struct();
 *
 *    Description:  Initialize following user obj structure variables respectively 
 *	
 *                  OSD_UOBJ_DIRECTORY_PAGE - for Uobj Directory Page
 *		    OSD_UOBJ_INFO_PAGE	- for Uobj Information Page
 *		    OSD_UOBJ_QUOTAS_PAGE - for Uobj Quotas page
 *		    OSD_UOBJ_TIMESTAMPS_PAGE - for Uobj Timestamps page	
 *		    OSD_UOBJ_SECURITY_PAGE  - for Uobj Security Page		
 *
 *			
 **/

	void initialize_uobject_directory_page_struct();
	void initialize_uobject_info_page_struct();
	void initialize_uobject_quotas_page_struct();
	void initialize_uobject_timestamps_page_struct();
	void initialize_uobject_security_page_struct();
	void initialize_uobject_collections_page_struct();
	

/**
 *
 *    Functions:     initialize_collection_directory_page_struct();
 *		     initialize_collection_info_page_struct();
 *		     initialize_collection_timestamps_page_struct();
 *		     initialize_collection_security_page_struct();
 *
 *    Description:  Initialize following Collecection structure variables 
 *		    respectively 
 *	
 *                  OSD_COLL_DIRECTORY_PAGE - for Coll Directory Page
 *		    OSD_COLL_INFO_PAGE	- for Coll Information Page
 *		    OSD_COLL_TIMESTAMPS_PAGE - for Coll Timestamps page	
 *		    OSD_COLL_SECURITY_PAGE  - for Coll Security Page		
 *
 *			
 **/

	void initialize_collection_directory_page_struct();
	void initialize_collection_info_page_struct();
	void initialize_collection_timestamps_page_struct();
	void initialize_collection_security_page_struct();

/**
 *
 *    Function:     initialize_current_command_pages_struct
 *
 *    Description:  Initialize OSD_CUR_CMD_PAGE structure 
 *
 **/
 	void initialize_current_command_pages_struct();	


/**
 *
 *    Function:     initialize_all_pages_struct
 *
 *    Description:  Call initialize page structur function for each page of ach 
* 		    object type
 *
 **/
void initialize_all_pages_struct();
//void free_all_pages_struct();
#endif	/* ifdef OSD_ATTR_H */
