/**
 *                      ==================
 *                      IITK OSD Simulator
 *                      ==================
 *   An Implementation of Simulator for Object based storage device(OSD)
 *
 *   Copyright (C) 2007-2008 CSE IIT KANPUR
 *
 *   Contact: Department of Computer Science and Engineering
 *            Indian Institute of Technology,Kanpur(IIT Kanpur)
 *            {moona,riyaz}@cse.iitk.ac.in
 *
 *   This file is part of IITK OSD Simulator Project.
 *
 *   IITK OSD Simulator Project is free software; you can redistribute it
 *   and/or modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   IITK OSD Simulator Project is distributed in the hope that it will be
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the  GNU General
 *   Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
# include "osd_initial_values.h"
# include "osd_time.h"
# include "osd_attr_manip.h"

/**
**********************************
 * File:  osd_attr1.c
 * Implements Functions to initialize each page of each object type
 **/


extern int DEBUG;	
//# define VENDOR_ID_SIZE 8
//# define ATTR_PAGE_ID_SIZE 32 
//Define debug level for this file as LOWER_FUNC
static int dlevel=LOWER_FUNC;

int create_root_attribute_pages(const char *osd_system_id,uint64_t capacity) 
{

	struct root_information_attribute_page root_info_attr_page_struct;
	struct root_quotas_attribute_page root_quota_attr_page_struct;	
	struct root_timestamps_attribute_page root_timestamp_attr_page_struct;	
	struct root_security_attribute_page  root_security_attr_page_struct;	
	uint48_t timestamp_val;
	
	timestamp_val=get_current_timestamp();	
	if (create_root_directory_page() )
	{
		print_msg(ERROR_MSG," Error in creating ROOT_DIRECTORY_PAGE  ");
	
		return 1;
	}
	memset( root_info_attr_page_struct.OSD_System_ID,0,OSD_ATTR_NUM_ROOT_INFO_OSD_SYSTEM_ID_size);
	strcpy((char *) (root_info_attr_page_struct.OSD_System_ID),osd_system_id);
	
	memset(root_info_attr_page_struct.Vendor_ID,0,OSD_ATTR_NUM_ROOT_INFO_VENDOR_ID_size);
	memset(root_info_attr_page_struct.Product_ID,0,OSD_ATTR_NUM_ROOT_INFO_PRODUCT_ID_size);
	memset(root_info_attr_page_struct.Product_Model,0,OSD_ATTR_NUM_ROOT_INFO_PRODUCT_MODEL_size);
	root_info_attr_page_struct.Product_Rev_Level=0;
	
	memset(root_info_attr_page_struct.Product_Serial_No,0,OSD_ATTR_NUM_ROOT_INFO_SERIAL_NUMBER_size);
	
	memset(root_info_attr_page_struct.OSD_Name,0, OSD_ATTR_NUM_ROOT_INFO_OSD_NAME_size);
	
	memcpy(root_info_attr_page_struct.OSD_Name, OSD_ATTR_NUM_ROOT_INFO_OSD_NAME_def_value,OSD_ATTR_NUM_ROOT_INFO_OSD_NAME_size);
	
	root_info_attr_page_struct.Total_Capacity=capacity;
	root_info_attr_page_struct.Used_Capacity=ROOT_PAGES_SIZE;
	root_info_attr_page_struct.No_of_Partitions=0;//this partition
	root_info_attr_page_struct.Clock=timestamp_val;//6 bytes 
	
	
	if(create_root_information_page(root_info_attr_page_struct))
	{
		print_msg(ERROR_MSG," Error in creating ROOT_INFO_PAGE  ");
		return 1;

	
	}
	
	root_quota_attr_page_struct.Default_Max_User_Object_Length=OSD_ATTR_NUM_ROOT_QUOTAS_DEFAULT_MAX_OBJ_LEN_def_value ;
	root_quota_attr_page_struct.Partition_Capacity_Quota= OSD_ATTR_NUM_ROOT_QUOTAS_PARTITION_CAPACITY_QUOTA_def_value;
	root_quota_attr_page_struct.Partition_Object_Count= OSD_ATTR_NUM_ROOT_QUOTAS_PARTITION_OBJ_COUNT_def_value;
	root_quota_attr_page_struct.Partition_Collections_Per_User_Object=
	 OSD_ATTR_NUM_ROOT_QUOTAS_PARTITION_COLLECTIONS_PER_OBJ_def_value;
	 
	root_quota_attr_page_struct.Partition_Count= OSD_ATTR_NUM_ROOT_QUOTAS_PARTITION_COUNT_def_value;
	
	if ( create_root_quotas_page(root_quota_attr_page_struct))
	{
		print_msg(ERROR_MSG," Error in creating ROOT_QUOTAS_PAGE  ");
		return 1;

	
	}	
	
	root_timestamp_attr_page_struct.Attr_Accessed_Time=timestamp_val;
	root_timestamp_attr_page_struct.Attr_Modified_Time=timestamp_val;
	root_timestamp_attr_page_struct.Timestamp_Bypass=OSD_ATTR_NUM_ROOT_TIMESTAMPS_TIMESTAMP_BYPASS_def_value;
	
	if(create_root_timestamps_page(root_timestamp_attr_page_struct))
	{
		print_msg(ERROR_MSG," Error in creating ROOT_TIMESTAMPS_PAGE  ");
	
		return 1;
	}	
	
	root_security_attr_page_struct.Default_Security_Method=OSD_ATTR_NUM_ROOT_SECURITY_DEFAULT_SEC_METHOD_def_value;
	memset(& (root_security_attr_page_struct.Oldest_Valid_Nonce_Limit),0,OSD_ATTR_NUM_ROOT_SECURITY_OLDEST_VALID_NONCE_size );
	memset(&(root_security_attr_page_struct.Newest_Valid_Nonce_Limit),0, OSD_ATTR_NUM_ROOT_SECURITY_NEWEST_VALID_NONCE_size);
	root_security_attr_page_struct.Partition_Default_Security_Method=OSD_ATTR_NUM_ROOT_SECURITY_PARTITION_DEFAULT_SEC_METHOD_def_value;
	root_security_attr_page_struct.Supported_Security_Methods=OSD_ATTR_NUM_ROOT_SECURITY_SUPPORTED_SEC_METHODS_def_value;
	root_security_attr_page_struct.Adjustable_Clock=timestamp_val;
	memset(&(root_security_attr_page_struct.Master_Key_ID),0, OSD_ATTR_NUM_ROOT_SECURITY_MASTER_KEY_ID_size);
	memset(&(root_security_attr_page_struct.Root_Key_ID),0, OSD_ATTR_NUM_ROOT_SECURITY_ROOT_KEY_ID_size);
	memset(root_security_attr_page_struct.Supported_Integrity_Chk_Value_Algorithm,0,16*OSD_ATTR_NUM_ROOT_SECURITY_ALG_1_size);
	memset(root_security_attr_page_struct.Supported_DH_Group,0,16*OSD_ATTR_NUM_ROOT_SECURITY_DH_GROUP_1_size);
	
	if (create_root_security_page(root_security_attr_page_struct))
	{
		print_msg(ERROR_MSG," Error in creating ROOT_SECURITY_PAGE  ");
	
		return 1;
	}
	return 0;
}


int create_partition_attribute_pages(uint64_t part_id,const char *username)
{
	
	struct partition_information_attribute_page part_info_attr_page_struct;
	struct partition_quotas_attribute_page part_quotas_attr_page_struct;
	struct partition_timestamps_attribute_page   part_timestamps_attr_page_struct;
	struct partition_security_attribute_page part_security_attr_page_struct;
	
	uint48_t timestamp_val;
	
	timestamp_val=get_current_timestamp();	

	if (create_partition_directory_page(part_id))
	{
		print_msg(ERROR_MSG," Error in creating PART_DIRECTORY_PAGE  ");
		return 1;
	}
	
	
	part_info_attr_page_struct.Partition_ID=part_id;
	memset(part_info_attr_page_struct.Username,0,OSD_ATTR_NUM_UOBJ_INFO_USERNAME_size); 
	memcpy(part_info_attr_page_struct.Username,username,OSD_ATTR_NUM_UOBJ_INFO_USERNAME_size);
	part_info_attr_page_struct.Used_Capacity=PART_PAGES_SIZE;

	part_info_attr_page_struct.No_CollectionUser_Objects=0;  
	
	if (create_partition_information_page(part_id,part_info_attr_page_struct))
	{
		print_msg(ERROR_MSG," Error in creating PART_INFO_PAGE  ");
		return 1;
	}
	
	//Inherit from root
		
	get_page_attr(0ULL,0ULL,OSD_PAGE_NUM_ROOT_QUOTAS,OSD_ATTR_NUM_ROOT_QUOTAS_DEFAULT_MAX_OBJ_LEN,&(part_quotas_attr_page_struct.Default_Max_User_Object_Length),ROOT_OBJECT_TYPE);
	
	
	get_page_attr(0ULL,0ULL,OSD_PAGE_NUM_ROOT_QUOTAS,OSD_ATTR_NUM_ROOT_QUOTAS_PARTITION_CAPACITY_QUOTA,&(part_quotas_attr_page_struct.Capacity_Quota),ROOT_OBJECT_TYPE);
	
	get_page_attr(0ULL,0ULL,OSD_PAGE_NUM_ROOT_QUOTAS,OSD_ATTR_NUM_ROOT_QUOTAS_PARTITION_OBJ_COUNT,&(part_quotas_attr_page_struct.Object_Count),ROOT_OBJECT_TYPE);
	
	get_page_attr(0ULL,0ULL,OSD_PAGE_NUM_ROOT_QUOTAS,OSD_ATTR_NUM_ROOT_QUOTAS_PARTITION_COLLECTIONS_PER_OBJ,&(part_quotas_attr_page_struct.Collections_Per_User_Object),ROOT_OBJECT_TYPE);
	
	
	
	if(create_partition_quotas_page(part_id,part_quotas_attr_page_struct))
	{
		print_msg(ERROR_MSG," Error in creating PART_QUOTAS_PAGE  ");
		return 1;
	
	
	}
		
	
	part_timestamps_attr_page_struct.Created_Time=timestamp_val;
	part_timestamps_attr_page_struct.Attr_Accessed_Time=timestamp_val;
	part_timestamps_attr_page_struct.Attr_Modified_Time=timestamp_val;
	part_timestamps_attr_page_struct.Data_Accessed_Time=timestamp_val;
	part_timestamps_attr_page_struct.Data_Modified_Time=timestamp_val;
	//part_timestamps_attr_page_struct.Timestamp_Bypass=OSD_ATTR_NUM_ROOT_TIMESTAMPS_TIMESTAMP_BYPASS_def_value;
	
	get_page_attr(0ULL,0ULL,OSD_PAGE_NUM_ROOT_TIMESTAMPS,OSD_ATTR_NUM_ROOT_TIMESTAMPS_TIMESTAMP_BYPASS,&(part_timestamps_attr_page_struct.Timestamp_Bypass),ROOT_OBJECT_TYPE);
	
	if( create_partition_timestamps_page(part_id,part_timestamps_attr_page_struct))
	{
	
		print_msg(ERROR_MSG," Error in creating PART_TIMESTAMPS_PAGE  ");
		return 1;
	
	
	
	}

	get_page_attr(0ULL,0ULL,OSD_PAGE_NUM_ROOT_SECURITY,OSD_ATTR_NUM_ROOT_SECURITY_DEFAULT_SEC_METHOD,&(part_security_attr_page_struct.Default_Security_Method),ROOT_OBJECT_TYPE);
	
	memset(&(part_security_attr_page_struct.Oldest_Valid_Nonce),0,OSD_ATTR_NUM_PART_SECURITY_OLDEST_VALID_NONCE_size );
	memset(&(part_security_attr_page_struct.Newest_Valid_Nonce),0, OSD_ATTR_NUM_PART_SECURITY_NEWEST_VALID_NONCE_size);
	
	part_security_attr_page_struct.Request_Nonce_List_Depth=0;
	part_security_attr_page_struct.Frozen_Working_Key_Bit_Mask=0;

	memset(&(part_security_attr_page_struct.Partition_Key_ID),0, OSD_ATTR_NUM_PART_SECURITY_PARTITION_KEY_ID_size);
	
	memset(&(part_security_attr_page_struct.Working_Key_ID),0,
	OSD_ATTR_NUM_PART_SECURITY_WORKING_KEY_ID_0_size);
	
	part_security_attr_page_struct.Policy_Access_Tag=0x7FFFFFFFULL;
	part_security_attr_page_struct.User_Object_Policy_Access_Tag=0x7FFFFFFFULL;
	

	if(create_partition_security_page(part_id,part_security_attr_page_struct ))
	{
	
		print_msg(ERROR_MSG," Error in creating PART_SECURITY_PAGE  ");
		return 1;
	
	
	
	}
	return 0;
}

int create_uobject_attribute_pages(uint64_t part_id,uint64_t object_id,const char *username)
{

	struct uobject_information_attribute_page  uobject_info_attr_page_struct;
	struct uobject_quotas_attribute_page uobject_quotas_attr_page_struct;
	struct uobject_timestamps_attribute_page uobject_timestamps_attr_page_struct;
	struct uobject_security_attribute_page  uobject_security_attr_page_struct;
	struct uobject_collections_attribute_page uobject_collections_attr_page_struct;
	uint48_t timestamp_val;
	timestamp_val=get_current_timestamp();	
	
	if ( create_uobject_directory_page(part_id,object_id))
	{
		print_msg(ERROR_MSG," Error in creating UOBJ_DIRECTORY_PAGE  ");
		return 1;
	}	
	
	
	uobject_info_attr_page_struct.Partition_ID=part_id;
	uobject_info_attr_page_struct.User_Object_ID=object_id;

	memset(uobject_info_attr_page_struct.Username,0,OSD_ATTR_NUM_UOBJ_INFO_USERNAME_size);
	memcpy(uobject_info_attr_page_struct.Username,username,OSD_ATTR_NUM_UOBJ_INFO_USERNAME_size);
	uobject_info_attr_page_struct.Used_Capacity= UOBJ_PAGES_SIZE;
	
	uobject_info_attr_page_struct.User_Object_Logical_Length=0;
	
	if(create_uobject_information_page(part_id,object_id,uobject_info_attr_page_struct))
	{
		print_msg(ERROR_MSG," Error in creating UOBJ_INFO_PAGE  ");
		return 1;
	}

	 get_page_attr(part_id,0ULL,OSD_PAGE_NUM_PART_QUOTAS,OSD_ATTR_NUM_PART_QUOTAS_DEFAULT_MAX_OBJ_LEN,&(uobject_quotas_attr_page_struct.Max_User_Object_Length),PART_OBJECT_TYPE);
	 
	 if(create_uobject_quotas_page(part_id,object_id,uobject_quotas_attr_page_struct)) 
	 {
		print_msg(ERROR_MSG," Error in creating UOBJ_QUOTAS_PAGE  ");
		return 1;
	 }

	uobject_timestamps_attr_page_struct.Created_Time=timestamp_val;
	uobject_timestamps_attr_page_struct.Attr_Accessed_Time=timestamp_val;
	uobject_timestamps_attr_page_struct.Attr_Modified_Time=timestamp_val;
	uobject_timestamps_attr_page_struct.Data_Accessed_Time=timestamp_val;
	uobject_timestamps_attr_page_struct.Data_Modified_Time=timestamp_val;
	 
	if(create_uobject_timestamps_page(part_id,object_id,uobject_timestamps_attr_page_struct))
	{
		print_msg(ERROR_MSG," Error in creating UOBJ_TIMESTAMPS_PAGE  ");
		return 1;
	}
	

	get_page_attr(part_id,0ULL,OSD_PAGE_NUM_PART_SECURITY ,OSD_ATTR_NUM_PART_SECURITY_USER_OBJ_POLICY_ACCESS_TAG,&(uobject_security_attr_page_struct.Policy_Access_Tag),PART_OBJECT_TYPE);
	 
	
	if(create_uobject_security_page(part_id,object_id,uobject_security_attr_page_struct))
	{
		print_msg(ERROR_MSG," Error in creating UOBJ_SECURITY_PAGE  ");
		return 1;
	}
	
	if(create_uobject_collections_page(part_id,object_id,uobject_collections_attr_page_struct))
	 {
		print_msg(ERROR_MSG," Error in creating UOBJ_COLLECTIONS_PAGE  ");
		return 1;
	 }

	return 0;

}


//--------------------------------------------------------------------------------------

int create_collection_attribute_pages(uint64_t part_id,uint64_t collection_object_id,const char *username)
{
	struct collection_information_attribute_page   coll_info_attr_page_struct;
	struct collection_timestamps_attribute_page  coll_timestamps_attr_page_struct;
	struct collection_security_attribute_page coll_security_attr_page_struct;
	uint48_t timestamp_val;
	timestamp_val=get_current_timestamp();	
	
	if(create_collection_directory_page(part_id,collection_object_id))
	{
		print_msg(ERROR_MSG," Error in creating COLL_DIRECTORY_PAGE  ");
		return 1;
	
	}
	

	coll_info_attr_page_struct.Partition_ID=part_id;
	coll_info_attr_page_struct.Collection_Object_ID=collection_object_id;

	memset(coll_info_attr_page_struct.Username,0,OSD_ATTR_NUM_COLL_INFO_USERNAME_size);
	memcpy(coll_info_attr_page_struct.Username,username,OSD_ATTR_NUM_COLL_INFO_USERNAME_size);
	
	coll_info_attr_page_struct.Used_Capacity=COLL_PAGES_SIZE ;  
	
	if(create_collection_information_page(part_id,collection_object_id,coll_info_attr_page_struct))
	{
		print_msg(ERROR_MSG," Error in creating COLL_INFO_PAGE  ");
		return 1;
	}
	

	coll_timestamps_attr_page_struct.Created_Time=timestamp_val;
	coll_timestamps_attr_page_struct.Attr_Accessed_Time=timestamp_val;
	coll_timestamps_attr_page_struct.Attr_Modified_Time=timestamp_val;
	coll_timestamps_attr_page_struct.Data_Accessed_Time=timestamp_val;
	coll_timestamps_attr_page_struct.Data_Modified_Time=timestamp_val;
	
	
	if(create_collection_timestamps_page(part_id,collection_object_id,coll_timestamps_attr_page_struct))
	{
		print_msg(ERROR_MSG," Error in creating COLL_TIMESTAMPS_PAGE ");
		return 1;
	}
	

	get_page_attr(part_id,0,OSD_PAGE_NUM_PART_SECURITY ,OSD_ATTR_NUM_PART_SECURITY_USER_OBJ_POLICY_ACCESS_TAG,&(coll_security_attr_page_struct.Policy_Access_Tag),PART_OBJECT_TYPE);
		
	if(create_collection_security_page(part_id,collection_object_id,coll_security_attr_page_struct))
	{
		print_msg(ERROR_MSG," Error in creating COLL_SECURITY_PAGE ");
		return 1;
	}


	return 0;
}

//--------------------------------------------------------------------------------------

int create_command_attribute_page(struct current_command_attribute_page current_command_attr_page_struct)
{

	char filename[FILENAME_SIZE];
	char buf[CURRENT_COMMAND_PAGE_SIZE];
	char *buf_ptr=buf;	
	uint32_t page_no=OSD_PAGE_NUM_ROOT_INFO,page_size=CURRENT_COMMAND_PAGE_SIZE;

	memcpy(buf_ptr,&(page_no),PAGE_NO_SIZE);
	buf_ptr+=PAGE_NO_SIZE;

	memcpy(buf_ptr,&(page_size),PAGE_LENGTH_SIZE);
	buf_ptr+=PAGE_NO_SIZE;
	

	memcpy(buf_ptr,(current_command_attr_page_struct.Response_Integirty_Chk_Value),OSD_ATTR_NUM_CUR_CMD_RESPONSE_INT_CHECK_VAL_size);
	buf_ptr+=OSD_ATTR_NUM_CUR_CMD_RESPONSE_INT_CHECK_VAL_size;
	
	memcpy(buf_ptr,&(current_command_attr_page_struct.Object_Type),OSD_ATTR_NUM_CUR_CMD_OBJ_TYPE_size);
	buf_ptr+=OSD_ATTR_NUM_CUR_CMD_OBJ_TYPE_size;
	
	memcpy(buf_ptr,&(current_command_attr_page_struct.Partition_ID),OSD_ATTR_NUM_CUR_CMD_PARTITION_ID_size);
	buf_ptr+=OSD_ATTR_NUM_CUR_CMD_PARTITION_ID_size;
	
	memcpy(buf_ptr,&(current_command_attr_page_struct.CollectionUser_Object_ID),OSD_ATTR_NUM_CUR_CMD_OBJ_ID_size);
	buf_ptr+=OSD_ATTR_NUM_CUR_CMD_OBJ_ID_size;
	
	memcpy(buf_ptr,&(current_command_attr_page_struct.Starting_Byte_Address_of_Append),OSD_ATTR_NUM_CUR_CMD_APPEND_STARTING_BYTE_size);
	buf_ptr+=OSD_ATTR_NUM_CUR_CMD_APPEND_STARTING_BYTE_size;

	if(get_attr_filename(0,0,OSD_PAGE_NUM_CUR_CMD,filename,CURR_CMD_OBJECT_TYPE)) //part_id=0  obj_id =0 pageno=CURENT COMMAND
	{
		print_msg(ERROR_MSG,"Error in getting CURRENT_COMMAND_PAGE file name for part_id =0 obj_id=0 page_no=OSD_PAGE_NUM_CUR_CMD "); 
		return 1;
	
	}
	

	if(file_write(filename,buf,CURRENT_COMMAND_PAGE_SIZE,0)){
		print_msg(ERROR_MSG,"Error in writting CURRENT_COMMAND_PAGE file  for part_id =0 obj_id=0 page_no=OSD_PAGE_NUM_CUR_CMD   filename=%s",filename); 
	return 1;

	}
	
	return 0;


}
//--------------------------------------------------------------------------------------

int create_root_directory_page()
{
	char filename[FILENAME_SIZE];
	char buf[ROOT_DIRECTORY_PAGE_SIZE];
	char space_buf[40]="                                        ";
	char *buf_ptr=buf;
	char vendor_id_buf[VENDOR_ID_SIZE]="INCITS  ";
	char page_attr_id_buf[ATTR_PAGE_ID_SIZE];
	
	
	memset(buf,0,ROOT_DIRECTORY_PAGE_SIZE);
	strcpy(page_attr_id_buf,"T10 Root Directory");
	strncat(page_attr_id_buf,space_buf,ATTR_PAGE_ID_SIZE-1-strlen("T10 Root Directory"));
	memcpy(buf_ptr,vendor_id_buf,VENDOR_ID_SIZE);
	buf_ptr+=VENDOR_ID_SIZE;
	memcpy(buf_ptr,page_attr_id_buf,ATTR_PAGE_ID_SIZE);
	buf_ptr+=ATTR_PAGE_ID_SIZE;
	
	strcpy(page_attr_id_buf,"T10 Root Information");
	strncat(page_attr_id_buf,space_buf,ATTR_PAGE_ID_SIZE-1-strlen("T10 Root Information"));
	memcpy(buf_ptr,vendor_id_buf,VENDOR_ID_SIZE);
	buf_ptr+=VENDOR_ID_SIZE;
	memcpy(buf_ptr,page_attr_id_buf,ATTR_PAGE_ID_SIZE);
	buf_ptr+=ATTR_PAGE_ID_SIZE;
	
	
	strcpy(page_attr_id_buf,"T10 Root Quotas");
	strncat(page_attr_id_buf,space_buf,ATTR_PAGE_ID_SIZE-1-strlen("T10 Root Quotas"));
	memcpy(buf_ptr,vendor_id_buf,VENDOR_ID_SIZE);
	buf_ptr+=VENDOR_ID_SIZE;
	memcpy(buf_ptr,page_attr_id_buf,ATTR_PAGE_ID_SIZE);
	buf_ptr+=ATTR_PAGE_ID_SIZE;
	
	strcpy(page_attr_id_buf,"T10 Root Timestamps");
	strncat(page_attr_id_buf,space_buf,ATTR_PAGE_ID_SIZE-1-strlen("T10 Root Timestamps"));
	memcpy(buf_ptr,vendor_id_buf,VENDOR_ID_SIZE);
	buf_ptr+=VENDOR_ID_SIZE;
	memcpy(buf_ptr,page_attr_id_buf,ATTR_PAGE_ID_SIZE);
	buf_ptr+=ATTR_PAGE_ID_SIZE;
	
	strcpy(page_attr_id_buf,"T10 Root Policy/Security");
	strncat(page_attr_id_buf,space_buf,ATTR_PAGE_ID_SIZE-1-strlen("T10 Root Policy/Security"));
	memcpy(buf_ptr,vendor_id_buf,VENDOR_ID_SIZE);
	buf_ptr+=VENDOR_ID_SIZE;
	memcpy(buf_ptr,page_attr_id_buf,ATTR_PAGE_ID_SIZE);
	buf_ptr+=ATTR_PAGE_ID_SIZE;
	//file_write
	
	if(get_attr_filename(0,0,OSD_PAGE_NUM_ROOT_DIRECTORY,filename,ROOT_OBJECT_TYPE))	 //part_id=0  obj_id =0 pageno=0
	{
		print_msg(ERROR_MSG,"Error in getting ROOT_DIRECTORY_PAGE file name for part_id =0 obj_id=0 page_no=0x%lx",OSD_PAGE_NUM_ROOT_DIRECTORY); 
		return 1;
	}

	
	if(file_write(filename,buf,ROOT_DIRECTORY_PAGE_SIZE,ROOT_DIRECTORY_PAGE_OFFSET_IN_FILE))
	{
		print_msg(ERROR_MSG,"Error in Writting ROOT_DIRECTORY_PAGE file  for part_id =0 obj_id=0 page_no=0x%lx filename=%s",OSD_PAGE_NUM_ROOT_DIRECTORY,filename); 
		return 1;
	}
	return 0;
}

//--------------------------------------------------------------------------------------///	
int create_root_information_page(struct root_information_attribute_page root_info_attr_page_struct )
{
  
	char filename[FILENAME_SIZE];
	char buf[ROOT_INFORMATION_PAGE_SIZE];
	char *buf_ptr=buf;	
	uint32_t page_no=OSD_PAGE_NUM_ROOT_INFO,page_size=ROOT_INFORMATION_PAGE_SIZE;


	
	memset(buf,0,ROOT_INFORMATION_PAGE_SIZE);
	memcpy(buf_ptr,&(page_no),PAGE_NO_SIZE);
	buf_ptr+=PAGE_NO_SIZE;

	memcpy(buf_ptr,&(page_size),PAGE_LENGTH_SIZE);
	buf_ptr+=PAGE_NO_SIZE;

	memcpy(buf_ptr,&(root_info_attr_page_struct.OSD_System_ID), OSD_ATTR_NUM_ROOT_INFO_OSD_SYSTEM_ID_size);
	buf_ptr+=OSD_ATTR_NUM_ROOT_INFO_OSD_SYSTEM_ID_size;

	memcpy(buf_ptr,&(root_info_attr_page_struct.Vendor_ID),OSD_ATTR_NUM_ROOT_INFO_VENDOR_ID_size);
	buf_ptr+=OSD_ATTR_NUM_ROOT_INFO_VENDOR_ID_size;

	memcpy(buf_ptr,&(root_info_attr_page_struct.Product_ID),OSD_ATTR_NUM_ROOT_INFO_PRODUCT_ID_size);
	buf_ptr+=OSD_ATTR_NUM_ROOT_INFO_PRODUCT_ID_size;


	memcpy(buf_ptr,&(root_info_attr_page_struct.Product_Model),OSD_ATTR_NUM_ROOT_INFO_PRODUCT_MODEL_size);
	buf_ptr+=OSD_ATTR_NUM_ROOT_INFO_PRODUCT_MODEL_size;

	memcpy(buf_ptr,&( root_info_attr_page_struct.Product_Rev_Level),OSD_ATTR_NUM_ROOT_INFO_PRODUCT_REVISION_LEVEL_size);
	buf_ptr+=OSD_ATTR_NUM_ROOT_INFO_PRODUCT_REVISION_LEVEL_size;

	memcpy(buf_ptr,&(root_info_attr_page_struct.Product_Serial_No),OSD_ATTR_NUM_ROOT_INFO_SERIAL_NUMBER_size);
	buf_ptr+=OSD_ATTR_NUM_ROOT_INFO_SERIAL_NUMBER_size;

	memcpy(buf_ptr,&(root_info_attr_page_struct.OSD_Name),OSD_ATTR_NUM_ROOT_INFO_OSD_NAME_size);
	buf_ptr+=OSD_ATTR_NUM_ROOT_INFO_OSD_NAME_size;

	memcpy(buf_ptr,&(root_info_attr_page_struct.Total_Capacity),OSD_ATTR_NUM_ROOT_INFO_TOTAL_CAPACITY_size);
	buf_ptr+=OSD_ATTR_NUM_ROOT_INFO_TOTAL_CAPACITY_size;

	memcpy(buf_ptr,&(root_info_attr_page_struct.Used_Capacity),OSD_ATTR_NUM_ROOT_INFO_USED_CAPACITY_size);
	buf_ptr+=OSD_ATTR_NUM_ROOT_INFO_USED_CAPACITY_size;

	memcpy(buf_ptr,&(root_info_attr_page_struct.No_of_Partitions),OSD_ATTR_NUM_ROOT_INFO_NUM_OF_PARTITIONS_size);
	buf_ptr+=OSD_ATTR_NUM_ROOT_INFO_NUM_OF_PARTITIONS_size;

	memcpy(buf_ptr,&(root_info_attr_page_struct.Clock),OSD_ATTR_NUM_ROOT_INFO_CLOCK_size);
	buf_ptr+=OSD_ATTR_NUM_ROOT_INFO_CLOCK_size;

	if(get_attr_filename(0,0,OSD_PAGE_NUM_ROOT_INFO,filename,ROOT_OBJECT_TYPE))	 //part_id=0  obj_id =0 pageno=1
	{
		print_msg(ERROR_MSG,"Error in getting ROOT_INFORMATION_PAGE file name for part_id =0 obj_id=0 page_no=0x%lx ",OSD_PAGE_NUM_ROOT_INFO); 
		return 1;
	
	}

	if(file_write(filename,buf,ROOT_INFORMATION_PAGE_SIZE,ROOT_INFORMATION_PAGE_OFFSET_IN_FILE)){
		print_msg(ERROR_MSG,"Error in Writting ROOT_INFORMATION_PAGE file  for part_id =0 obj_id=0 page_no= 0x%lx   filename=%s",OSD_PAGE_NUM_ROOT_INFO,filename); 
		return 1;
	
	
	}
	return 0;
	
}
//--------------------------------------------------------------------------------------
int create_root_quotas_page(struct root_quotas_attribute_page root_quota_attr_page_struct)
{

	char filename[FILENAME_SIZE];
	char buf[ROOT_QUOTAS_PAGE_SIZE];
	char *buf_ptr=buf;	
	uint32_t page_no=OSD_PAGE_NUM_ROOT_QUOTAS,page_size=ROOT_QUOTAS_PAGE_SIZE;
	

	memset(buf,0,ROOT_QUOTAS_PAGE_SIZE);
	
	memcpy(buf_ptr,&(page_no),PAGE_NO_SIZE);
	buf_ptr+=PAGE_NO_SIZE;
	memcpy(buf_ptr,&(page_size),PAGE_LENGTH_SIZE);
	buf_ptr+=PAGE_NO_SIZE;
	
	memcpy(buf_ptr,&(root_quota_attr_page_struct.Default_Max_User_Object_Length),OSD_ATTR_NUM_ROOT_QUOTAS_DEFAULT_MAX_OBJ_LEN_size);
	buf_ptr+= OSD_ATTR_NUM_ROOT_QUOTAS_DEFAULT_MAX_OBJ_LEN_size;

	
	
	memcpy(buf_ptr,&(root_quota_attr_page_struct.Partition_Capacity_Quota),OSD_ATTR_NUM_ROOT_QUOTAS_PARTITION_CAPACITY_QUOTA_size);
	buf_ptr+=OSD_ATTR_NUM_ROOT_QUOTAS_PARTITION_CAPACITY_QUOTA_size;
	
	memcpy(buf_ptr,&(root_quota_attr_page_struct.Partition_Object_Count),OSD_ATTR_NUM_ROOT_QUOTAS_PARTITION_OBJ_COUNT_size);
	buf_ptr+=OSD_ATTR_NUM_ROOT_QUOTAS_PARTITION_OBJ_COUNT_size;
	
	memcpy(buf_ptr,&(root_quota_attr_page_struct.Partition_Collections_Per_User_Object),OSD_ATTR_NUM_ROOT_QUOTAS_PARTITION_COLLECTIONS_PER_OBJ_size);
	buf_ptr+=OSD_ATTR_NUM_ROOT_QUOTAS_PARTITION_COLLECTIONS_PER_OBJ_size;
	

	memcpy(buf_ptr,&(root_quota_attr_page_struct.Partition_Count),OSD_ATTR_NUM_ROOT_QUOTAS_PARTITION_COUNT_size);
	buf_ptr+=OSD_ATTR_NUM_ROOT_QUOTAS_PARTITION_COUNT_size;
	
	

	if(get_attr_filename(0,0,OSD_PAGE_NUM_ROOT_QUOTAS,filename,ROOT_OBJECT_TYPE))	 //part_id=0  obj_id =0 pageno=2
	{
		print_msg(ERROR_MSG," Error in getting ROOT_QUOTAS_PAGE file name for part_id =0 obj_id=0 page_no=0x%lx   ",OSD_PAGE_NUM_ROOT_QUOTAS); 
		return 1;
	
	}
	
	if(file_write(filename,buf,ROOT_QUOTAS_PAGE_SIZE,ROOT_QUOTAS_PAGE_OFFSET_IN_FILE)){
		print_msg(ERROR_MSG,"Error in Writting ROOT_QUOTAS_PAGE file  for part_id =0 obj_id=0 page_no=0x%lx   filename=%s ",OSD_PAGE_NUM_ROOT_QUOTAS,filename); 
		return 1;
	}

	return 0;
}
//--------------------------------------------------------------------------------------
	
int create_root_timestamps_page(struct root_timestamps_attribute_page root_timestamp_attr_page_struct)
{

	char filename[FILENAME_SIZE];
	char buf[ROOT_TIMESTAMPS_PAGE_SIZE];
	char *buf_ptr=buf;	
	uint32_t page_no=OSD_PAGE_NUM_ROOT_TIMESTAMPS,page_size=ROOT_TIMESTAMPS_PAGE_SIZE;

	memset(buf,0,ROOT_TIMESTAMPS_PAGE_SIZE);
	memcpy(buf_ptr,&(page_no),PAGE_NO_SIZE);
	buf_ptr+=PAGE_NO_SIZE;

	memcpy(buf_ptr,&(page_size),PAGE_LENGTH_SIZE);
	buf_ptr+=PAGE_NO_SIZE;

	memcpy(buf_ptr,&(root_timestamp_attr_page_struct.Attr_Accessed_Time),OSD_ATTR_NUM_ROOT_TIMESTAMPS_ATTR_ACCESSED_TIME_size);
	buf_ptr+=OSD_ATTR_NUM_ROOT_TIMESTAMPS_ATTR_ACCESSED_TIME_size;

	memcpy(buf_ptr,&(root_timestamp_attr_page_struct.Attr_Modified_Time),OSD_ATTR_NUM_ROOT_TIMESTAMPS_ATTR_MODIFIED_TIME_size);
	buf_ptr+=OSD_ATTR_NUM_ROOT_TIMESTAMPS_ATTR_MODIFIED_TIME_size;

	memcpy(buf_ptr,&(root_timestamp_attr_page_struct.Timestamp_Bypass),OSD_ATTR_NUM_ROOT_TIMESTAMPS_TIMESTAMP_BYPASS_size);
	buf_ptr+=OSD_ATTR_NUM_ROOT_TIMESTAMPS_TIMESTAMP_BYPASS_size;
	
	if(get_attr_filename(0,0,OSD_PAGE_NUM_ROOT_TIMESTAMPS,filename,ROOT_OBJECT_TYPE))	 //part_id=0  obj_id =0 pageno=3
	{
		print_msg(ERROR_MSG,"Error in getting ROOT_TIMESTAMPS_PAGE file name for part_id =0 obj_id=0 page_no=0x%lx  ", OSD_PAGE_NUM_ROOT_TIMESTAMPS); 
		return 1;
	
	}
	print_msg(dlevel,"filename =%s ",filename);
	
	if(file_write(filename,buf,ROOT_TIMESTAMPS_PAGE_SIZE,ROOT_TIMESTAMPS_PAGE_OFFSET_IN_FILE)){
		print_msg(ERROR_MSG,"Error in Writting ROOT_TIMESTAMPS_PAGE file  for part_id =0 obj_id=0 page_no=0x%lx   filename=%s ", OSD_PAGE_NUM_ROOT_TIMESTAMPS, filename); 
		return 1;

	}
	return 0;
}

//--------------------------------------------------------------------------------------

int create_root_security_page(struct root_security_attribute_page  root_security_attr_page_struct)
{

	char filename[FILENAME_SIZE];
	char buf[ROOT_SECURITY_PAGE_SIZE];
	char *buf_ptr=buf;	
	uint32_t page_no=OSD_PAGE_NUM_ROOT_INFO,page_size=ROOT_SECURITY_PAGE_SIZE;


	memset(buf,0,ROOT_SECURITY_PAGE_SIZE);
	memcpy(buf_ptr,&(page_no),PAGE_NO_SIZE);
	buf_ptr+=PAGE_NO_SIZE;

	memcpy(buf_ptr,&(page_size),PAGE_LENGTH_SIZE);
	buf_ptr+=PAGE_NO_SIZE;

	memcpy(buf_ptr,&(root_security_attr_page_struct.Default_Security_Method),OSD_ATTR_NUM_ROOT_SECURITY_DEFAULT_SEC_METHOD_size);
	buf_ptr+=OSD_ATTR_NUM_ROOT_SECURITY_DEFAULT_SEC_METHOD_size;

	memcpy(buf_ptr,&(root_security_attr_page_struct.Oldest_Valid_Nonce_Limit),OSD_ATTR_NUM_ROOT_SECURITY_OLDEST_VALID_NONCE_size);
	buf_ptr+=OSD_ATTR_NUM_ROOT_SECURITY_OLDEST_VALID_NONCE_size;

	memcpy(buf_ptr,&(root_security_attr_page_struct.Newest_Valid_Nonce_Limit),OSD_ATTR_NUM_ROOT_SECURITY_NEWEST_VALID_NONCE_size);
	buf_ptr+=OSD_ATTR_NUM_ROOT_SECURITY_NEWEST_VALID_NONCE_size;

	memcpy(buf_ptr,&(root_security_attr_page_struct.Partition_Default_Security_Method),OSD_ATTR_NUM_ROOT_SECURITY_PARTITION_DEFAULT_SEC_METHOD_size);
	buf_ptr+=OSD_ATTR_NUM_ROOT_SECURITY_PARTITION_DEFAULT_SEC_METHOD_size;


	memcpy(buf_ptr,&(root_security_attr_page_struct.Supported_Security_Methods),OSD_ATTR_NUM_ROOT_SECURITY_SUPPORTED_SEC_METHODS_size);
	buf_ptr+=OSD_ATTR_NUM_ROOT_SECURITY_SUPPORTED_SEC_METHODS_size;
	
	memcpy(buf_ptr,&(root_security_attr_page_struct.Adjustable_Clock),OSD_ATTR_NUM_ROOT_SECURITY_ADJUSTABLE_CLOCK_size);
	buf_ptr+=OSD_ATTR_NUM_ROOT_SECURITY_ADJUSTABLE_CLOCK_size;

	memcpy(buf_ptr,&(root_security_attr_page_struct.Master_Key_ID),OSD_ATTR_NUM_ROOT_SECURITY_MASTER_KEY_ID_size);
	buf_ptr+=OSD_ATTR_NUM_ROOT_SECURITY_MASTER_KEY_ID_size;


	memcpy(buf_ptr,&(root_security_attr_page_struct.Root_Key_ID),OSD_ATTR_NUM_ROOT_SECURITY_ROOT_KEY_ID_size);
	buf_ptr+=OSD_ATTR_NUM_ROOT_SECURITY_ROOT_KEY_ID_size;

	memcpy(buf_ptr,(root_security_attr_page_struct.Supported_Integrity_Chk_Value_Algorithm),16*OSD_ATTR_NUM_ROOT_SECURITY_ALG_1_size);
	buf_ptr+=(16*OSD_ATTR_NUM_ROOT_SECURITY_ALG_1_size);


	memcpy(buf_ptr,(root_security_attr_page_struct.Supported_DH_Group),16*OSD_ATTR_NUM_ROOT_SECURITY_DH_GROUP_1_size);
	buf_ptr+= (OSD_ATTR_NUM_ROOT_SECURITY_DH_GROUP_1_size*16);

	

	if(get_attr_filename(0,0,OSD_PAGE_NUM_ROOT_SECURITY,filename,ROOT_OBJECT_TYPE))	 //part_id=0  obj_id =0 pageno=5
	{
		print_msg(ERROR_MSG,"Error in getting ROOT_SECURITY_PAGE file name for part_id =0 obj_id=0 page_no=0x%lx ",OSD_PAGE_NUM_ROOT_SECURITY); 
		return 1;
	
	}

	if(file_write(filename,buf,ROOT_SECURITY_PAGE_SIZE,ROOT_SECURITY_PAGE_OFFSET_IN_FILE)){
		print_msg(ERROR_MSG,"Error in Writting ROOT_SECURITY_PAGE file  for part_id =0 obj_id=0 page_no=0x%lx   filename=%s",  OSD_PAGE_NUM_ROOT_SECURITY, filename); 
		return 1;
	}
	
	return 0;
}

//--------------------------------------------------------------------------------------
	
int create_partition_directory_page(uint64_t part_id)
{
	char filename[FILENAME_SIZE];
	char buf[PART_DIRECTORY_PAGE_SIZE];
	char *buf_ptr=buf;
	char space_buf[40]="                                        ";
	char vendor_id_buf[VENDOR_ID_SIZE]="INCITS  ";
	char page_attr_id_buf[ATTR_PAGE_ID_SIZE];
	

	strcpy(page_attr_id_buf,"T10 Partition Directory");
	strncat(page_attr_id_buf,space_buf,ATTR_PAGE_ID_SIZE-1-strlen("T10 Partition Directory"));
	memcpy(buf_ptr,vendor_id_buf,VENDOR_ID_SIZE);
	buf_ptr+=VENDOR_ID_SIZE;
	memcpy(buf_ptr,page_attr_id_buf,ATTR_PAGE_ID_SIZE);
	buf_ptr+=ATTR_PAGE_ID_SIZE;
	
	
	strcpy(page_attr_id_buf,"T10 Partition Information");
	strncat(page_attr_id_buf,space_buf,ATTR_PAGE_ID_SIZE-1-strlen("T10 Partition Information"));
	memcpy(buf_ptr,vendor_id_buf,VENDOR_ID_SIZE);
	buf_ptr+=VENDOR_ID_SIZE;
	memcpy(buf_ptr,page_attr_id_buf,ATTR_PAGE_ID_SIZE);
	buf_ptr+=ATTR_PAGE_ID_SIZE;
	
	strcpy(page_attr_id_buf,"T10 Partition Quotas");
	strncat(page_attr_id_buf,space_buf,ATTR_PAGE_ID_SIZE-1-strlen("T10 Partition Quotas"));
	memcpy(buf_ptr,vendor_id_buf,VENDOR_ID_SIZE);
	buf_ptr+=VENDOR_ID_SIZE;
	memcpy(buf_ptr,page_attr_id_buf,ATTR_PAGE_ID_SIZE);
	buf_ptr+=ATTR_PAGE_ID_SIZE;
	
	strcpy(page_attr_id_buf,"T10 Partition Timestamps");
	strncat(page_attr_id_buf,space_buf,ATTR_PAGE_ID_SIZE-1-strlen("T10 Partition Timestamps"));
	memcpy(buf_ptr,vendor_id_buf,VENDOR_ID_SIZE);
	buf_ptr+=VENDOR_ID_SIZE;
	memcpy(buf_ptr,page_attr_id_buf,ATTR_PAGE_ID_SIZE);
	buf_ptr+=ATTR_PAGE_ID_SIZE;
	
	strcpy(page_attr_id_buf,"T10 Partition Policy/Security");
	strncat(page_attr_id_buf,space_buf,ATTR_PAGE_ID_SIZE-1-strlen("T10 Partition Policy/Security"));
	memcpy(buf_ptr,vendor_id_buf,VENDOR_ID_SIZE);
	buf_ptr+=VENDOR_ID_SIZE;
	memcpy(buf_ptr,page_attr_id_buf,ATTR_PAGE_ID_SIZE);
	buf_ptr+=ATTR_PAGE_ID_SIZE;
	//file_write
	if (get_attr_filename(part_id,0,OSD_PAGE_NUM_PART_DIRECTORY,filename,PART_OBJECT_TYPE)) //obj_id =0 pageno=0
	{
		print_msg(ERROR_MSG,"Error in getting PARTT_DIRECTORY_PAGE file name for part_id =0x%llx obj_id=0 page_no=0x%lx ",part_id,OSD_PAGE_NUM_PART_DIRECTORY); 
		return 1;
	}
		
	if(file_write(filename,buf,PART_DIRECTORY_PAGE_SIZE,PART_DIRECTORY_PAGE_OFFSET_IN_FILE))
	{
		print_msg(ERROR_MSG,"Error in Writting PART_DIRECTORY_PAGE file  for part_id =0x%llx obj_id=0 page_no=0x%lx   filename=%s",part_id, OSD_PAGE_NUM_PART_DIRECTORY,filename); 
		return 1;
	}
	return 0;
}
//--------------------------------------------------------------------------------------

int create_partition_information_page(uint64_t part_id,struct partition_information_attribute_page part_info_attr_page_struct)
{
	char filename[FILENAME_SIZE];
	char buf[PART_INFORMATION_PAGE_SIZE];
	char *buf_ptr=buf;	
	uint32_t page_no=OSD_PAGE_NUM_ROOT_INFO,page_size=PART_INFORMATION_PAGE_SIZE;

	print_msg(dlevel," ");
	memset(buf,0,PART_INFORMATION_PAGE_SIZE);
	
	memcpy(buf_ptr,&(page_no),PAGE_NO_SIZE);
	buf_ptr+=PAGE_NO_SIZE;

	memcpy(buf_ptr,&(page_size),PAGE_LENGTH_SIZE);
	buf_ptr+=PAGE_NO_SIZE;

	memcpy(buf_ptr,&(part_info_attr_page_struct.Partition_ID),OSD_ATTR_NUM_PART_INFO_PARTITION_ID_size);
	buf_ptr+=OSD_ATTR_NUM_PART_INFO_PARTITION_ID_size;
	
	memcpy(buf_ptr,part_info_attr_page_struct.Username,OSD_ATTR_NUM_PART_INFO_USERNAME_size);
	buf_ptr+=OSD_ATTR_NUM_PART_INFO_USERNAME_size;
	
	memcpy(buf_ptr,&(part_info_attr_page_struct.Used_Capacity),OSD_ATTR_NUM_PART_INFO_USED_CAPACITY_size);
	buf_ptr+=OSD_ATTR_NUM_PART_INFO_USED_CAPACITY_size;
	
	memcpy(buf_ptr,&(part_info_attr_page_struct.No_CollectionUser_Objects),OSD_ATTR_NUM_PART_INFO_NUM_OF_OBJECTS_size);
	buf_ptr+=OSD_ATTR_NUM_PART_INFO_NUM_OF_OBJECTS_size;
	
	
	if(get_attr_filename(part_id,0,OSD_PAGE_NUM_PART_INFO,filename,PART_OBJECT_TYPE))	 //part_id=0  obj_id =0 pageno=1
	{
		print_msg(ERROR_MSG,"Error in getting PART_INFORMATION_PAGE file name for part_id =0x%llx obj_id=0 page_no=0x%lx ",part_id, OSD_PAGE_NUM_PART_INFO); 
		return 1;
	
	}

	if(file_write(filename,buf,PART_INFORMATION_PAGE_SIZE,PART_INFORMATION_PAGE_OFFSET_IN_FILE)){
		print_msg(ERROR_MSG,"Error in Writting PART_INFORMATION_PAGE file  for part_id =0x%llx obj_id=0 page_no=0x%lx   filename=%s",part_id, OSD_PAGE_NUM_PART_INFO,filename); 
		return 1;
	}
	return 0;
	


}
//--------------------------------------------------------------------------------------	
int create_partition_quotas_page(uint64_t part_id,struct partition_quotas_attribute_page part_quotas_attr_page_struct)
{
	char filename[FILENAME_SIZE];
	char buf[PART_QUOTAS_PAGE_SIZE];
	char *buf_ptr=buf;	
	uint32_t page_no=OSD_PAGE_NUM_ROOT_INFO,page_size=PART_QUOTAS_PAGE_SIZE;

	memset(buf,0,PART_QUOTAS_PAGE_SIZE);
	memcpy(buf_ptr,&(page_no),PAGE_NO_SIZE);
	buf_ptr+=PAGE_NO_SIZE;

	memcpy(buf_ptr,&(page_size),PAGE_LENGTH_SIZE);
	buf_ptr+=PAGE_NO_SIZE;


	memcpy(buf_ptr,&(part_quotas_attr_page_struct.Default_Max_User_Object_Length),OSD_ATTR_NUM_PART_QUOTAS_DEFAULT_MAX_OBJ_LEN_size);
	buf_ptr+=OSD_ATTR_NUM_PART_QUOTAS_DEFAULT_MAX_OBJ_LEN_size;

	memcpy(buf_ptr,&(part_quotas_attr_page_struct.Capacity_Quota),OSD_ATTR_NUM_PART_QUOTAS_CAPACITY_QUOTA_size);
	buf_ptr+=OSD_ATTR_NUM_PART_QUOTAS_CAPACITY_QUOTA_size;
	
	memcpy(buf_ptr,&(part_quotas_attr_page_struct.Object_Count),OSD_ATTR_NUM_PART_QUOTAS_OBJ_COUNT_size);
	buf_ptr+=OSD_ATTR_NUM_PART_QUOTAS_OBJ_COUNT_size;
	
	memcpy(buf_ptr,&(part_quotas_attr_page_struct.Collections_Per_User_Object),OSD_ATTR_NUM_PART_QUOTAS_COLLECTIONS_PER_OBJ_size);
	buf_ptr+=OSD_ATTR_NUM_PART_QUOTAS_COLLECTIONS_PER_OBJ_size;


	if(get_attr_filename(part_id,0,OSD_PAGE_NUM_PART_QUOTAS,filename,PART_OBJECT_TYPE))	 //part_id=0  obj_id =0 pageno=2
	{
		print_msg(ERROR_MSG,"Error in getting PART_INFORMATION_PAGE file name for part_id =0x%llx obj_id=0 page_no=0x%lx ",part_id,OSD_PAGE_NUM_PART_QUOTAS ); 
		return 1;
	
	}
	
	if(file_write(filename,buf,PART_QUOTAS_PAGE_SIZE,PART_QUOTAS_PAGE_OFFSET_IN_FILE)){
		print_msg(ERROR_MSG,"Error in Writting PART_QUOTAS_PAGE file  for part_id =0x%llx obj_id=0 page_no=0x%lx   filename=%s",part_id,OSD_PAGE_NUM_PART_QUOTAS,filename); 
		return 1;
	}
	
	return 0;
	
}


int create_partition_timestamps_page(uint64_t part_id,struct partition_timestamps_attribute_page part_timestamps_attr_page_struct)
{

	char filename[FILENAME_SIZE];
	char buf[PART_QUOTAS_PAGE_SIZE];
	char *buf_ptr=buf;	
	uint32_t page_no=OSD_PAGE_NUM_ROOT_INFO,page_size=PART_TIMESTAMPS_PAGE_SIZE;


	memset(buf,0,PART_QUOTAS_PAGE_SIZE);
	memcpy(buf_ptr,&(page_no),PAGE_NO_SIZE);
	buf_ptr+=PAGE_NO_SIZE;

	memcpy(buf_ptr,&(page_size),PAGE_LENGTH_SIZE);
	buf_ptr+=PAGE_NO_SIZE;

	memcpy(buf_ptr,&(part_timestamps_attr_page_struct.Created_Time),OSD_ATTR_NUM_PART_TIMESTAMPS_CREATED_TIME_size);
	buf_ptr+=OSD_ATTR_NUM_PART_TIMESTAMPS_CREATED_TIME_size;

	memcpy(buf_ptr,&(part_timestamps_attr_page_struct.Attr_Accessed_Time),OSD_ATTR_NUM_PART_TIMESTAMPS_ATTR_ACCESSED_TIME_size);
	buf_ptr+=OSD_ATTR_NUM_PART_TIMESTAMPS_ATTR_ACCESSED_TIME_size;

	memcpy(buf_ptr,&(part_timestamps_attr_page_struct.Attr_Modified_Time),OSD_ATTR_NUM_PART_TIMESTAMPS_ATTR_MODIFIED_TIME_size);
	buf_ptr+=OSD_ATTR_NUM_PART_TIMESTAMPS_ATTR_MODIFIED_TIME_size;

	memcpy(buf_ptr,&(part_timestamps_attr_page_struct.Data_Accessed_Time),OSD_ATTR_NUM_PART_TIMESTAMPS_DATA_ACCESSED_TIME_size);
	buf_ptr+=OSD_ATTR_NUM_PART_TIMESTAMPS_DATA_ACCESSED_TIME_size;

	memcpy(buf_ptr,&(part_timestamps_attr_page_struct.Data_Modified_Time),OSD_ATTR_NUM_PART_TIMESTAMPS_DATA_MODIFIED_TIME_size);
	buf_ptr+=OSD_ATTR_NUM_PART_TIMESTAMPS_DATA_MODIFIED_TIME_size;
	
	memcpy(buf_ptr,&(part_timestamps_attr_page_struct.Timestamp_Bypass),OSD_ATTR_NUM_PART_TIMESTAMPS_TIMESTAMP_BYPASS_size);
	buf_ptr+=OSD_ATTR_NUM_PART_TIMESTAMPS_TIMESTAMP_BYPASS_size;


	if(get_attr_filename(part_id,0,OSD_PAGE_NUM_PART_TIMESTAMPS,filename,PART_OBJECT_TYPE))	 //part_id=part_id  obj_id =0 pageno=3
	{
		print_msg(ERROR_MSG,"Error in getting PART_TIMESTAMPS_PAGE file name for part_id =0x%llx obj_id=0 page_no=0x%lx ",part_id,OSD_PAGE_NUM_PART_TIMESTAMPS); 
		return 1;
	
	}
	
	if(file_write(filename,buf,PART_TIMESTAMPS_PAGE_SIZE,PART_TIMESTAMPS_PAGE_OFFSET_IN_FILE)){
		print_msg(ERROR_MSG,"Error in Writting PART_TIMESTAMPS_PAGE file  for part_id =0x%llx obj_id=0 page_no=0x%lx   filename=%s",part_id,OSD_PAGE_NUM_PART_TIMESTAMPS,filename); 
		return 1;
	}
	return 0;

}
int create_partition_security_page(uint64_t part_id,struct partition_security_attribute_page part_security_attr_page_struct)
{
	char filename[FILENAME_SIZE];
	char buf[PART_SECURITY_PAGE_SIZE];
	char *buf_ptr=buf;	
	uint32_t page_no=OSD_PAGE_NUM_ROOT_INFO,page_size=PART_SECURITY_PAGE_SIZE;


	memset(buf,0,PART_SECURITY_PAGE_SIZE);
	memcpy(buf_ptr,&(page_no),PAGE_NO_SIZE);
	buf_ptr+=PAGE_NO_SIZE;

	memcpy(buf_ptr,&(page_size),PAGE_LENGTH_SIZE);
	buf_ptr+=PAGE_NO_SIZE;

	
	memcpy(buf_ptr,&(part_security_attr_page_struct.Default_Security_Method),OSD_ATTR_NUM_PART_SECURITY_DEFAULT_SEC_METHOD_size);
	buf_ptr+=OSD_ATTR_NUM_PART_SECURITY_DEFAULT_SEC_METHOD_size;

	memcpy(buf_ptr,&(part_security_attr_page_struct.Oldest_Valid_Nonce),OSD_ATTR_NUM_PART_SECURITY_OLDEST_VALID_NONCE_size);
	buf_ptr+=OSD_ATTR_NUM_PART_SECURITY_OLDEST_VALID_NONCE_size;

	memcpy(buf_ptr,&(part_security_attr_page_struct.Newest_Valid_Nonce),OSD_ATTR_NUM_PART_SECURITY_NEWEST_VALID_NONCE_size);
	buf_ptr+=OSD_ATTR_NUM_PART_SECURITY_NEWEST_VALID_NONCE_size;

	memcpy(buf_ptr,&(part_security_attr_page_struct.Request_Nonce_List_Depth),OSD_ATTR_NUM_PART_SECURITY_REQ_NONCE_LIST_DEPTH_size);
	buf_ptr+=OSD_ATTR_NUM_PART_SECURITY_REQ_NONCE_LIST_DEPTH_size;


	memcpy(buf_ptr,&(part_security_attr_page_struct.Frozen_Working_Key_Bit_Mask),OSD_ATTR_NUM_PART_SECURITY_FROZEN_KEYS_MASK_size);
	buf_ptr+=OSD_ATTR_NUM_PART_SECURITY_FROZEN_KEYS_MASK_size;
	
	memcpy(buf_ptr,&(part_security_attr_page_struct.Partition_Key_ID),OSD_ATTR_NUM_PART_SECURITY_PARTITION_KEY_ID_size);
	buf_ptr+=OSD_ATTR_NUM_PART_SECURITY_PARTITION_KEY_ID_size;
	
	memcpy(buf_ptr,(part_security_attr_page_struct.Working_Key_ID),OSD_ATTR_NUM_PART_SECURITY_WORKING_KEY_ID_0_size*16);
	buf_ptr+=OSD_ATTR_NUM_PART_SECURITY_WORKING_KEY_ID_0_size*16;

	memcpy(buf_ptr,&(part_security_attr_page_struct.Policy_Access_Tag),OSD_ATTR_NUM_PART_SECURITY_POLICY_ACCESS_TAG_size);
	buf_ptr+=OSD_ATTR_NUM_PART_SECURITY_POLICY_ACCESS_TAG_size;

	memcpy(buf_ptr,&(part_security_attr_page_struct.User_Object_Policy_Access_Tag),OSD_ATTR_NUM_PART_SECURITY_USER_OBJ_POLICY_ACCESS_TAG_size);
	buf_ptr+=OSD_ATTR_NUM_PART_SECURITY_USER_OBJ_POLICY_ACCESS_TAG_size;


	if(get_attr_filename(part_id,0,OSD_PAGE_NUM_PART_SECURITY,filename,PART_OBJECT_TYPE))	 //part_id=part_id  obj_id =0 pageno=3
	{
		print_msg(ERROR_MSG,"Error in getting PART_SECURITY_PAGE file name for part_id =0x%llx obj_id=0 page_no=0x%lx ",part_id,OSD_PAGE_NUM_PART_SECURITY); 
		return 1;
	
	}

	if(file_write(filename,buf,PART_SECURITY_PAGE_SIZE,PART_SECURITY_PAGE_OFFSET_IN_FILE)){
		print_msg(ERROR_MSG,"Error in Writting PART_SECURITY_PAGE file  for part_id =0x%llx obj_id=0 page_no=0x%lx   filename=%s",part_id,OSD_PAGE_NUM_PART_SECURITY,filename); 
		return 1;
	}
	
	return 0;

}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

int create_uobject_directory_page(uint64_t part_id,uint64_t object_id)
{
	char buf[UOBJ_DIRECTORY_PAGE_SIZE];
	char *buf_ptr=buf;
	char vendor_id_buf[VENDOR_ID_SIZE]="INCITS  ";
	char page_attr_id_buf[ATTR_PAGE_ID_SIZE];
	char filename[FILENAME_SIZE];
	char space_buf[40]="                                        ";                                                                          

	strcpy(page_attr_id_buf,"T10 User Object Directory");
	strncat(page_attr_id_buf,space_buf,ATTR_PAGE_ID_SIZE-1-strlen("T10 User Object Directory"));
	memcpy(buf_ptr,vendor_id_buf,VENDOR_ID_SIZE);
	buf_ptr+=VENDOR_ID_SIZE;
	memcpy(buf_ptr,page_attr_id_buf,ATTR_PAGE_ID_SIZE);
	buf_ptr+=ATTR_PAGE_ID_SIZE;
	
	strcpy(page_attr_id_buf,"T10 User Object Information");
	strncat(page_attr_id_buf,space_buf,ATTR_PAGE_ID_SIZE-1-strlen("T10 User Object Information"));
	memcpy(buf_ptr,vendor_id_buf,VENDOR_ID_SIZE);
	buf_ptr+=VENDOR_ID_SIZE;
	memcpy(buf_ptr,page_attr_id_buf,ATTR_PAGE_ID_SIZE);
	buf_ptr+=ATTR_PAGE_ID_SIZE;
	
	strcpy(page_attr_id_buf,"T10 User Object Quotas");
	strncat(page_attr_id_buf,space_buf,ATTR_PAGE_ID_SIZE-1-strlen("T10 User Object Quotas"));
	memcpy(buf_ptr,vendor_id_buf,VENDOR_ID_SIZE);
	buf_ptr+=VENDOR_ID_SIZE;
	memcpy(buf_ptr,page_attr_id_buf,ATTR_PAGE_ID_SIZE);
	buf_ptr+=ATTR_PAGE_ID_SIZE;
	
	strcpy(page_attr_id_buf,"T10 User Object Timestamps");
	strncat(page_attr_id_buf,space_buf,ATTR_PAGE_ID_SIZE-1-strlen("T10 User Object Timestamps"));
	memcpy(buf_ptr,vendor_id_buf,VENDOR_ID_SIZE);
	buf_ptr+=VENDOR_ID_SIZE;
	memcpy(buf_ptr,page_attr_id_buf,ATTR_PAGE_ID_SIZE);
	buf_ptr+=ATTR_PAGE_ID_SIZE;
	
	strcpy(page_attr_id_buf,"T10 Collections");
	strncat(page_attr_id_buf,space_buf,ATTR_PAGE_ID_SIZE-1-strlen("T10 Collections"));
	memcpy(buf_ptr,vendor_id_buf,VENDOR_ID_SIZE);
	buf_ptr+=VENDOR_ID_SIZE;
	memcpy(buf_ptr,page_attr_id_buf,ATTR_PAGE_ID_SIZE);
	buf_ptr+=ATTR_PAGE_ID_SIZE;
	
	strcpy(page_attr_id_buf,"T10 User Object Policy/Security");
	strncat(page_attr_id_buf,space_buf,ATTR_PAGE_ID_SIZE-1-strlen("T10 User Object Policy/Security"));
	memcpy(buf_ptr,vendor_id_buf,VENDOR_ID_SIZE);
	buf_ptr+=VENDOR_ID_SIZE;
	memcpy(buf_ptr,page_attr_id_buf,ATTR_PAGE_ID_SIZE);
	buf_ptr+=ATTR_PAGE_ID_SIZE;
	//file_write
	if(get_attr_filename(part_id,object_id,OSD_PAGE_NUM_UOBJ_DIRECTORY,filename,UOBJ_OBJECT_TYPE)) //pageno=0
	{
		print_msg(ERROR_MSG,"Error in getting UOBJ_DIRECTORY_PAGE file name for part_id =0x%llx obj_id=0x%llx page_no=0x%lx ",part_id,object_id,OSD_PAGE_NUM_UOBJ_DIRECTORY); 
		return 1;
	
	
	}

	if(file_write(filename,buf,UOBJ_DIRECTORY_PAGE_SIZE,UOBJ_DIRECTORY_PAGE_OFFSET_IN_FILE))
	{
		print_msg(ERROR_MSG,"Error in Writting UOBJ_DIRECTORY_PAGE file  for part_id =0x%llx obj_id=0x%llx page_no=0x%lx   filename=%s",part_id,object_id,OSD_PAGE_NUM_UOBJ_DIRECTORY,filename); 
		return 1;
	}
	return 0;

}
//--------------------------------------------------------------------------------------
int create_uobject_information_page(uint64_t part_id,uint64_t object_id,struct uobject_information_attribute_page  uobject_info_attr_page_struct)
{
	char filename[FILENAME_SIZE];
	char buf[UOBJ_INFORMATION_PAGE_SIZE];
	char *buf_ptr=buf;	
	uint32_t page_no=OSD_PAGE_NUM_ROOT_INFO,page_size=UOBJ_INFORMATION_PAGE_SIZE;


	memset(buf,0,UOBJ_INFORMATION_PAGE_SIZE);
	memcpy(buf_ptr,&(page_no),PAGE_NO_SIZE);
	buf_ptr+=PAGE_NO_SIZE;

	memcpy(buf_ptr,&(page_size),PAGE_LENGTH_SIZE);
	buf_ptr+=PAGE_NO_SIZE;

	memcpy(buf_ptr,&(uobject_info_attr_page_struct.Partition_ID),OSD_ATTR_NUM_UOBJ_INFO_PARTITION_ID_size);
	buf_ptr+=OSD_ATTR_NUM_UOBJ_INFO_PARTITION_ID_size;
	
	memcpy(buf_ptr,&(uobject_info_attr_page_struct.User_Object_ID),OSD_ATTR_NUM_UOBJ_INFO_OBJECT_ID_size);
	buf_ptr+=OSD_ATTR_NUM_UOBJ_INFO_OBJECT_ID_size;

	memcpy(buf_ptr,(uobject_info_attr_page_struct.Username),OSD_ATTR_NUM_UOBJ_INFO_USERNAME_size);
	buf_ptr+=OSD_ATTR_NUM_UOBJ_INFO_USERNAME_size;


	memcpy(buf_ptr,&(uobject_info_attr_page_struct.Used_Capacity),OSD_ATTR_NUM_UOBJ_INFO_USED_CAPACITY_size);
	buf_ptr+=OSD_ATTR_NUM_UOBJ_INFO_USED_CAPACITY_size;

	memcpy(buf_ptr,&(uobject_info_attr_page_struct.User_Object_Logical_Length),OSD_ATTR_NUM_UOBJ_INFO_OBJ_LENGTH_size);
	buf_ptr+=OSD_ATTR_NUM_UOBJ_INFO_OBJ_LENGTH_size;

	//file_write
	if(get_attr_filename(part_id,object_id,OSD_PAGE_NUM_UOBJ_INFO,filename,UOBJ_OBJECT_TYPE)) //pageno=1
	{
		print_msg(ERROR_MSG,"Error in getting UOBJ_INFORMATION_PAGE file name for part_id =0x%llx obj_id=0x%llx page_no=0x%lx ",part_id,object_id,OSD_PAGE_NUM_UOBJ_INFO); 
		return 1;
	
	
	}

	if(file_write(filename,buf,UOBJ_INFORMATION_PAGE_SIZE,UOBJ_INFORMATION_PAGE_OFFSET_IN_FILE))
	{
		print_msg(ERROR_MSG,"Error in Writting UOBJ_INFORMATION_PAGE file  for part_id =0x%llx obj_id=0x%llx page_no=0x%lx   filename=%s",part_id,object_id,OSD_PAGE_NUM_UOBJ_INFO,filename); 
		return 1;
	}

	return 0;

}
	
	
//--------------------------------------------------------------------------------------
	
int create_uobject_quotas_page(uint64_t part_id,uint64_t object_id,struct uobject_quotas_attribute_page uobject_quotas_attr_page_struct)
{

	char filename[FILENAME_SIZE];
	char buf[UOBJ_QUOTAS_PAGE_SIZE];
	char *buf_ptr=buf;	
	uint32_t page_no=OSD_PAGE_NUM_ROOT_INFO,page_size=UOBJ_QUOTAS_PAGE_SIZE;


	memset(buf,0,UOBJ_QUOTAS_PAGE_SIZE);
	memcpy(buf_ptr,&(page_no),PAGE_NO_SIZE);
	buf_ptr+=PAGE_NO_SIZE;

	memcpy(buf_ptr,&(page_size),PAGE_LENGTH_SIZE);
	buf_ptr+=PAGE_NO_SIZE;

	memcpy(buf_ptr,&(uobject_quotas_attr_page_struct.Max_User_Object_Length),OSD_ATTR_NUM_UOBJ_QUOTAS_MAX_OBJ_LEN_size);
	buf_ptr+=OSD_ATTR_NUM_UOBJ_QUOTAS_MAX_OBJ_LEN_size;
	
	
	//file_write
	if(get_attr_filename(part_id,object_id,OSD_PAGE_NUM_UOBJ_QUOTAS,filename,UOBJ_OBJECT_TYPE)) //pageno=2
	{
		print_msg(ERROR_MSG,"Error in getting UOBJ_QUOTAS_PAGE file name for part_id =0x%llx obj_id=0x%llx page_no=0x%lx ",part_id,object_id,OSD_PAGE_NUM_UOBJ_QUOTAS); 
		return 1;
	}
	
	if(file_write(filename,buf,UOBJ_QUOTAS_PAGE_SIZE,UOBJ_QUOTAS_PAGE_OFFSET_IN_FILE))
	{
		print_msg(ERROR_MSG,"Error in Writting UOBJ_QUOTAS_PAGE file  for part_id =0x%llx obj_id=0x%llx page_no=0x%lx   filename=%s",part_id,object_id,OSD_PAGE_NUM_UOBJ_QUOTAS,filename); 
		return 1;
	}

	return 0;

}
//--------------------------------------------------------------------------------------	
int create_uobject_timestamps_page(uint64_t part_id,uint64_t object_id,struct uobject_timestamps_attribute_page  uobject_timestamps_attr_page_struct)
{

	char filename[FILENAME_SIZE];
	char buf[UOBJ_TIMESTAMPS_PAGE_SIZE];
	char *buf_ptr=buf;	
	uint32_t page_no=OSD_PAGE_NUM_ROOT_INFO,page_size=UOBJ_TIMESTAMPS_PAGE_SIZE;

	memset(buf,0,UOBJ_TIMESTAMPS_PAGE_SIZE);
	memcpy(buf_ptr,&(page_no),PAGE_NO_SIZE);
	buf_ptr+=PAGE_NO_SIZE;

	memcpy(buf_ptr,&(page_size),PAGE_LENGTH_SIZE);
	buf_ptr+=PAGE_NO_SIZE;

	
	memcpy(buf_ptr,&(uobject_timestamps_attr_page_struct.Created_Time),OSD_ATTR_NUM_UOBJ_TIMESTAMPS_CREATED_TIME_size);
	buf_ptr+=OSD_ATTR_NUM_UOBJ_TIMESTAMPS_CREATED_TIME_size;
	
	
	memcpy(buf_ptr,&(uobject_timestamps_attr_page_struct.Attr_Accessed_Time),OSD_ATTR_NUM_UOBJ_TIMESTAMPS_ATTR_ACCESSED_TIME_size);
	buf_ptr+=OSD_ATTR_NUM_UOBJ_TIMESTAMPS_ATTR_ACCESSED_TIME_size;
	

	memcpy(buf_ptr,&(uobject_timestamps_attr_page_struct.Attr_Modified_Time),OSD_ATTR_NUM_UOBJ_TIMESTAMPS_ATTR_MODIFIED_TIME_size);
	buf_ptr+=OSD_ATTR_NUM_UOBJ_TIMESTAMPS_ATTR_MODIFIED_TIME_size;
	
	memcpy(buf_ptr,&(uobject_timestamps_attr_page_struct.Data_Accessed_Time),OSD_ATTR_NUM_UOBJ_TIMESTAMPS_DATA_ACCESSED_TIME_size);
	buf_ptr+=OSD_ATTR_NUM_UOBJ_TIMESTAMPS_DATA_ACCESSED_TIME_size;
	

	memcpy(buf_ptr,&(uobject_timestamps_attr_page_struct.Data_Modified_Time),OSD_ATTR_NUM_UOBJ_TIMESTAMPS_DATA_MODIFIED_TIME_size);
	buf_ptr+=OSD_ATTR_NUM_UOBJ_TIMESTAMPS_DATA_MODIFIED_TIME_size;
	

	if(get_attr_filename(part_id,object_id,OSD_PAGE_NUM_UOBJ_TIMESTAMPS,filename,UOBJ_OBJECT_TYPE)) //pageno=3
	{
		print_msg(ERROR_MSG,"Error in getting UOBJ_TIMESTAMPS_PAGE file name for part_id =0x%llx obj_id=0x%llx page_no=0x%lx ",part_id,object_id,OSD_PAGE_NUM_UOBJ_TIMESTAMPS); 
		return 1;
	}

	if(file_write(filename,buf,UOBJ_TIMESTAMPS_PAGE_SIZE,UOBJ_TIMESTAMPS_PAGE_OFFSET_IN_FILE))
	{
		print_msg(ERROR_MSG,"Error in Writting UOBJ_TIMESTAMPS_PAGE file  for part_id =0x%llx obj_id=0x%llx page_no=0x%lx   filename=%s",part_id,object_id,OSD_PAGE_NUM_UOBJ_TIMESTAMPS,filename); 
		return 1;
	}
	
	return 0;



}
//--------------------------------------------------------------------------------------
	
int create_uobject_security_page(uint64_t part_id,uint64_t object_id,struct uobject_security_attribute_page uobject_security_attr_page_struct)
{


	char filename[FILENAME_SIZE];
	char buf[UOBJ_SECURITY_PAGE_SIZE];
	char *buf_ptr=buf;	
	uint32_t page_no=OSD_PAGE_NUM_ROOT_INFO,page_size=UOBJ_SECURITY_PAGE_SIZE;

	memset(buf,0,UOBJ_SECURITY_PAGE_SIZE);
	
	memcpy(buf_ptr,&(page_no),PAGE_NO_SIZE);
	buf_ptr+=PAGE_NO_SIZE;

	memcpy(buf_ptr,&(page_size),PAGE_LENGTH_SIZE);
	buf_ptr+=PAGE_NO_SIZE;

	
	memcpy(buf_ptr,&(uobject_security_attr_page_struct.Policy_Access_Tag),OSD_ATTR_NUM_UOBJ_SECURITY_POLICY_ACCESS_TAG_size);
	buf_ptr+=OSD_ATTR_NUM_UOBJ_SECURITY_POLICY_ACCESS_TAG_size;
	memcpy(buf_ptr,"xxxxxxxxxx",11);
		
	
	if(get_attr_filename(part_id,object_id, OSD_PAGE_NUM_UOBJ_SECURITY,filename,UOBJ_OBJECT_TYPE)) //pageno=5
	{
	print_msg(ERROR_MSG,"Error in getting UOBJ_SECURITY_PAGE file name for part_id =%llx obj_id=%llx page_no=0x%lx  ",part_id,object_id, OSD_PAGE_NUM_UOBJ_SECURITY); 
		return 1;

	}

	if(file_write(filename,buf,UOBJ_SECURITY_PAGE_SIZE,UOBJ_SECURITY_PAGE_OFFSET_IN_FILE))
	{
	print_msg(ERROR_MSG,"Error in writting UOBJ_SECURITY_PAGE file  for part_id =%llx obj_id=%llx page_no=0x%lx filename=%s  ",part_id,object_id, OSD_PAGE_NUM_UOBJ_SECURITY,filename); 
		return 1;
	
	}

	return 0;

}
//--------------------------------------------------------------------------------------
int create_uobject_collections_page(uint64_t part_id,uint64_t object_id,struct uobject_collections_attribute_page uobject_collections_attr_page_struct)
{


	char filename[FILENAME_SIZE];
	char buf[UOBJ_COLLECTIONS_PAGE_SIZE];
	char *buf_ptr=buf;	
	uint32_t page_no=OSD_PAGE_NUM_ROOT_INFO,page_size=UOBJ_COLLECTIONS_PAGE_SIZE;

	memcpy(buf_ptr,&(page_no),PAGE_NO_SIZE);
	buf_ptr+=PAGE_NO_SIZE;

	memcpy(buf_ptr,&(page_size),PAGE_LENGTH_SIZE);
	buf_ptr+=PAGE_NO_SIZE;


	if(get_attr_filename(part_id,object_id,OSD_PAGE_NUM_T10_COLLECTIONS,filename,UOBJ_OBJECT_TYPE)) //pageno=5
	{
	print_msg(ERROR_MSG,"Error in getting UOBJ_COLLECTIONS_PAGE file name for part_id =0x%llx obj_id=0x%llx page_no=0x%lx ",part_id,object_id,OSD_PAGE_NUM_T10_COLLECTIONS); 
		return 1;
	
	
	}
	
	if(file_write(filename,buf,UOBJ_COLLECTIONS_PAGE_SIZE,UOBJ_COLLECTIONS_PAGE_OFFSET_IN_FILE))
	{
	print_msg(ERROR_MSG,"Error in writting UOBJ_COLLECTIONS_PAGE file  for part_id =0x%llx obj_id=0x%llx page_no=0x%lx filename=%s ",part_id,object_id,OSD_PAGE_NUM_T10_COLLECTIONS,filename); 
		return 1;
	
	}
	return 0;

}

//--------------------------------------------------------------------------------------
	
int create_collection_directory_page(uint64_t part_id,uint64_t collection_object_id)
{

	char buf[COLL_DIRECTORY_PAGE_SIZE];
	char *buf_ptr=buf;
	char vendor_id_buf[VENDOR_ID_SIZE]="INCITS  ";
	char page_attr_id_buf[ATTR_PAGE_ID_SIZE];
	char filename[FILENAME_SIZE];
	char space_buf[40]="                                        ";                      
	
	strcpy(page_attr_id_buf,"T10 Collection Directory");
	strncat(page_attr_id_buf,space_buf,ATTR_PAGE_ID_SIZE-1-strlen("T10 Collection Directory"));
	memcpy(buf_ptr,vendor_id_buf,VENDOR_ID_SIZE);
	buf_ptr+=VENDOR_ID_SIZE;
	memcpy(buf_ptr,page_attr_id_buf,ATTR_PAGE_ID_SIZE);
	buf_ptr+=ATTR_PAGE_ID_SIZE;
	
	
	strcpy(page_attr_id_buf,"T10 Collection Information");
	strncat(page_attr_id_buf,space_buf,ATTR_PAGE_ID_SIZE-1-strlen("T10 Collection Information"));
	memcpy(buf_ptr,vendor_id_buf,VENDOR_ID_SIZE);
	buf_ptr+=VENDOR_ID_SIZE;
	memcpy(buf_ptr,page_attr_id_buf,ATTR_PAGE_ID_SIZE);
	buf_ptr+=ATTR_PAGE_ID_SIZE;
	
	
	strcpy(page_attr_id_buf,"T10 Collection Timestamps");
	strncat(page_attr_id_buf,space_buf,ATTR_PAGE_ID_SIZE-1-strlen("T10 Collection Timestamps"));
	memcpy(buf_ptr,vendor_id_buf,VENDOR_ID_SIZE);
	buf_ptr+=VENDOR_ID_SIZE;
	memcpy(buf_ptr,page_attr_id_buf,ATTR_PAGE_ID_SIZE);
	buf_ptr+=ATTR_PAGE_ID_SIZE;
	
	strcpy(page_attr_id_buf,"T10 Collection Policy/Security");
	strncat(page_attr_id_buf,space_buf,ATTR_PAGE_ID_SIZE-1-strlen("T10 Collection Policy/Security"));
	memcpy(buf_ptr,vendor_id_buf,VENDOR_ID_SIZE);
	buf_ptr+=VENDOR_ID_SIZE;
	memcpy(buf_ptr,page_attr_id_buf,ATTR_PAGE_ID_SIZE);
	buf_ptr+=ATTR_PAGE_ID_SIZE;
	
	//file_write
	if(get_attr_filename(part_id,collection_object_id, OSD_PAGE_NUM_COLL_DIRECTORY,filename,COLL_OBJECT_TYPE)) //pageno=0
	{
		print_msg(ERROR_MSG,"Error in getting COLL_DIRECTORY_PAGE file name for part_id =0x%llx collection_obj_id=0x%llx page_no=0x%lx ",part_id,collection_object_id, OSD_PAGE_NUM_COLL_DIRECTORY); 
		return 1;
	}

	if(file_write(filename,buf,COLL_DIRECTORY_PAGE_SIZE,COLL_DIRECTORY_PAGE_OFFSET_IN_FILE))
	{

		print_msg(ERROR_MSG,"Error in writting COLL_DIRECTORY_PAGE file  for part_id =0x%llx coll_obj_id=0x%llx page_no=0x%lx   filename=%s",part_id,collection_object_id, OSD_PAGE_NUM_COLL_DIRECTORY,filename); 
		return 1;
	
	}
	return 0;	
}
//--------------------------------------------------------------------------------------
int create_collection_information_page(uint64_t part_id,uint64_t collection_object_id,struct collection_information_attribute_page  collection_info_attr_page_struct)
{

	char filename[FILENAME_SIZE];
	char buf[COLL_INFORMATION_PAGE_SIZE];
	char *buf_ptr=buf;	
	uint32_t page_no=OSD_PAGE_NUM_ROOT_INFO,page_size=COLL_INFORMATION_PAGE_SIZE;


	memset(buf_ptr,0,COLL_INFORMATION_PAGE_SIZE);
	memcpy(buf_ptr,&(page_no),PAGE_NO_SIZE);
	buf_ptr+=PAGE_NO_SIZE;

	memcpy(buf_ptr,&(page_size),PAGE_LENGTH_SIZE);
	buf_ptr+=PAGE_NO_SIZE;

	
	memcpy(buf_ptr,&(collection_info_attr_page_struct.Partition_ID),OSD_ATTR_NUM_COLL_INFO_PARTITION_ID_size);
	buf_ptr+=OSD_ATTR_NUM_COLL_INFO_PARTITION_ID_size;
	
	memcpy(buf_ptr,&(collection_info_attr_page_struct.Collection_Object_ID),OSD_ATTR_NUM_COLL_INFO_OBJECT_ID_size); 
	buf_ptr+=OSD_ATTR_NUM_COLL_INFO_OBJECT_ID_size ;
	
	memcpy(buf_ptr,(collection_info_attr_page_struct.Username),OSD_ATTR_NUM_COLL_INFO_USERNAME_size);
	buf_ptr+=OSD_ATTR_NUM_COLL_INFO_USERNAME_size;
	
	memcpy(buf_ptr,&(collection_info_attr_page_struct.Used_Capacity),OSD_ATTR_NUM_COLL_INFO_USED_CAPACITY_size);
	buf_ptr+=OSD_ATTR_NUM_COLL_INFO_USED_CAPACITY_size;


	//file_write
	if(get_attr_filename(part_id,collection_object_id,OSD_PAGE_NUM_COLL_INFO,filename,COLL_OBJECT_TYPE)) //pageno=1
	{
		print_msg(ERROR_MSG,"Error in getting COLL_INFORMATION_PAGE file name for part_id =0x%llx collection_obj_id=0x%llx page_no=0x%lx ",part_id,collection_object_id,OSD_PAGE_NUM_COLL_INFO); 
		return 1;

	}

	if(file_write(filename,buf,COLL_INFORMATION_PAGE_SIZE,COLL_INFORMATION_PAGE_OFFSET_IN_FILE))
	{
	
		print_msg(ERROR_MSG,"Error in Writting COLL_INFORMATION_PAGE file  for part_id =0x%llx coll_obj_id=0x%llx page_no=0x%lx   filename=%s",part_id,collection_object_id,OSD_PAGE_NUM_COLL_INFO ,filename); 
		return 1;
	
	}
	return 0;	



}
//--------------------------------------------------------------------------------------	
int create_collection_timestamps_page(uint64_t part_id,uint64_t collection_object_id,struct collection_timestamps_attribute_page collection_timestamps_attr_page_struct)
{

	char filename[FILENAME_SIZE];
	char buf[COLL_TIMESTAMPS_PAGE_SIZE];
	char *buf_ptr=buf;	
	uint32_t page_no=OSD_PAGE_NUM_ROOT_INFO,page_size=COLL_TIMESTAMPS_PAGE_SIZE;

	memset(buf,0,COLL_TIMESTAMPS_PAGE_SIZE);
	memcpy(buf_ptr,&(page_no),PAGE_NO_SIZE);
	buf_ptr+=PAGE_NO_SIZE;

	memcpy(buf_ptr,&(page_size),PAGE_LENGTH_SIZE);
	buf_ptr+=PAGE_NO_SIZE;

	

	memcpy(buf_ptr,&(collection_timestamps_attr_page_struct.Created_Time),OSD_ATTR_NUM_COLL_TIMESTAMPS_CREATED_TIME_size);
	buf_ptr+=OSD_ATTR_NUM_COLL_TIMESTAMPS_CREATED_TIME_size;

	
	memcpy(buf_ptr,&(collection_timestamps_attr_page_struct.Attr_Accessed_Time),OSD_ATTR_NUM_COLL_TIMESTAMPS_ATTR_ACCESSED_TIME_size);
	buf_ptr+=OSD_ATTR_NUM_COLL_TIMESTAMPS_ATTR_ACCESSED_TIME_size ;

	memcpy(buf_ptr,&(collection_timestamps_attr_page_struct.Attr_Modified_Time),OSD_ATTR_NUM_COLL_TIMESTAMPS_ATTR_MODIFIED_TIME_size);
	buf_ptr+=OSD_ATTR_NUM_COLL_TIMESTAMPS_ATTR_MODIFIED_TIME_size;

	memcpy(buf_ptr,&(collection_timestamps_attr_page_struct.Data_Accessed_Time),OSD_ATTR_NUM_COLL_TIMESTAMPS_DATA_ACCESSED_TIME_size);
	buf_ptr+=OSD_ATTR_NUM_COLL_TIMESTAMPS_DATA_ACCESSED_TIME_size;


	memcpy(buf_ptr,&(collection_timestamps_attr_page_struct.Data_Modified_Time),OSD_ATTR_NUM_COLL_TIMESTAMPS_DATA_MODIFIED_TIME_size);
	buf_ptr+=OSD_ATTR_NUM_COLL_TIMESTAMPS_DATA_MODIFIED_TIME_size;

	
 //file_write
	if(get_attr_filename(part_id,collection_object_id,OSD_PAGE_NUM_COLL_TIMESTAMPS,filename,COLL_OBJECT_TYPE)) //pageno=1
	{
	print_msg(ERROR_MSG,"Error in getting COLL_TIMESTAMPS_PAGE file name for part_id =0x%llx collection_obj_id=0x%llx page_no=0x%lx ",part_id,collection_object_id,OSD_PAGE_NUM_COLL_TIMESTAMPS); 
		return 1;
	
	
	}

	if(file_write(filename,buf,COLL_TIMESTAMPS_PAGE_SIZE,COLL_TIMESTAMPS_PAGE_OFFSET_IN_FILE))
	{
	
		print_msg(ERROR_MSG,"Error in Writting COLL_TIMESTAMPS_PAGE file  for part_id =0x%llx coll_obj_id=0x%llx page_no=0x%lx   filename=%s",part_id,collection_object_id,OSD_PAGE_NUM_COLL_TIMESTAMPS,filename); 
		return 1;
	
	}
	return 0;	

}
//-------------------------------------------------------------------------------------
int create_collection_security_page(uint64_t part_id,uint64_t collection_object_id ,struct collection_security_attribute_page collection_security_attr_page_struct)
{

	char filename[FILENAME_SIZE];
	char buf[COLL_SECURITY_PAGE_SIZE];
	char *buf_ptr=buf;	
	uint32_t page_no=OSD_PAGE_NUM_ROOT_INFO,page_size=COLL_SECURITY_PAGE_SIZE;


	memset(buf,0,COLL_SECURITY_PAGE_SIZE);
	memcpy(buf_ptr,&(page_no),PAGE_NO_SIZE);
	buf_ptr+=PAGE_NO_SIZE;

	memcpy(buf_ptr,&(page_size),PAGE_LENGTH_SIZE);
	buf_ptr+=PAGE_NO_SIZE;


	memcpy(buf_ptr,&(collection_security_attr_page_struct.Policy_Access_Tag),OSD_ATTR_NUM_COLL_SECURITY_POLICY_ACCESS_TAG_size);
	buf_ptr+=OSD_ATTR_NUM_COLL_SECURITY_POLICY_ACCESS_TAG_size;

	if(get_attr_filename(part_id,collection_object_id,OSD_PAGE_NUM_COLL_SECURITY,filename,COLL_OBJECT_TYPE)) //pageno=1
	{
		print_msg(ERROR_MSG,"Error in getting COLL_SECURITY_PAGE file name for part_id =0x%llx collection_obj_id=0x%llx page_no=0x%lx ",part_id,collection_object_id,OSD_PAGE_NUM_COLL_SECURITY); 
		return 1;
	}
	
	if(file_write(filename,buf,COLL_SECURITY_PAGE_SIZE,COLL_SECURITY_PAGE_OFFSET_IN_FILE))
	{
		print_msg(ERROR_MSG,"Error in writting COLL_SECURITY_PAGE file  for part_id =0x%llx coll_obj_id=0x%llx page_no=0x%lx   filename=%s",part_id,collection_object_id,OSD_PAGE_NUM_COLL_SECURITY ,filename); 
		return 1;
	
	}
	
	return 0;	
}

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------
	
