/**
 *                      ==================
 *                      IITK OSD Simulator
 *                      ==================
 *   An Implementation of Simulator for Object based storage device(OSD)
 *
 *   Copyright (C) 2007-2008 CSE IIT KANPUR
 *
 *   Contact: Department of Computer Science and Engineering
 *            Indian Institute of Technology,Kanpur(IIT Kanpur)
 *            {moona,riyaz}@cse.iitk.ac.in
 *
 *   This file is part of IITK OSD Simulator Project.
 *
 *   IITK OSD Simulator Project is free software; you can redistribute it
 *   and/or modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   IITK OSD Simulator Project is distributed in the hope that it will be
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the  GNU General
 *   Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#ifndef OSD_ATTR1_H
#define OSD_ATTR1_H

# include<stdint.h>
# include "osd_attr.h"	


/**
**********************************
 * File:  osd_attr1.h
 * Declaration of Structure for Page storage 
 * Definition of  structure variable for each page
 * Functions to initialize each page of each object type
 **/

//-------------------------------------------------------------------------
/** Macros for PAGE SIZE of each ROOT attribute page 
    and their offset in ROOT attribute file */
# define ROOT_DIRECTORY_PAGE_SIZE  200
# define ROOT_INFORMATION_PAGE_SIZE  358
//360
# define ROOT_QUOTAS_PAGE_SIZE  55     
	//55
# define ROOT_TIMESTAMPS_PAGE_SIZE  30 
	//30
# define ROOT_SECURITY_PAGE_SIZE  79
	//80

# define  ROOT_PAGES_SIZE ROOT_DIRECTORY_PAGE_SIZE+ROOT_INFORMATION_PAGE_SIZE+ROOT_QUOTAS_PAGE_SIZE+ROOT_TIMESTAMPS_PAGE_SIZE+ROOT_SECURITY_PAGE_SIZE

# define ROOT_DIRECTORY_PAGE_OFFSET_IN_FILE  0
# define ROOT_INFORMATION_PAGE_OFFSET_IN_FILE 200
# define ROOT_QUOTAS_PAGE_OFFSET_IN_FILE  560   
# define ROOT_TIMESTAMPS_PAGE_OFFSET_IN_FILE  615 
# define ROOT_SECURITY_PAGE_OFFSET_IN_FILE  645
//-------------------------------------------------------------------------

/** Macros for PAGE SIZE of each Partition attribute page 
    and their offset in Partition attribute file */

# define PART_DIRECTORY_PAGE_SIZE  200
# define PART_INFORMATION_PAGE_SIZE  175
//175
# define PART_QUOTAS_PAGE_SIZE  50  
//50 
# define PART_TIMESTAMPS_PAGE_SIZE  50 
//50
# define PART_SECURITY_PAGE_SIZE  170
//170

# define PART_PAGES_SIZE PART_DIRECTORY_PAGE_SIZE+PART_INFORMATION_PAGE_SIZE+PART_QUOTAS_PAGE_SIZE+PART_TIMESTAMPS_PAGE_SIZE+PART_SECURITY_PAGE_SIZE

# define PART_DIRECTORY_PAGE_OFFSET_IN_FILE  0
# define PART_INFORMATION_PAGE_OFFSET_IN_FILE  200
# define PART_QUOTAS_PAGE_OFFSET_IN_FILE  375   
# define PART_TIMESTAMPS_PAGE_OFFSET_IN_FILE  425
# define PART_SECURITY_PAGE_OFFSET_IN_FILE  475

//-------------------------------------------------------------------------
/** Macros for PAGE SIZE of each User Object attribute page 
    and their offset in User Object attribute file */

# define UOBJ_DIRECTORY_PAGE_SIZE  240
//240
# define UOBJ_INFORMATION_PAGE_SIZE  185
//185
# define UOBJ_QUOTAS_PAGE_SIZE  25   
//25
# define UOBJ_TIMESTAMPS_PAGE_SIZE  50 
//50
# define UOBJ_SECURITY_PAGE_SIZE  25
//25
# define UOBJ_COLLECTIONS_PAGE_SIZE  8
//50

# define UOBJ_PAGES_SIZE  UOBJ_DIRECTORY_PAGE_SIZE+UOBJ_INFORMATION_PAGE_SIZE+UOBJ_QUOTAS_PAGE_SIZE+UOBJ_TIMESTAMPS_PAGE_SIZE+UOBJ_SECURITY_PAGE_SIZE+UOBJ_COLLECTIONS_PAGE_SIZE

# define UOBJ_DIRECTORY_PAGE_OFFSET_IN_FILE  0
# define UOBJ_INFORMATION_PAGE_OFFSET_IN_FILE  240 
# define UOBJ_QUOTAS_PAGE_OFFSET_IN_FILE   425
# define UOBJ_TIMESTAMPS_PAGE_OFFSET_IN_FILE  450
# define UOBJ_SECURITY_PAGE_OFFSET_IN_FILE   500
# define UOBJ_COLLECTIONS_PAGE_OFFSET_IN_FILE  525
//-------------------------------------------------------------------------

/** Macros for PAGE SIZE of each Collection attribute page 
    and their offset in Collection attribute file */

# define COLL_DIRECTORY_PAGE_SIZE  240
//240
# define COLL_INFORMATION_PAGE_SIZE  175
//175
# define COLL_TIMESTAMPS_PAGE_SIZE  50 
//50
# define COLL_SECURITY_PAGE_SIZE  20
//20

#  define COLL_PAGES_SIZE COLL_DIRECTORY_PAGE_SIZE+COLL_INFORMATION_PAGE_SIZE+COLL_TIMESTAMPS_PAGE_SIZE+COLL_SECURITY_PAGE_SIZE

# define COLL_DIRECTORY_PAGE_OFFSET_IN_FILE   0
# define COLL_INFORMATION_PAGE_OFFSET_IN_FILE  240
# define COLL_TIMESTAMPS_PAGE_OFFSET_IN_FILE 415
# define COLL_SECURITY_PAGE_OFFSET_IN_FILE  465
//-------------------------------------------------------------------------

/** Macros for PAGE SIZE of Current command page */

# define CURRENT_COMMAND_PAGE_SIZE 55 
  //45+8
//-------------------------------------------------------------------------
//Except directory and info page, we strore page no(4) and page length(4) in each page 

/** Structure for root information page */
struct root_information_attribute_page
{
uint8_t OSD_System_ID[20];
uint8_t Vendor_ID[8];
uint8_t Product_ID[16];
uint8_t Product_Model[32];
uint32_t Product_Rev_Level;
uint8_t Product_Serial_No[100];
uint8_t OSD_Name[100];
uint64_t Total_Capacity;
uint64_t Used_Capacity;
uint64_t No_of_Partitions;
uint48_t Clock;//6 bytes 
};

/** Structure for partition information page */
struct partition_information_attribute_page
{
uint64_t Partition_ID;
uint8_t Username[100];
uint64_t Used_Capacity;
uint64_t No_CollectionUser_Objects;  
};

/** Structure for user object information page */
struct uobject_information_attribute_page
{
uint64_t Partition_ID;
uint64_t User_Object_ID;
uint8_t Username[100];
uint64_t Used_Capacity;
uint64_t User_Object_Logical_Length;

};

/** Structure for collection information page */
struct collection_information_attribute_page
{
uint64_t Partition_ID;
uint64_t Collection_Object_ID;
uint8_t Username[100];
uint64_t Used_Capacity;
};


/** Structure for root quotas page */
struct root_quotas_attribute_page
{
uint64_t Default_Max_User_Object_Length;
uint64_t Partition_Capacity_Quota;
uint64_t Partition_Object_Count;
uint32_t Partition_Collections_Per_User_Object;
uint64_t Partition_Count;
};

/** Structure for partition quotas page */
struct partition_quotas_attribute_page
{
uint64_t Default_Max_User_Object_Length;
uint64_t Capacity_Quota;
uint64_t Object_Count;
uint32_t Collections_Per_User_Object;
};

/** Structure for user object quotas page */
struct uobject_quotas_attribute_page
{
uint64_t Max_User_Object_Length;
};

/** Structure for root timestamps page */
struct root_timestamps_attribute_page
{
uint48_t Attr_Accessed_Time;
uint48_t Attr_Modified_Time;
uint8_t Timestamp_Bypass;
};

/** Structure for partition timestamps page */
struct partition_timestamps_attribute_page
{
uint48_t Created_Time;
uint48_t Attr_Accessed_Time;
uint48_t Attr_Modified_Time;
uint48_t Data_Accessed_Time;
uint48_t Data_Modified_Time;
uint8_t Timestamp_Bypass;
};
/** Structure for user object timestamps page */
struct uobject_timestamps_attribute_page
{
uint48_t Created_Time;
uint48_t Attr_Accessed_Time;
uint48_t Attr_Modified_Time;
uint48_t Data_Accessed_Time;
uint48_t Data_Modified_Time;
};

/** Structure for collection object timestamps page */
struct collection_timestamps_attribute_page
{
uint48_t Created_Time;
uint48_t Attr_Accessed_Time;
uint48_t Attr_Modified_Time;
uint48_t Data_Accessed_Time;
uint48_t Data_Modified_Time;
};

/** Structure for user object collections page */
struct uobject_collections_attribute_page
{
uint64_t *Collection_Pointer;
};

/** Structure for root object security  page */
struct root_security_attribute_page
{
uint8_t Default_Security_Method;
uint48_t Oldest_Valid_Nonce_Limit;
uint48_t Newest_Valid_Nonce_Limit;
uint8_t Partition_Default_Security_Method;
uint16_t Supported_Security_Methods;
uint48_t Adjustable_Clock;
uint56_t Master_Key_ID;
uint56_t Root_Key_ID;
uint8_t Supported_Integrity_Chk_Value_Algorithm[16];
uint8_t Supported_DH_Group[16];
};

/** Structure for partition object security  page */
struct partition_security_attribute_page
{
uint8_t Default_Security_Method;
uint48_t Oldest_Valid_Nonce;
uint48_t Newest_Valid_Nonce;
uint16_t Request_Nonce_List_Depth;
uint16_t Frozen_Working_Key_Bit_Mask;
uint56_t Partition_Key_ID;
uint56_t Working_Key_ID[16];
uint32_t Policy_Access_Tag;
uint32_t User_Object_Policy_Access_Tag;
};

/** Structure for user object security  page */
struct uobject_security_attribute_page
{
uint32_t Policy_Access_Tag;

};

/** Structure for collection object security  page */
struct collection_security_attribute_page
{
uint32_t Policy_Access_Tag;

};


/** Structure for current command   page */
typedef struct current_command_attribute_page
{
uint8_t Response_Integirty_Chk_Value[OSD_ATTR_NUM_CUR_CMD_RESPONSE_INT_CHECK_VAL_size];
uint8_t Object_Type;
uint64_t Partition_ID;
uint64_t CollectionUser_Object_ID;
uint64_t Starting_Byte_Address_of_Append;
}CURRENT_COMMAND_ATTRIBUTE_PAGE;


/**
 *
 *    Functions:     create_root_directory_page
 *		     create_root_information_page
 *		     create_root_quotas_page
 *		     create_root_timestamps_page
 *		     create_root_security_page
 *
 *
 *    Description:  Creates Root Attribute Pages
 *
 *    Arguments:   respective structures  
 *	
 *    Return codes: 0 if success,1 otherwise 
 *
 **/

	int create_root_directory_page( );
	int create_root_information_page(struct root_information_attribute_page );
	int create_root_quotas_page(struct root_quotas_attribute_page );
	int create_root_timestamps_page(struct root_timestamps_attribute_page );
	int create_root_security_page(struct root_security_attribute_page);


/**
 *
 *    Functions:     create_root_attribute_pages
 *
 *    Description:  Parent function which calls functions to create each Root Attribute 
 *		    page
 *
 *    Arguments:    @osd_system_id - OSD system Identifier
		    @capacity - Capacity of OSD Lun	 
 *	
 *    Return codes: 0 if success,1 otherwise 
 *
 **/

int create_root_attribute_pages(const char *osd_system_id,uint64_t capacity); 
	
	

/**
 *
 *    Functions: create_partition_directory_page    
 *		 create_partition_information_page
 *		 create_partition_quotas_page
 *		 create_partition_timestamps_page
 *		 create_partition_security_page	
 *
 *    Description:  Creates Partition Attribute Pages
 *
 *    Arguments: part_id and respective structures  
 *	
 *    Return codes: 0 if success, 1 otherwise 
 *
 **/

	int create_partition_directory_page(uint64_t );
	int create_partition_information_page(uint64_t,struct partition_information_attribute_page );
	int create_partition_quotas_page(uint64_t,struct partition_quotas_attribute_page );
	int create_partition_timestamps_page(uint64_t,struct partition_timestamps_attribute_page );
	int create_partition_security_page(uint64_t,struct partition_security_attribute_page);

/**
 *
 *    Functions:     create_partition_attribute_pages
 *
 *    Description:  Parent function which calls functions to create each Partition
 *		    Attribute page
 *
 *    Arguments:    @part_id  - Partition ID
		    @username - User Name for the partition
 *	
 *    Return codes: 0 if success,1 otherwise 
 *
 **/

int create_partition_attribute_pages(uint64_t part_id,const char *username);
	

/**
 *
 *    Functions: create_uobject_directory_page    
 *		 create_uobject_information_page
 *		 create_uobject_quotas_page
 *		 create_uobject_timestamps_page
 *		 create_uobject_security_page
 * 		 create_uobject_collections_page
 *		 
 *    Description:  Creates Partition Attribute Pages
 *
 *    Arguments: part_id,object_id and respective structures  
 *	
 *    Return codes: 0 if success, 1 otherwise 
 *
 **/
	int create_uobject_directory_page(uint64_t,uint64_t);
	int create_uobject_information_page(uint64_t,uint64_t,struct uobject_information_attribute_page );
	int create_uobject_quotas_page(uint64_t,uint64_t,struct uobject_quotas_attribute_page );
	int create_uobject_timestamps_page(uint64_t,uint64_t,struct uobject_timestamps_attribute_page );
	int create_uobject_security_page(uint64_t,uint64_t,struct uobject_security_attribute_page);
	int create_uobject_collections_page(uint64_t,uint64_t,struct uobject_collections_attribute_page);


/**
 *
 *    Functions:    create_uobject_attribute_pages
 *
 *    Description:  Parent function which calls functions to create Object
 *		    Attribute page
 *
 *    Arguments:    @part_id  - Partition ID
 *		    @object_id - Object ID	
 *		    @username - User Name for the object
 *	
 *    Return codes: 0 if success,1 otherwise 
 *
 **/


int create_uobject_attribute_pages(uint64_t part_id,uint64_t object_id,const char *username);
	

/**
 *
 *    Functions: create_collection_directory_page    
 *		 create_collection_information_page
 *		 create_collection_timestamps_page
 *		 create_collection_security_page
 *		 
 *    Description:  Creates Collection Attribute Pages
 *
 *    Arguments: part_id ,collection_object_id and respective structures  
 *	
 *    Return codes: 0 if success, 1 otherwise 
 *
 **/

	int create_collection_directory_page(uint64_t,uint64_t);
	int create_collection_information_page(uint64_t,uint64_t,struct collection_information_attribute_page );
	int create_collection_timestamps_page(uint64_t,uint64_t,struct collection_timestamps_attribute_page );
	int create_collection_security_page(uint64_t,uint64_t,struct collection_security_attribute_page);

/**
 *
 *    Functions:    create_collection_attribute_pages
 *
 *    Description:  Parent function which calls functions to create Collection
 *		    Attribute page
 *
 *    Arguments:    @part_id  - Partition ID
 *		    @collection_object_id - Collection Object ID	
 *		    @username - User Name for the object
 *	
 *    Return codes: 0 if success,1 otherwise 
 *
 **/

int create_collection_attribute_pages(uint64_t part_id,uint64_t collection_object_id,const char *username);


/**
 *
 *    Functions: create_command_attribute_page    
 *		 
 *    Description:  Creates Command Attribute Page
 *
 *    Arguments: @current_command_attribute_page - Command attribute page structure
 *	
 *    Return codes: 0 if success, 1 otherwise 
 *
 **/
int create_command_attribute_page(struct current_command_attribute_page);
	
# endif	 /* ifdef OSD_ATTR1_H */ 
