/**
 *                      ==================
 *                      IITK OSD Simulator
 *                      ==================
 *   An Implementation of Simulator for Object based storage device(OSD)
 *
 *   Copyright (C) 2007-2008 CSE IIT KANPUR
 *
 *   Contact: Department of Computer Science and Engineering
 *            Indian Institute of Technology,Kanpur(IIT Kanpur)
 *            {moona,riyaz}@cse.iitk.ac.in
 *
 *   This file is part of IITK OSD Simulator Project.
 *
 *   IITK OSD Simulator Project is free software; you can redistribute it
 *   and/or modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   IITK OSD Simulator Project is distributed in the hope that it will be
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the  GNU General
 *   Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
# include "osd_attr2.h"
# include "osd_time.h"

/**
**********************************
 * File:  osd_attr2.c
 * Implements functions for changes in attribute pages as side effect of OSD command * 
 * execution
 **/

extern int DEBUG;
//Define  debug level as 2 for this file
static int dlevel=LOWER_FUNC;
//------------------------------------------------------------------------------------
int osd_format_attribute_pages_effects(const char *osd_system_id,uint64_t capacity,const char *username)
{ 
	int ret;
 	print_msg(dlevel," osd_system_id =%s   capacity=0x%llx  username=%s  ",osd_system_id,capacity,username);
 	if ((ret=create_root_attribute_pages(osd_system_id,capacity)))
 	{
 		print_msg(ERROR_MSG,"Error in create_root_attribute_pages osd_system_id=%s capacity=0x%llx   ",osd_system_id,capacity);
		return ret;
 
 	}
 
 
	if((ret=create_partition_attribute_pages_effects(0,username)))
	{
		print_msg(ERROR_MSG,"Error in create_partition_attribute_pages_effects osd_system_id=%s capacity=0x%llx part_id=0   ",osd_system_id,capacity);
		return ret; 
	}
	
	return 0;
}

//------------------------------------------------------------------------------------
//Username =NULL except for partition 0
int create_partition_attribute_pages_effects(uint64_t part_id,const char *username)
{

	uint64_t root_no_of_partitions,root_partition_quota;
	uint64_t root_total_capacity,root_used_capacity;
	uint48_t timestamp_val;
	char user_name[100];
	//get_page_attr(uint64_t part_id,uint64_t  object_id, uint64_t page_no,uint64_t attr_no,void *attr_value,int object_type)
	if(part_id !=0)
	{
		get_page_attr(0ULL,0ULL,OSD_PAGE_NUM_PART_INFO,OSD_ATTR_NUM_PART_INFO_USERNAME,user_name,PART_OBJECT_TYPE);
	}
	else
		memcpy(user_name,username,100);
	
	get_page_attr(0ULL,0ULL,OSD_PAGE_NUM_ROOT_INFO,OSD_ATTR_NUM_ROOT_INFO_NUM_OF_PARTITIONS,&root_no_of_partitions,ROOT_OBJECT_TYPE);
	get_page_attr(0ULL,0ULL,OSD_PAGE_NUM_ROOT_QUOTAS,OSD_ATTR_NUM_ROOT_QUOTAS_PARTITION_COUNT,&root_partition_quota,ROOT_OBJECT_TYPE);

		
	if(root_no_of_partitions ==  root_partition_quota )
	{
			print_msg(ERROR_MSG,"No of Partition Quota Exceed: Cant Add New partition  ");
			print_msg(ERROR_MSG, "No of partitions = 0x%llx  ",root_partition_quota);
			return ROOT_NO_PART_OBJ_QUOTA_EXCEED;
	}

	print_msg(dlevel, "No of partitions= 0x%llx  Partition quota=0x%llx   ",root_no_of_partitions,root_partition_quota);

	get_page_attr(0ULL,0ULL,OSD_PAGE_NUM_ROOT_INFO,OSD_ATTR_NUM_ROOT_INFO_TOTAL_CAPACITY,&root_total_capacity,ROOT_OBJECT_TYPE);
	get_page_attr(0ULL,0ULL,OSD_PAGE_NUM_ROOT_INFO ,OSD_ATTR_NUM_ROOT_INFO_USED_CAPACITY ,&root_used_capacity,ROOT_OBJECT_TYPE);
	
	print_msg(dlevel, "Total Capacity = 0x%llx  Used Capacity =0x%llx Page Size=0x%x    ",root_total_capacity,root_used_capacity,PART_PAGES_SIZE);
	
	if(root_used_capacity+PART_PAGES_SIZE >root_total_capacity )
	{
			print_msg(ERROR_MSG,"Root Total Caapcity Exceed: Cant Create New Parttiton Pages   ");
			print_msg(ERROR_MSG, "Total Capacity = 0x%llx  Used Capacity =0x%llx Part Page Size =0x%x  ",root_total_capacity,root_used_capacity,PART_PAGES_SIZE);
			return ROOT_CAPACITY_QUOTA_EXCEED;
	
	}

	if (create_partition_attribute_pages(part_id,user_name))
	{
			
		print_msg(ERROR_MSG,"Error in create_partition_attribute_pages part_id=0x%llx user_name=%s   ",part_id,user_name);
		return 1;
	}
	
	root_used_capacity=root_used_capacity+PART_PAGES_SIZE;
	root_no_of_partitions++;
	print_msg(dlevel," Update Root No of partition and Root Used Capacity  ");
	set_page_attr(0ULL,0ULL,OSD_PAGE_NUM_ROOT_INFO,OSD_ATTR_NUM_ROOT_INFO_NUM_OF_PARTITIONS,&root_no_of_partitions,ROOT_OBJECT_TYPE);
	set_page_attr(0ULL,0ULL,OSD_PAGE_NUM_ROOT_INFO,OSD_ATTR_NUM_ROOT_INFO_USED_CAPACITY,&root_used_capacity,ROOT_OBJECT_TYPE);

	//update part=0 data modified timestamp
	timestamp_val=get_current_timestamp();
	set_page_attr(0ULL,0ULL,OSD_PAGE_NUM_PART_TIMESTAMPS,OSD_ATTR_NUM_PART_TIMESTAMPS_DATA_MODIFIED_TIME,&timestamp_val,PART_OBJECT_TYPE);
	
	//FUTURE
	//Need to update root accordngly uint48_t oldest_nonce,uint48_t newest_nonce

	return 0;	
	

}

//------------------------------------------------------------------------------------
int create_uobj_coll_attribute_pages_effects(uint64_t part_id,uint64_t object_id,int object_type)
{
	char username[100];
	uint64_t root_total_capacity,root_used_capacity;
	uint64_t part_quota,part_used_capacity;
	uint64_t part_no_of_objects;
	uint64_t  pages_size;
	uint48_t timestamp_val;
	
	if(object_type==COLL_OBJECT_TYPE)
	pages_size= COLL_PAGES_SIZE;
	else 
	pages_size= UOBJ_PAGES_SIZE;
	
	
	get_page_attr(part_id,0ULL,OSD_PAGE_NUM_PART_INFO,OSD_ATTR_NUM_PART_INFO_USERNAME,username,PART_OBJECT_TYPE);
	
	get_page_attr(part_id,0ULL,OSD_PAGE_NUM_PART_INFO,OSD_ATTR_NUM_PART_INFO_NUM_OF_OBJECTS,&part_no_of_objects,PART_OBJECT_TYPE);
	get_page_attr(part_id,0ULL,OSD_PAGE_NUM_PART_QUOTAS,OSD_ATTR_NUM_PART_QUOTAS_OBJ_COUNT,&part_quota,PART_OBJECT_TYPE);
	
	print_msg(dlevel, "Part No of objects=0x%llx part Quota=0x%llx  ",part_no_of_objects,part_quota);
	
	if( part_no_of_objects == part_quota)
	{
		print_msg(ERROR_MSG,"No of Objects For Part Quota Exceed: Cant Add New object  ");
		print_msg(ERROR_MSG, "No of objects = 0x%llx  ",part_quota);
		return PART_NO_USER_OBJ_QUOTA_EXCEED;
	}
	
	
	
	get_page_attr(0ULL,0ULL,OSD_PAGE_NUM_ROOT_INFO,OSD_ATTR_NUM_ROOT_INFO_TOTAL_CAPACITY,&root_total_capacity,ROOT_OBJECT_TYPE);
	get_page_attr(0ULL,0ULL,OSD_PAGE_NUM_ROOT_INFO,OSD_ATTR_NUM_ROOT_INFO_USED_CAPACITY,&root_used_capacity,ROOT_OBJECT_TYPE);
	
	print_msg(dlevel, "Root Total Capacity = 0x%llx  Used Capacity =0x%llx   ",root_total_capacity,root_used_capacity);
	
	if(root_used_capacity+pages_size >root_total_capacity )
	{
			print_msg(ERROR_MSG,"Root Total Capacity Exceed: Cant Create New UObject Pages   ");
			print_msg(ERROR_MSG, "Root Total Capacity = 0x%llx  Used Capacity =0x%llx Obj Page Size =0x%llx ",root_total_capacity,root_used_capacity,pages_size);
			return ROOT_CAPACITY_QUOTA_EXCEED;
	
	}

	get_page_attr(part_id,0ULL,OSD_PAGE_NUM_PART_QUOTAS,OSD_ATTR_NUM_PART_QUOTAS_CAPACITY_QUOTA,&part_quota,PART_OBJECT_TYPE);
	get_page_attr(part_id,0ULL,OSD_PAGE_NUM_PART_INFO,OSD_ATTR_NUM_PART_INFO_USED_CAPACITY,&part_used_capacity,PART_OBJECT_TYPE);
	
	print_msg(dlevel, "Part  Total Quota = 0x%llx  Used Capacity =0x%llx obj page size=0x%llx   ",part_quota,part_used_capacity,pages_size);
	
	if(part_used_capacity+pages_size > part_quota )
	{
			print_msg(ERROR_MSG,"Part Total Capacity Exceed: Cant Create New Object Pages   ");
			print_msg(ERROR_MSG, "Part  Total Quota = 0x%llx  Used Capacity =0x%llx Obj Page Size =0x%llx   ",part_quota,part_used_capacity,pages_size);
			return PART_CAPACITY_QUOTA_EXCEED;
			
	}		
	
	if(object_type==COLL_OBJECT_TYPE)
	{

		if(create_collection_attribute_pages(part_id,object_id,username))
		{
			print_msg(ERROR_MSG,"Error in create_collection_attribute_pages part_id=0x%llx object_id=0x%llx username=%s   ",part_id,object_id,username);
			return 1;
	
		}

	}
	else
	{

		if(create_uobject_attribute_pages(part_id,object_id,username))
		{
			print_msg(ERROR_MSG,"Error in create_uobject_attribute_pages part_id=0x%llx object_id=0x%llx username=%s   ",part_id,object_id,username);
			return 1;
	
		}
	}
	
	print_msg(dlevel," Update Part Uobject count and Partion used capacity  ");
	
	part_no_of_objects++;
	set_page_attr(part_id,0ULL,OSD_PAGE_NUM_PART_INFO,OSD_ATTR_NUM_PART_INFO_NUM_OF_OBJECTS,&part_no_of_objects,PART_OBJECT_TYPE);
	part_used_capacity+=pages_size;
	set_page_attr(part_id,0ULL,OSD_PAGE_NUM_PART_INFO,OSD_ATTR_NUM_PART_INFO_USED_CAPACITY,&part_used_capacity,PART_OBJECT_TYPE);
	
	//update part data modified timestamp
	timestamp_val=get_current_timestamp();
	set_page_attr(part_id,0ULL,OSD_PAGE_NUM_PART_TIMESTAMPS,OSD_ATTR_NUM_PART_TIMESTAMPS_DATA_MODIFIED_TIME,&timestamp_val,PART_OBJECT_TYPE);
	
	
	//FUTURE if set attr side effect implemented for part_used capacity then not required
	print_msg(dlevel," Update Root Used capacity  ");
	root_used_capacity+=pages_size;
	set_page_attr(0ULL,0ULL,OSD_PAGE_NUM_ROOT_INFO,OSD_ATTR_NUM_ROOT_INFO_USED_CAPACITY,&root_used_capacity,ROOT_OBJECT_TYPE);

	return 0;


}
//------------------------------------------------------------------------------------
int read_attribute_pages_effects(uint64_t part_id,uint64_t object_id)
{
	uint48_t timestamp_val=get_current_timestamp();
	set_page_attr(part_id,object_id,OSD_PAGE_NUM_UOBJ_TIMESTAMPS,OSD_ATTR_NUM_UOBJ_TIMESTAMPS_DATA_ACCESSED_TIME,&timestamp_val,UOBJ_OBJECT_TYPE);
	return 0;	
}

//------------------------------------------------------------------------------------
int list_attribute_pages_effects(uint64_t part_id,uint64_t coll_object_id)
{
// update part data accessed timestamp for LIST PARTIIOTN 
// update collection data accessed timestamp for LIST COLLECTION

	uint48_t timestamp_val=get_current_timestamp();
	if(coll_object_id == 0)
	{
		set_page_attr(part_id,coll_object_id,OSD_PAGE_NUM_PART_TIMESTAMPS,OSD_ATTR_NUM_PART_TIMESTAMPS_DATA_ACCESSED_TIME,&timestamp_val,PART_OBJECT_TYPE);
		
	}
		
	else
	{
		//attr  no and  page no same for both partiiton object and collection object
		set_page_attr(part_id,coll_object_id,OSD_PAGE_NUM_COLL_TIMESTAMPS,OSD_ATTR_NUM_COLL_TIMESTAMPS_DATA_ACCESSED_TIME,&timestamp_val,COLL_OBJECT_TYPE);
	
	}
	return 0;
	
}
//------------------------------------------------------------------------------------
int remove_partition_attribute_pages_effects(uint64_t part_id)
{
	//update part=0 data modified timestamp
	uint64_t root_no_of_partitions;
	uint64_t root_used_capacity;
	uint48_t timestamp_val;
	//get_page_attr(uint64_t part_id,uint64_t  object_id, uint64_t page_no,uint64_t attr_no,void *attr_value,int object_type)

	get_page_attr(0ULL,0ULL,OSD_PAGE_NUM_ROOT_INFO,OSD_ATTR_NUM_ROOT_INFO_NUM_OF_PARTITIONS,&root_no_of_partitions,ROOT_OBJECT_TYPE);
	
	print_msg(dlevel, "No of partitions= 0x%llx   ",root_no_of_partitions);

	get_page_attr(0ULL,0ULL,OSD_PAGE_NUM_ROOT_INFO ,OSD_ATTR_NUM_ROOT_INFO_USED_CAPACITY ,&root_used_capacity,ROOT_OBJECT_TYPE);
	
	print_msg(dlevel, "Root Used Capacity =0x%llx  PART Page Size=0x%x    ",root_used_capacity,PART_PAGES_SIZE);
	
	root_used_capacity=root_used_capacity-PART_PAGES_SIZE;
	root_no_of_partitions--;
	print_msg(dlevel,"Update Root No of partiton and Root Used Capacity  ");
	set_page_attr(0ULL,0ULL,OSD_PAGE_NUM_ROOT_INFO,OSD_ATTR_NUM_ROOT_INFO_NUM_OF_PARTITIONS,&root_no_of_partitions,ROOT_OBJECT_TYPE);
	set_page_attr(0ULL,0ULL,OSD_PAGE_NUM_ROOT_INFO,OSD_ATTR_NUM_ROOT_INFO_USED_CAPACITY,&root_used_capacity,ROOT_OBJECT_TYPE);

	//update part=0 data modified timestamp
	timestamp_val=get_current_timestamp();
	set_page_attr(0ULL,0ULL,OSD_PAGE_NUM_PART_TIMESTAMPS,OSD_ATTR_NUM_PART_TIMESTAMPS_DATA_MODIFIED_TIME,&timestamp_val,PART_OBJECT_TYPE);
	
	//FUTURE
	//Need to update root accordngly uint48_t oldest_nonce,uint48_t newest_nonce
	return 0;	

}

//------------------------------------------------------------------------------------
int remove_uobj_coll_attribute_pages_effects(uint64_t part_id,uint64_t object_id,int object_type)
{

//update part data modified timestamp
	uint64_t root_used_capacity;
	uint64_t part_used_capacity;
	uint64_t part_no_of_objects;
	uint64_t  pages_size;
	uint48_t timestamp_val;
	
	if(object_type== COLL_OBJECT_TYPE)
	pages_size= COLL_PAGES_SIZE;
	else 
	pages_size= UOBJ_PAGES_SIZE;
	
	get_page_attr(part_id,0ULL,OSD_PAGE_NUM_PART_INFO,OSD_ATTR_NUM_PART_INFO_NUM_OF_OBJECTS,&part_no_of_objects,PART_OBJECT_TYPE);
	
	print_msg(dlevel, "Part No of objects=0x%llx ",part_no_of_objects);
	
	get_page_attr(0ULL,0ULL,OSD_PAGE_NUM_ROOT_INFO,OSD_ATTR_NUM_ROOT_INFO_USED_CAPACITY,&root_used_capacity,ROOT_OBJECT_TYPE);
	
	print_msg(dlevel, "Root  Used Capacity =0x%llx ",root_used_capacity);
	
	get_page_attr(part_id,0ULL,OSD_PAGE_NUM_PART_INFO,OSD_ATTR_NUM_PART_INFO_USED_CAPACITY,&part_used_capacity,PART_OBJECT_TYPE);
	
	print_msg(dlevel, "Part Used Capacity =0x%llx obj page size=0x%llx   ",part_used_capacity,pages_size);
	
	print_msg(dlevel,"Update Part Uobject count and Partion used capacity  ");
	
	part_no_of_objects--;
	set_page_attr(part_id,0ULL,OSD_PAGE_NUM_PART_INFO,OSD_ATTR_NUM_PART_INFO_NUM_OF_OBJECTS,&part_no_of_objects,PART_OBJECT_TYPE);
	part_used_capacity-=pages_size;
	set_page_attr(part_id,0ULL,OSD_PAGE_NUM_PART_INFO,OSD_ATTR_NUM_PART_INFO_USED_CAPACITY,&part_used_capacity,PART_OBJECT_TYPE);
	
	
	//update part data modified timestamp
	timestamp_val=get_current_timestamp();
	set_page_attr(part_id,0ULL,OSD_PAGE_NUM_PART_TIMESTAMPS,OSD_ATTR_NUM_PART_TIMESTAMPS_DATA_MODIFIED_TIME,&timestamp_val,PART_OBJECT_TYPE);
	
	 print_msg(dlevel,"Update Root Used capacity  ");
	
	//FUTURE if set attr side effect implemented for part_used capacity then not required
	root_used_capacity-=pages_size;
	set_page_attr(0ULL,0ULL,OSD_PAGE_NUM_ROOT_INFO,OSD_ATTR_NUM_ROOT_INFO_USED_CAPACITY,&root_used_capacity,ROOT_OBJECT_TYPE);
	
	
	return 0;



}
//------------------------------------------------------------------------------------

int getattr_attribute_pages_effects(uint64_t part_id,uint64_t object_id,int object_type)
{
	uint48_t timestamp_val=get_current_timestamp();

	if(object_type == ROOT_OBJECT_TYPE)
		set_page_attr(0ULL,0ULL,OSD_PAGE_NUM_ROOT_TIMESTAMPS,OSD_ATTR_NUM_ROOT_TIMESTAMPS_ATTR_ACCESSED_TIME,&timestamp_val,ROOT_OBJECT_TYPE);
	else if(object_id == 0)
		set_page_attr(part_id,0ULL,OSD_PAGE_NUM_PART_TIMESTAMPS,OSD_ATTR_NUM_PART_TIMESTAMPS_ATTR_ACCESSED_TIME,&timestamp_val,PART_OBJECT_TYPE);
	else if(object_type == UOBJ_OBJECT_TYPE)
		set_page_attr(part_id,object_id,OSD_PAGE_NUM_UOBJ_TIMESTAMPS,OSD_ATTR_NUM_UOBJ_TIMESTAMPS_ATTR_ACCESSED_TIME,&timestamp_val,UOBJ_OBJECT_TYPE);
	else //collection object
		set_page_attr(part_id,object_id,OSD_PAGE_NUM_UOBJ_TIMESTAMPS,OSD_ATTR_NUM_COLL_TIMESTAMPS_ATTR_ACCESSED_TIME,&timestamp_val,COLL_OBJECT_TYPE);
	
	return 0;	
}
//------------------------------------------------------------------------------------
int setattr_attribute_pages_effects(uint64_t part_id,uint64_t object_id,int object_type)
{
	uint48_t timestamp_val=get_current_timestamp();

	if(object_type == ROOT_OBJECT_TYPE)
	{	//root object
		set_page_attr(0ULL,0ULL,OSD_PAGE_NUM_ROOT_TIMESTAMPS,OSD_ATTR_NUM_ROOT_TIMESTAMPS_ATTR_ACCESSED_TIME,&timestamp_val,ROOT_OBJECT_TYPE);
		set_page_attr(0ULL,0ULL,OSD_PAGE_NUM_ROOT_TIMESTAMPS,OSD_ATTR_NUM_ROOT_TIMESTAMPS_ATTR_MODIFIED_TIME,&timestamp_val,ROOT_OBJECT_TYPE);
		return 0;
	}
	if( object_id == 0 )
	{	//partition object
	
		set_page_attr(part_id,0ULL,OSD_PAGE_NUM_PART_TIMESTAMPS,OSD_ATTR_NUM_PART_TIMESTAMPS_ATTR_ACCESSED_TIME,&timestamp_val,PART_OBJECT_TYPE);
		set_page_attr(part_id,0ULL,OSD_PAGE_NUM_PART_TIMESTAMPS,OSD_ATTR_NUM_PART_TIMESTAMPS_ATTR_MODIFIED_TIME,&timestamp_val,PART_OBJECT_TYPE);
		return 0;
	}

	if(object_type == UOBJ_OBJECT_TYPE)
	{
		//user object
		set_page_attr(part_id,object_id,OSD_PAGE_NUM_UOBJ_TIMESTAMPS,OSD_ATTR_NUM_UOBJ_TIMESTAMPS_ATTR_ACCESSED_TIME,&timestamp_val,UOBJ_OBJECT_TYPE);
		set_page_attr(part_id,object_id,OSD_PAGE_NUM_UOBJ_TIMESTAMPS,OSD_ATTR_NUM_UOBJ_TIMESTAMPS_ATTR_MODIFIED_TIME,&timestamp_val,UOBJ_OBJECT_TYPE);
		return 0;	
	}
	
	//collection object
	set_page_attr(part_id,object_id,OSD_PAGE_NUM_COLL_TIMESTAMPS,OSD_ATTR_NUM_COLL_TIMESTAMPS_ATTR_ACCESSED_TIME,&timestamp_val,COLL_OBJECT_TYPE);
	set_page_attr(part_id,object_id,OSD_PAGE_NUM_COLL_TIMESTAMPS,OSD_ATTR_NUM_COLL_TIMESTAMPS_ATTR_MODIFIED_TIME,&timestamp_val,COLL_OBJECT_TYPE);
	
	return 0;	
}

//-------------------------------------------------------------------------------
int get_and_check_write_affected_attributes(uint16_t service_action,WRITE_AFFECTED_ATTRS *write_affected_attrs)
{
 
	uint64_t object_id=write_affected_attrs->object_id;
	uint64_t part_id=write_affected_attrs->part_id;
	
	uint64_t user_max_obj_len;
	uint64_t part_capacity_quota;
	uint64_t root_total_capacity;
	uint64_t add_len;
 
	get_page_attr(part_id,object_id,OSD_PAGE_NUM_UOBJ_INFO,OSD_ATTR_NUM_UOBJ_INFO_OBJ_LENGTH,&(write_affected_attrs->user_obj_logical_len),UOBJ_OBJECT_TYPE);
	
	get_page_attr(part_id,object_id,OSD_PAGE_NUM_UOBJ_QUOTAS,OSD_ATTR_NUM_UOBJ_QUOTAS_MAX_OBJ_LEN,&(user_max_obj_len),UOBJ_OBJECT_TYPE);
 	
	get_page_attr(part_id,object_id,OSD_PAGE_NUM_UOBJ_INFO,OSD_ATTR_NUM_UOBJ_INFO_USED_CAPACITY,&(write_affected_attrs->user_used_capacity),UOBJ_OBJECT_TYPE);
	
	print_msg(dlevel, "Uobj user_obj_logical_len=0x%llx  user_max_obj_len=0x%llx user_used_capacity=0x%llx   ",write_affected_attrs->user_obj_logical_len,user_max_obj_len,write_affected_attrs->user_used_capacity);
	
	if(service_action == OSD_CMD_APPEND)
	{
		write_affected_attrs->starting_byte_offset=write_affected_attrs->user_obj_logical_len;
		add_len=write_affected_attrs->length;
		
	}
	else //for write and create_write
	{
		if(write_affected_attrs->user_obj_logical_len>write_affected_attrs->starting_byte_offset+write_affected_attrs->length) 
		//As user object logical length is not changing, no need to do 
		return 0;
		else
		add_len=write_affected_attrs->starting_byte_offset+write_affected_attrs->length-write_affected_attrs->user_obj_logical_len;
		
	}
	
	print_msg(dlevel," Write add_length=0x%llx",add_len);
	
	if(write_affected_attrs->user_obj_logical_len+add_len>user_max_obj_len)
	{
		print_msg(ERROR_MSG,"Uobj Max Obj Len Exceed: Cant Write to   Object ");
		print_msg(ERROR_MSG, "Uobj user_obj_logical_len=0x%llx  user_max_obj_len=0x%llx user_used_capacity=0x%llx   ",write_affected_attrs->user_obj_logical_len,user_max_obj_len,write_affected_attrs->user_used_capacity);
		print_msg(ERROR_MSG,"add_length=0x%llx",add_len);
		return UOBJ_LENGTH_QUOTA_EXCEED;
	}
	
	get_page_attr(part_id,0ULL,OSD_PAGE_NUM_PART_QUOTAS,OSD_ATTR_NUM_PART_QUOTAS_CAPACITY_QUOTA,&(part_capacity_quota),PART_OBJECT_TYPE);
	get_page_attr(part_id,0ULL,OSD_PAGE_NUM_PART_INFO,OSD_ATTR_NUM_PART_INFO_USED_CAPACITY,&(write_affected_attrs->part_used_capacity),PART_OBJECT_TYPE);
	
	print_msg(dlevel, "Part  Total Quota = 0x%llx  Used Capacity =0x%llx add_len=0x%llx   ",part_capacity_quota,write_affected_attrs->part_used_capacity,add_len);
	
	if(write_affected_attrs->part_used_capacity+add_len > part_capacity_quota )
	{
		print_msg(ERROR_MSG,"Part Total Capacity Exceed: Cant Write to   Object ");
		print_msg(ERROR_MSG, "Part  Total Quota = 0x%llx  Used Capacity =0x%llx add_len=0x%llx   ",part_capacity_quota,write_affected_attrs->part_used_capacity,add_len);
		return PART_CAPACITY_QUOTA_EXCEED;
			
	}		

	get_page_attr(0ULL,0ULL,OSD_PAGE_NUM_ROOT_INFO,OSD_ATTR_NUM_ROOT_INFO_TOTAL_CAPACITY,&root_total_capacity,ROOT_OBJECT_TYPE);
	get_page_attr(0ULL,0ULL,OSD_PAGE_NUM_ROOT_INFO,OSD_ATTR_NUM_ROOT_INFO_USED_CAPACITY,&(write_affected_attrs->root_used_capacity),ROOT_OBJECT_TYPE);
	
	print_msg(dlevel, "Root Total Capacity = 0x%llx  Used Capacity =0x%llx  add_len=0x%llx ",root_total_capacity,write_affected_attrs->root_used_capacity,add_len);
	
	if(write_affected_attrs->root_used_capacity+add_len >root_total_capacity)
	{
			print_msg(ERROR_MSG,"Root Total Capacity Exceed: Cant Write to Object ");
			print_msg(ERROR_MSG, "Root Total Capacity = 0x%llx  Used Capacity =0x%llx Add_len=0x%llx  ",root_total_capacity,write_affected_attrs->root_used_capacity,add_len);
			return ROOT_CAPACITY_QUOTA_EXCEED;
	
	}

	return 0;
	
}

//-------------------------------------------------------------------------------
int set_write_affected_attributes(WRITE_AFFECTED_ATTRS *write_affected_attrs)
{	//FUTURE there can be internal error here

	uint64_t add_length;
	uint64_t object_id=write_affected_attrs->object_id;
 	uint64_t part_id=write_affected_attrs->part_id;
 	uint48_t timestamp_val;
	add_length=write_affected_attrs->starting_byte_offset+write_affected_attrs->length-write_affected_attrs->user_obj_logical_len;
	
	if(add_length<= 0 )
	{	//As user object logical length is not changing no need to do 
	goto  update_timestamps;	
	}
	
	write_affected_attrs->user_obj_logical_len+=add_length;
	set_page_attr(part_id,object_id,OSD_PAGE_NUM_UOBJ_INFO,OSD_ATTR_NUM_UOBJ_INFO_OBJ_LENGTH,&(write_affected_attrs->user_obj_logical_len),UOBJ_OBJECT_TYPE);
	
	write_affected_attrs->user_used_capacity+=add_length;
	set_page_attr(part_id,object_id,OSD_PAGE_NUM_UOBJ_INFO,OSD_ATTR_NUM_UOBJ_INFO_USED_CAPACITY,&(write_affected_attrs->user_used_capacity),UOBJ_OBJECT_TYPE);
	
	write_affected_attrs->part_used_capacity+=add_length;
	set_page_attr(part_id,0ULL,OSD_PAGE_NUM_PART_INFO,OSD_ATTR_NUM_PART_INFO_USED_CAPACITY,&(write_affected_attrs->part_used_capacity),PART_OBJECT_TYPE);
	
	write_affected_attrs->root_used_capacity+=add_length;
	get_page_attr(0ULL,0ULL,OSD_PAGE_NUM_ROOT_INFO,OSD_ATTR_NUM_ROOT_INFO_USED_CAPACITY,&(write_affected_attrs->root_used_capacity),ROOT_OBJECT_TYPE);

update_timestamps:
	timestamp_val=get_current_timestamp();
	set_page_attr(part_id,object_id,OSD_PAGE_NUM_UOBJ_TIMESTAMPS,OSD_ATTR_NUM_UOBJ_TIMESTAMPS_DATA_ACCESSED_TIME,&timestamp_val,UOBJ_OBJECT_TYPE);
	set_page_attr(part_id,object_id,OSD_PAGE_NUM_UOBJ_TIMESTAMPS,OSD_ATTR_NUM_UOBJ_TIMESTAMPS_DATA_MODIFIED_TIME,&timestamp_val,UOBJ_OBJECT_TYPE);
	
	return 0;

}
//-------------------------------------------------------------------------------

int add_to_coll_attribute_pages_effects(uint64_t part_id,uint64_t coll_object_id,int object_id)
{
	uint64_t add_len=8,coll_used_capacity;
	uint64_t part_capacity_quota,part_used_capacity;
	uint64_t root_total_capacity,root_used_capacity;
	uint48_t timestamp_val;
	
	get_page_attr(part_id,0ULL,OSD_PAGE_NUM_PART_QUOTAS,OSD_ATTR_NUM_PART_QUOTAS_CAPACITY_QUOTA,&(part_capacity_quota),PART_OBJECT_TYPE);
	get_page_attr(part_id,0ULL,OSD_PAGE_NUM_PART_INFO,OSD_ATTR_NUM_PART_INFO_USED_CAPACITY,&(part_used_capacity),PART_OBJECT_TYPE);
	
	print_msg(dlevel, "Part  Total Quota = 0x%llx  Used Capacity =0x%llx add_len=0x%llx   ",part_capacity_quota,part_used_capacity,add_len);
	
	if(part_used_capacity+add_len > part_capacity_quota )
	{
		print_msg(ERROR_MSG,"Part Total Capacity Exceed: Cant Add new object to collection");
		print_msg(ERROR_MSG, "Part  Total Quota = 0x%llx  Used Capacity =0x%llx add_len=0x%llx   ",part_capacity_quota,part_used_capacity,add_len);
		return PART_CAPACITY_QUOTA_EXCEED;
			
	}		

	
	get_page_attr(0ULL,0ULL,OSD_PAGE_NUM_ROOT_INFO,OSD_ATTR_NUM_ROOT_INFO_TOTAL_CAPACITY,&root_total_capacity,ROOT_OBJECT_TYPE);
	get_page_attr(0ULL,0ULL,OSD_PAGE_NUM_ROOT_INFO,OSD_ATTR_NUM_ROOT_INFO_USED_CAPACITY,&(root_used_capacity),ROOT_OBJECT_TYPE);
	
	print_msg(dlevel, "Root Total Capacity = 0x%llx  Used Capacity =0x%llx  add_len=0x%llx ",root_total_capacity,root_used_capacity,add_len);
	
	if(root_used_capacity+add_len >root_total_capacity)
	{
			print_msg(ERROR_MSG,"Root Total Capacity Exceed: Cant Write to Object ");
			print_msg(ERROR_MSG, "Root Total Capacity = 0x%llx  Used Capacity =0x%llx Add_len=0x%llx  ",root_total_capacity,root_used_capacity,add_len);
			return ROOT_CAPACITY_QUOTA_EXCEED;
	
	}

	get_page_attr(part_id,coll_object_id,OSD_PAGE_NUM_COLL_INFO,OSD_ATTR_NUM_COLL_INFO_USED_CAPACITY,&coll_used_capacity,COLL_OBJECT_TYPE);

	add_object_to_collection_list(part_id,coll_object_id,object_id,UOBJ_OBJECT_TYPE);
		
	coll_used_capacity+=8;

	set_page_attr(part_id,coll_object_id,OSD_PAGE_NUM_COLL_INFO,OSD_ATTR_NUM_COLL_INFO_USED_CAPACITY,&coll_used_capacity,COLL_OBJECT_TYPE);

	part_used_capacity+=8;
	set_page_attr(part_id,0ULL,OSD_PAGE_NUM_PART_INFO,OSD_ATTR_NUM_PART_INFO_USED_CAPACITY,&(part_used_capacity),PART_OBJECT_TYPE);

	root_used_capacity+=8;
	set_page_attr(0ULL,0ULL,OSD_PAGE_NUM_ROOT_INFO,OSD_ATTR_NUM_ROOT_INFO_USED_CAPACITY,&(root_used_capacity),ROOT_OBJECT_TYPE);

	timestamp_val=get_current_timestamp();
	set_page_attr(part_id,object_id,OSD_PAGE_NUM_COLL_TIMESTAMPS,OSD_ATTR_NUM_COLL_TIMESTAMPS_DATA_MODIFIED_TIME,&timestamp_val,COLL_OBJECT_TYPE);
	return 0;
	
	//FUTURE: Need To update uobject collections page
	//set_page_attr(part_id,object_id,OSD_PAGE_NUM_UOBJ_TIMESTAMPS,OSD_ATTR_NUM_UOBJ_TIMESTAMPS_DATA_MODIFIED_TIME,&timestamp_val,UOBJ_OBJECT_TYPE);

}
//-------------------------------------------------------------------------------
int remove_from_coll_attribute_pages_effects(uint64_t part_id,uint64_t coll_object_id,int object_id)
{
	uint64_t coll_used_capacity;
	uint64_t part_used_capacity;
	uint64_t root_used_capacity;
	uint48_t timestamp_val;
	
	if(remove_object_from_collection_list(part_id,coll_object_id,object_id,UOBJ_OBJECT_TYPE))
	return 1;
	
	get_page_attr(part_id,0ULL,OSD_PAGE_NUM_PART_INFO,OSD_ATTR_NUM_PART_INFO_USED_CAPACITY,&(part_used_capacity),PART_OBJECT_TYPE);
	
	print_msg(dlevel, "Part  Used Capacity =0x%llx",part_used_capacity);
	
	get_page_attr(0ULL,0ULL,OSD_PAGE_NUM_ROOT_INFO,OSD_ATTR_NUM_ROOT_INFO_USED_CAPACITY,&(root_used_capacity),ROOT_OBJECT_TYPE);
	
	print_msg(dlevel, "Root Used Capacity =0x%llx  ",root_used_capacity);
	
	
	get_page_attr(part_id,coll_object_id,OSD_PAGE_NUM_COLL_INFO,OSD_ATTR_NUM_COLL_INFO_USED_CAPACITY,&coll_used_capacity,COLL_OBJECT_TYPE);

	
	coll_used_capacity-=8;

	set_page_attr(part_id,coll_object_id,OSD_PAGE_NUM_COLL_INFO,OSD_ATTR_NUM_COLL_INFO_USED_CAPACITY,&coll_used_capacity,COLL_OBJECT_TYPE);

	part_used_capacity-=8;
	set_page_attr(part_id,0ULL,OSD_PAGE_NUM_PART_INFO,OSD_ATTR_NUM_PART_INFO_USED_CAPACITY,&(part_used_capacity),PART_OBJECT_TYPE);

	root_used_capacity-=8;
	set_page_attr(0ULL,0ULL,OSD_PAGE_NUM_ROOT_INFO,OSD_ATTR_NUM_ROOT_INFO_USED_CAPACITY,&(root_used_capacity),ROOT_OBJECT_TYPE);

	timestamp_val=get_current_timestamp();
	set_page_attr(part_id,object_id,OSD_PAGE_NUM_COLL_TIMESTAMPS,OSD_ATTR_NUM_COLL_TIMESTAMPS_DATA_MODIFIED_TIME,&timestamp_val,COLL_OBJECT_TYPE);
	return 0;
	
	//FUTURE NEED TO update uobject collections page
	//set_page_attr(part_id,object_id,OSD_PAGE_NUM_UOBJ_TIMESTAMPS,OSD_ATTR_NUM_UOBJ_TIMESTAMPS_DATA_MODIFIED_TIME,&timestamp_val,UOBJ_OBJECT_TYPE);

}
//-------------------------------------------------------------------------------
