/**
 *                      ==================
 *                      IITK OSD Simulator
 *                      ==================
 *   An Implementation of Simulator for Object based storage device(OSD)
 *
 *   Copyright (C) 2007-2008 CSE IIT KANPUR
 *
 *   Contact: Department of Computer Science and Engineering
 *            Indian Institute of Technology,Kanpur(IIT Kanpur)
 *            {moona,riyaz}@cse.iitk.ac.in
 *
 *   This file is part of IITK OSD Simulator Project.
 *
 *   IITK OSD Simulator Project is free software; you can redistribute it
 *   and/or modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   IITK OSD Simulator Project is distributed in the hope that it will be
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the  GNU General
 *   Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
# include "osd_attr_manip.h"

/**
**********************************
 * File:  osd_attr_manip.c
 * Implements functions related to get/set attribute
**/

extern int DEBUG;
//Defines debug level for this file as 2
static int dlevel=LOWER_FUNC;


uint16_t get_offset_of_page_in_file( uint64_t part_id,uint64_t object_id, uint32_t page_no,int object_type)
{
	
	if(page_no == OSD_PAGE_NUM_CUR_CMD)
	{
		return 0;
	
	}
	if(object_type == ROOT_OBJECT_TYPE )
	{
		switch(page_no)
		{
		case OSD_PAGE_NUM_ROOT_DIRECTORY : 
			return ROOT_DIRECTORY_PAGE_OFFSET_IN_FILE;
			
		case OSD_PAGE_NUM_ROOT_INFO: 
			 return ROOT_INFORMATION_PAGE_OFFSET_IN_FILE;
			 
		case OSD_PAGE_NUM_ROOT_QUOTAS:  
			return ROOT_QUOTAS_PAGE_OFFSET_IN_FILE;
			
		case OSD_PAGE_NUM_ROOT_TIMESTAMPS: 
			return ROOT_TIMESTAMPS_PAGE_OFFSET_IN_FILE;
			
		case OSD_PAGE_NUM_ROOT_SECURITY: 
			return ROOT_SECURITY_PAGE_OFFSET_IN_FILE ;
		}
		return 0xFFFF;
	}
	if (object_type == PART_OBJECT_TYPE )
	{
		switch(page_no)
		{
		
		case OSD_PAGE_NUM_PART_DIRECTORY : 
			return PART_DIRECTORY_PAGE_OFFSET_IN_FILE ;
			
		case OSD_PAGE_NUM_PART_INFO: 
			return PART_INFORMATION_PAGE_OFFSET_IN_FILE ;
			
			
		case OSD_PAGE_NUM_PART_QUOTAS: 
			return PART_QUOTAS_PAGE_OFFSET_IN_FILE ;
			
			
		case OSD_PAGE_NUM_PART_TIMESTAMPS:
			return PART_TIMESTAMPS_PAGE_OFFSET_IN_FILE ;
			
		case OSD_PAGE_NUM_PART_SECURITY: 
			return PART_SECURITY_PAGE_OFFSET_IN_FILE ;
		}
	
		return 0xFFFF;
	}
	
	if(object_type == COLL_OBJECT_TYPE)
	{
		switch(page_no)
		{
		case OSD_PAGE_NUM_COLL_DIRECTORY: 
			return COLL_DIRECTORY_PAGE_OFFSET_IN_FILE;
		case OSD_PAGE_NUM_COLL_INFO: 
			return COLL_INFORMATION_PAGE_OFFSET_IN_FILE ;
			
		case OSD_PAGE_NUM_COLL_TIMESTAMPS:
			return COLL_TIMESTAMPS_PAGE_OFFSET_IN_FILE ;
		case OSD_PAGE_NUM_COLL_SECURITY:
			return COLL_SECURITY_PAGE_OFFSET_IN_FILE;
		}
	
		return 0xFFFF;
	}
	
	if(object_type == UOBJ_OBJECT_TYPE)
	{
	
		switch(page_no)
		{
		case OSD_PAGE_NUM_UOBJ_DIRECTORY: 
			return UOBJ_DIRECTORY_PAGE_OFFSET_IN_FILE;
			
		case OSD_PAGE_NUM_UOBJ_INFO: 
			return UOBJ_INFORMATION_PAGE_OFFSET_IN_FILE  ;
			
		case OSD_PAGE_NUM_UOBJ_QUOTAS: 
			return UOBJ_QUOTAS_PAGE_OFFSET_IN_FILE   ;
			
		case OSD_PAGE_NUM_UOBJ_TIMESTAMPS: 
			return UOBJ_TIMESTAMPS_PAGE_OFFSET_IN_FILE ;
			
		case OSD_PAGE_NUM_UOBJ_SECURITY: 
			return UOBJ_SECURITY_PAGE_OFFSET_IN_FILE ;
			
		case OSD_PAGE_NUM_T10_COLLECTIONS: 
			return UOBJ_COLLECTIONS_PAGE_OFFSET_IN_FILE;
	
		}
		return 0xFFFF;
	}
	return 0xFFFF;
}
//------------------------------------------------------------------------------

OSD_ATTR_INFO get_osd_attr_struct(uint64_t part_id,uint64_t object_id, uint32_t page_no,uint32_t attr_no,int object_type)
{
	OSD_ATTR_INFO osd_attr_info;
	
	if(page_no == OSD_PAGE_NUM_CUR_CMD)
	{
	
	osd_attr_info=find_offset_current_command_page(page_no,attr_no);
	
	} else if(object_type == ROOT_OBJECT_TYPE ) 
	{//root object	
		osd_attr_info=find_offset_root_page(page_no,attr_no);
		
	}
	else if(object_type == PART_OBJECT_TYPE)
	{ //partition object 
	 osd_attr_info=find_offset_partition_page(page_no,attr_no);
	  
	}else if(object_type ==UOBJ_OBJECT_TYPE)
	{//user object
	osd_attr_info=find_offset_uobject_page(page_no,attr_no);
	
	}else  if (object_type == COLL_OBJECT_TYPE)
	{
	//collection 
	osd_attr_info=find_offset_collection_page(page_no,attr_no);
	}	
	
	return osd_attr_info;

}


//------------------------------------------------------------------------------
int  set_page_attr(uint64_t part_id,uint64_t object_id, uint32_t page_no,uint32_t attr_no,void *attr_value,int object_type)
{
	uint16_t attr_offset_file;
	uint16_t attr_page_offset_file;
	char filename[FILENAME_SIZE];
	OSD_ATTR_INFO osd_attr_info;
	
	print_msg(dlevel,"part_id=0x%llx  object_id=0x%llx page_no=0x%lx attr_no=0x%lx object_type=0x%x    ",part_id,object_id,page_no,attr_no,object_type);
		
	if ( get_attr_filename(part_id,object_id,page_no,filename,object_type))
	{
		print_msg(ERROR_MSG,"Error in getting file name for part_id =0x%llx obj_id=0x%llx page_no=0x%lx atr_no=0x%lx object_type=0x%x",part_id,object_id,page_no,attr_no,object_type); 
		return 1;
	}	
	
	print_msg(dlevel,"Filename = %s   ",filename); 
	
	osd_attr_info= get_osd_attr_struct(part_id,object_id,page_no,attr_no,object_type);
	
	if (osd_attr_info.attr_offset ==OSD_ATTR_WRONG_OFFSET)
	{
		print_msg(ERROR_MSG,"Error in getting file offset for part_id =0x%llx obj_id=0x%llx page_no=0x%lx attr_no=0x%lx object_type=0x%x",part_id,object_id,page_no,attr_no,object_type); 
		return 1;	
	}
	
	attr_page_offset_file=get_offset_of_page_in_file(part_id,object_id,page_no,object_type);
	attr_offset_file=attr_page_offset_file+osd_attr_info.attr_offset;
	
	print_msg(dlevel," Attr Offset =0x%x  Page offset =0x%x  Total Offset=0x%x    ",osd_attr_info.attr_offset, attr_page_offset_file,attr_offset_file);

	if(file_write(filename,attr_value,osd_attr_info.attr_size,attr_offset_file) )
	{
		print_msg(ERROR_MSG,"Error in set attribute writing file offset for part_id =0x%llx obj_id=0x%llx page_no=0x%lx attr_no=0x%lx object_type=0x%x  filename=%s  ",part_id,object_id,page_no,attr_no,object_type,filename); 
		return 1;	
	}
	print_msg (dlevel, " Set Attr done successfully    ");
	return 0;
} 
//-------------------------------------------------------------------------------
//-------------------------
int get_page_attr(uint64_t part_id,uint64_t object_id, uint32_t page_no,uint32_t attr_no,void *attr_value,int object_type)
{
	uint16_t attr_offset_file;
	uint16_t attr_page_offset_file;
	char filename[FILENAME_SIZE];
	OSD_ATTR_INFO osd_attr_info;
	
	print_msg(dlevel,"part_id=0x%llx  object_id=0x%llx page_no=0x%lx attr_no=0x%lx  object_type=0x%x    ",part_id,object_id,page_no,attr_no,object_type);
	
	if ( get_attr_filename(part_id,object_id,page_no,filename,object_type))
	{
		print_msg(ERROR_MSG,"Error in getting file name for part_id =0x%llx obj_id=0x%llx page_no=0x%lx attr_no=0x%lx object_type=0x%x    ",part_id,object_id,page_no,attr_no,object_type); 
		return 1;
	}	
	print_msg(dlevel,"Filename = %s   ",filename); 
	
	osd_attr_info= get_osd_attr_struct(part_id,object_id,page_no,attr_no,object_type);
	
	if (osd_attr_info.attr_offset == OSD_ATTR_WRONG_OFFSET)
	{
		print_msg(ERROR_MSG,"Error in getting file offset for part_id =0x%llx obj_id=0x%llx page_no=0x%lx attr_no=0x%lx object_type=0x%x   ",part_id,object_id,page_no,attr_no,object_type); 
		return 1;	
	}
	
	attr_page_offset_file=get_offset_of_page_in_file(part_id,object_id,page_no,object_type);
	attr_offset_file=attr_page_offset_file +osd_attr_info.attr_offset;
	print_msg(dlevel," Attr Offset = 0x%x  Page offset =0x%x  Total Offset=0x%x    ",osd_attr_info.attr_offset, attr_page_offset_file,attr_offset_file);
		
	if(file_read(filename,attr_value,osd_attr_info.attr_size,attr_offset_file) )
	{
		print_msg(ERROR_MSG,"Error in get attribute reading file offset for part_id =0x%llx obj_id=0x%llx page_no=0x%lx attr_no=0x%lx object_type=0x%x  filename=%s  ",part_id,object_id,page_no,attr_no,object_type,filename); 
		return 1;	
	}
	
	print_msg (dlevel," Get Attr done successfully    ");
	return 0;
} 
//-------------------------------------------------------------------------------
OSD_ATTR_INFO find_offset_root_page( uint32_t page_no,uint32_t attr_no)
{

	int i;
	OSD_ATTR_INFO osd_attr_info;
	if(page_no ==  OSD_PAGE_NUM_ROOT_DIRECTORY)
	{	//root directory
		if(attr_no < 5)
		osd_attr_info.attr_offset=40*attr_no;
		else if(attr_no ==5)
		osd_attr_info.attr_offset=160;
		else 
		goto error;
		
		osd_attr_info.attr_size=40;
		osd_attr_info.attr_no=attr_no;
		osd_attr_info.attr_set=0;
		return osd_attr_info;
	}
	
	else if(page_no == OSD_PAGE_NUM_ROOT_INFO)
	{		//root info
	
		for (i=0; i< OSD_ROOT_INFO_PAGE.attr_count;i++)
		{
			
			if(OSD_ROOT_INFO_PAGE.attributes[i].attr_no ==attr_no)
			{
				
				return OSD_ROOT_INFO_PAGE.attributes[i];
			
			}
		
		}
	}
	else if (page_no == OSD_PAGE_NUM_ROOT_QUOTAS)
	{	//root quotas page
		for (i=0; i< OSD_ROOT_QUOTAS_PAGE.attr_count;i++)
		{
			if(OSD_ROOT_QUOTAS_PAGE.attributes[i].attr_no ==attr_no)
			{
				return OSD_ROOT_QUOTAS_PAGE.attributes[i];
			}
		
		}
	}
	else if(page_no == OSD_PAGE_NUM_ROOT_TIMESTAMPS)
	{	//Root timestamps 
		for (i=0; i< OSD_ROOT_TIMESTAMPS_PAGE.attr_count;i++)
		{
			if(OSD_ROOT_TIMESTAMPS_PAGE.attributes[i].attr_no ==attr_no)
			{
				return OSD_ROOT_TIMESTAMPS_PAGE.attributes[i];
			
			}
		}
	}
	else if(page_no == OSD_PAGE_NUM_ROOT_SECURITY)
	{	//Policy security 
		for (i=0; i< OSD_ROOT_SECURITY_PAGE.attr_count;i++)
		{
			if(OSD_ROOT_SECURITY_PAGE.attributes[i].attr_no ==attr_no)
			{
				return OSD_ROOT_SECURITY_PAGE.attributes[i];
			}
		}
	}
	
error:	
	osd_attr_info.attr_offset=OSD_ATTR_WRONG_OFFSET;
	osd_attr_info.attr_size=0;
	osd_attr_info.attr_no=attr_no;
	osd_attr_info.attr_set=1;
	return osd_attr_info;	
}
//----------------------------------------------------------------------------------
OSD_ATTR_INFO find_offset_partition_page( uint32_t page_no,uint32_t attr_no)
{
	int i;
	OSD_ATTR_INFO osd_attr_info;
	
	if(page_no == OSD_PAGE_NUM_PART_DIRECTORY )
	{	//part directory
		if(attr_no < 5)
		osd_attr_info.attr_offset=40*attr_no;
		else if(attr_no ==5)
		osd_attr_info.attr_offset=160;
		else 
		goto error;
		
		osd_attr_info.attr_size=40;
		osd_attr_info.attr_no=attr_no;
		osd_attr_info.attr_set=0;
		return osd_attr_info;
	
	}
	else if(page_no == OSD_PAGE_NUM_PART_INFO )
	{		//part info
		for (i=0; i< OSD_PART_INFO_PAGE.attr_count;i++)
		{
			if(OSD_PART_INFO_PAGE.attributes[i].attr_no ==attr_no)
			{
				return OSD_PART_INFO_PAGE.attributes[i];
			}
		}
	}
	else if (page_no == OSD_PAGE_NUM_PART_QUOTAS)
	{	//part quotas page
		for (i=0; i< OSD_PART_QUOTAS_PAGE.attr_count;i++)
		{
			if(OSD_PART_QUOTAS_PAGE.attributes[i].attr_no ==attr_no)
			{
				return OSD_PART_QUOTAS_PAGE.attributes[i];
			}
		}
	}
	else if(page_no == OSD_PAGE_NUM_PART_TIMESTAMPS)
	{	//part timestamps 
		for (i=0; i< OSD_PART_TIMESTAMPS_PAGE.attr_count;i++)
		{
			if(OSD_PART_TIMESTAMPS_PAGE.attributes[i].attr_no ==attr_no)
			{
				return OSD_PART_TIMESTAMPS_PAGE.attributes[i];
			}
		}
	}
	else if(page_no == OSD_PAGE_NUM_PART_SECURITY)
	{	//Part Policy security 
		for (i=0; i< OSD_PART_SECURITY_PAGE.attr_count;i++)
		{
			if(OSD_PART_SECURITY_PAGE.attributes[i].attr_no ==attr_no)
			{
				return OSD_PART_SECURITY_PAGE.attributes[i];
			
			}
		}
	}
error:	
		osd_attr_info.attr_offset=OSD_ATTR_WRONG_OFFSET;
		osd_attr_info.attr_size=0;
		osd_attr_info.attr_no=attr_no;
		osd_attr_info.attr_set=1;
		return osd_attr_info;	
}

//----------------------------------------------------------------------------------
OSD_ATTR_INFO find_offset_uobject_page( uint32_t page_no,uint32_t attr_no)
{

	int i;
	OSD_ATTR_INFO osd_attr_info;
	if(page_no ==  OSD_PAGE_NUM_UOBJ_DIRECTORY)
	{
		//uobj directory
		if(attr_no < 6)
			osd_attr_info.attr_offset=40*attr_no;
		else 
			goto error;
			
		osd_attr_info.attr_size=40;
		osd_attr_info.attr_no=attr_no;
		osd_attr_info.attr_set=0;
		return osd_attr_info;
		
	}
	else if(page_no ==  OSD_PAGE_NUM_UOBJ_INFO)
	{		//uobj info
		for (i=0; i< OSD_UOBJ_INFO_PAGE.attr_count;i++)
		{
			if(OSD_UOBJ_INFO_PAGE.attributes[i].attr_no ==attr_no)
			{
				return OSD_UOBJ_INFO_PAGE.attributes[i];
			
			}
		
		}
	}
	else if (page_no == OSD_PAGE_NUM_UOBJ_QUOTAS)
	{	//uobj quotas page
		for (i=0; i< OSD_UOBJ_QUOTAS_PAGE.attr_count;i++)
		{
			if(OSD_UOBJ_QUOTAS_PAGE.attributes[i].attr_no ==attr_no)
			{
				return OSD_UOBJ_QUOTAS_PAGE.attributes[i];
			
			}
		
		}
	}
	else if(page_no == OSD_PAGE_NUM_UOBJ_TIMESTAMPS)
	{	//Uobj timestamps 
		for (i=0; i< OSD_UOBJ_TIMESTAMPS_PAGE.attr_count;i++)
		{
			if(OSD_UOBJ_TIMESTAMPS_PAGE.attributes[i].attr_no ==attr_no)
			{
				return OSD_UOBJ_TIMESTAMPS_PAGE.attributes[i];
			
			}
		
		}

	}
	else if(page_no == OSD_PAGE_NUM_T10_COLLECTIONS )
	{	//Uobj timestamps 
		for (i=0; i< OSD_UOBJ_COLLECTIONS_PAGE.attr_count;i++)
		{
			if(OSD_UOBJ_COLLECTIONS_PAGE.attributes[i].attr_no ==attr_no)
			{
				return OSD_UOBJ_COLLECTIONS_PAGE.attributes[i];
			}
		}
	
	}
	else if(page_no == OSD_PAGE_NUM_UOBJ_SECURITY)
	{	//Uobj Policy security 
		for (i=0; i< OSD_UOBJ_SECURITY_PAGE.attr_count;i++)
		{
			if(OSD_UOBJ_SECURITY_PAGE.attributes[i].attr_no ==attr_no)
			{
				return OSD_UOBJ_SECURITY_PAGE.attributes[i];
			}
		}
	}
	
error:	
	osd_attr_info.attr_offset=OSD_ATTR_WRONG_OFFSET;
	osd_attr_info.attr_size=0;
	osd_attr_info.attr_no=attr_no;
	osd_attr_info.attr_set=1;
	return osd_attr_info;	
}

//----------------------------------------------------------------------------------
OSD_ATTR_INFO find_offset_collection_page(uint32_t page_no,uint32_t attr_no)
{
	int i;
	OSD_ATTR_INFO osd_attr_info;
	if(page_no ==  OSD_PAGE_NUM_COLL_DIRECTORY)
	{	//coll directory
		switch (attr_no)
		{
		case 0 : osd_attr_info.attr_offset=0; break;
		case 1 : osd_attr_info.attr_offset=40; break;
		case 3 : osd_attr_info.attr_offset=80; break;
		case 5 : osd_attr_info.attr_offset=120; break;
		default:	
			goto error;
		}
		osd_attr_info.attr_size=40;
		osd_attr_info.attr_no=attr_no;
		osd_attr_info.attr_set=0;
		return osd_attr_info;
	}
	else if(page_no == OSD_PAGE_NUM_COLL_INFO)
	{		//coll info
		for (i=0; i< OSD_COLL_INFO_PAGE.attr_count;i++)
		{
			if(OSD_COLL_INFO_PAGE.attributes[i].attr_no ==attr_no)
			{
				return OSD_COLL_INFO_PAGE.attributes[i];
			}
		}
	}
	else if(page_no == OSD_PAGE_NUM_COLL_TIMESTAMPS)
	{	//Coll timestamps 
		for (i=0; i< OSD_COLL_TIMESTAMPS_PAGE.attr_count;i++)
		{
			if(OSD_COLL_TIMESTAMPS_PAGE.attributes[i].attr_no ==attr_no)
			{
				return OSD_COLL_TIMESTAMPS_PAGE.attributes[i];
			}
		}
	}
	else if(page_no == OSD_PAGE_NUM_COLL_SECURITY )
	{	//Coll Policy security 
		for (i=0; i< OSD_COLL_SECURITY_PAGE.attr_count;i++)
		{
			if(OSD_COLL_SECURITY_PAGE.attributes[i].attr_no ==attr_no)
			{
				return OSD_COLL_SECURITY_PAGE.attributes[i];
			}
		}
	}

error:	
	osd_attr_info.attr_offset=OSD_ATTR_WRONG_OFFSET;
	osd_attr_info.attr_size=0;
	osd_attr_info.attr_no=attr_no;
	osd_attr_info.attr_set=1;
	return osd_attr_info;	
}

//----------------------------------------------------------------------------------
OSD_ATTR_INFO find_offset_current_command_page(uint32_t page_no,uint32_t attr_no)
{
	OSD_ATTR_INFO osd_attr_info;
	int i;
	for (i=0; i< OSD_CUR_CMD_PAGE.attr_count;i++)
	{
		if(OSD_CUR_CMD_PAGE.attributes[i].attr_no ==attr_no)
		{
			return OSD_CUR_CMD_PAGE.attributes[i];
		}
	}
	osd_attr_info.attr_offset=OSD_ATTR_WRONG_OFFSET;
	osd_attr_info.attr_size=0;
	osd_attr_info.attr_no=attr_no;
	osd_attr_info.attr_set=0;
	return osd_attr_info;	
}
//----------------------------------------------------------------------------------
 

