/**
 *                      ==================
 *                      IITK OSD Simulator
 *                      ==================
 *   An Implementation of Simulator for Object based storage device(OSD)
 *
 *   Copyright (C) 2007-2008 CSE IIT KANPUR
 *
 *   Contact: Department of Computer Science and Engineering
 *            Indian Institute of Technology,Kanpur(IIT Kanpur)
 *            {moona,riyaz}@cse.iitk.ac.in
 *
 *   This file is part of IITK OSD Simulator Project.
 *
 *   IITK OSD Simulator Project is free software; you can redistribute it
 *   and/or modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   IITK OSD Simulator Project is distributed in the hope that it will be
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the  GNU General
 *   Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

# include <stdio.h>
# include <string.h>
#include <sys/stat.h>
# include "osd_export.h"
# define MAX_SIZE 4096
/**
**********************************
 * File:  osd_export.c
 * Implements functions related to exporting OSD Luns
 **/
extern int DEBUG;
//Set debug level as 2 for this file
static int dlevel=LOWER_FUNC;	

int export_osd_luns(LUN_STRUCT *osd_luns,char *iscsi_export_file)
{
	char buf[MAX_SIZE];
	LUN_STRUCT *temp=osd_luns;
	char ss[20];
	FILE *fp;
	char write_buf[MAX_SIZE/4];
	fp=fopen(iscsi_export_file,"r");
	if(fp==NULL) return 1;	
	fread(buf,MAX_SIZE,1,fp);
	fclose(fp);
	fp=fopen(iscsi_export_file,"a");

	while(temp!=NULL)
	{
		//Check if entry already exists
		//We dont support fot same target with diff Lun no
		if(strstr(buf,temp->osd_lun.osd_target_name) == NULL)
		{
			print_msg(dlevel," Entry creating for target =%s\n",temp->osd_lun.osd_target_name);

			//printf(" OSD_TARGET NAME=%s Lun NO=%d Base Dir=%s Capacity=%llx OSD System ID=%s Username=%s \n",temp->osd_lun.osd_target_name,temp->osd_lun.lun_no,temp->osd_lun.osd_lun_base_directory,temp->osd_lun.capacity,temp->osd_lun.osd_system_id,temp->osd_lun.username);
		
			strcpy(write_buf,"\nTarget  ");
			strcat(write_buf,temp->osd_lun.osd_target_name);
			strcat(write_buf,"\n       Lun ");
			sprintf(ss,"%d",temp->osd_lun.lun_no);
			strcat(write_buf,ss);
			strcat(write_buf," Path=");
			strcat(write_buf,temp->osd_lun.osd_lun_base_directory);
			strcat(write_buf,",Type=osdio\n");
			fwrite(write_buf,strlen(write_buf),1,fp);
		}
		else
			print_msg(dlevel," Entry found for target =%s\n",temp->osd_lun.osd_target_name);
	
		printf(" Exporting Target =%s Lun=%d\n",temp->osd_lun.osd_target_name,temp->osd_lun.lun_no);
		temp=temp->next;
	}
	fclose(fp);
	system("service iscsi-target restart");
	return 0;
}


LUN_STRUCT *prepare_osd_luns(char *conf_file)
{
	LUN_STRUCT *lun_struct,*temp,*curr;
 	FILE *fp; int i;char *s;
	lun_struct=NULL;curr=NULL;
	fp=fopen(conf_file,"r");
	char str[MAX_BUF];
	if(fp==NULL) return NULL;

	uint64_t capacity;

	int lun_no;	
	char* token;
	//Add each entry in conf_file to list
	while(!feof(fp))
	{
		str[0]='\0';
		fgets(str,MAX_BUF,fp);
		s=str; i=0;
		if(strlen(s)==0) break;
		if(s[0]=='\n') continue;
		while(isspace(*s)) s++; 
		if(strlen(s)==0) continue;
		if(s[0]=='#') continue;//# for comments

		temp=malloc(sizeof(LUN_STRUCT));
		temp->next=NULL;
		
		token = strtok(s, " ");
		strcpy(temp->osd_lun.osd_target_name,token);
		
		token = strtok(NULL," ");
		sscanf(token,"%d",&lun_no);
		temp->osd_lun.lun_no=lun_no;
		
		token = strtok(NULL," ");
		strcpy(temp->osd_lun.osd_lun_base_directory,token);

		token = strtok(NULL," ");
		sscanf(token,"%llx",&capacity);
		temp->osd_lun.capacity=capacity;

		token = strtok(NULL," ");
		strcpy(temp->osd_lun.osd_system_id,token);

		token = strtok(NULL," ");
		strcpy(temp->osd_lun.username,token);
	
		print_msg(dlevel," OSD_TARGET NAME=%s Lun NO=%d Base Dir=%s Capacity=%llx OSD System ID=%s Username=%s \n",temp->osd_lun.osd_target_name,temp->osd_lun.lun_no,temp->osd_lun.osd_lun_base_directory,temp->osd_lun.capacity,temp->osd_lun.osd_system_id,temp->osd_lun.username);
		
		{
		struct stat stat_buf;
		
		if(stat(temp->osd_lun.osd_lun_base_directory,&stat_buf) == -1){
		print_msg(ERROR_MSG,"Cant create Lun as base_direcotry=%s doesnt exists",temp->osd_lun.osd_lun_base_directory);
		free(temp);
		continue;
		}		
	 	}
		
		if(lun_struct==NULL) 
		{	lun_struct=temp;curr=temp;
			
		}else
		{
			curr->next=temp;curr=temp;
		}
			
	}
		
 	return lun_struct;
}


int get_osd_lun(LUN_STRUCT *lun_struct,char *target_name,int lun,OSD_LUN *osd_lun)
{
	LUN_STRUCT *temp;
	temp=lun_struct;
	print_msg(dlevel,"Searching Target=%s  Lun No=%d \n",target_name,lun);
	while(temp!=NULL)
	{
		print_msg(dlevel,"entry found OSD_TARGET NAME=%s Lun NO=%d Base Dir=%s Capacity=%llx OSD System ID=%s Username=%s \n",temp->osd_lun.osd_target_name,temp->osd_lun.lun_no,temp->osd_lun.osd_lun_base_directory,temp->osd_lun.capacity,temp->osd_lun.osd_system_id,temp->osd_lun.username);
				
		if(!strcmp(temp->osd_lun.osd_target_name,target_name) && temp->osd_lun.lun_no==lun)
		{
			*osd_lun=temp->osd_lun;
			return 0;
		}
			temp=temp->next;

	}
	return 1;
}
