/**
 *                      ==================
 *                      IITK OSD Simulator
 *                      ==================
 *   An Implementation of Simulator for Object based storage device(OSD)
 *
 *   Copyright (C) 2007-2008 CSE IIT KANPUR
 *
 *   Contact: Department of Computer Science and Engineering
 *            Indian Institute of Technology,Kanpur(IIT Kanpur)
 *            {moona,riyaz}@cse.iitk.ac.in
 *
 *   This file is part of IITK OSD Simulator Project.
 *
 *   IITK OSD Simulator Project is free software; you can redistribute it
 *   and/or modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   IITK OSD Simulator Project is distributed in the hope that it will be
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the  GNU General
 *   Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef OSD_EXPORT_H
#define OSD_EXPORT_H
# include "osd_ops.h"
# define MAX_BUF 1024

/**
**********************************
 * File:  osd_export.h
 * Functions related to exporting OSD Luns
 **/

/**
 *
 *    Function:    prepare_osd_luns
 *
 *    Description: Scan conf_file File for OSD Lun entry and added to 
 *		   program maintained list  LUN_STRUCT of OSD Luns 
 * 
 *    Arguments:    @conf_file  - OSD configuration file
 *    Return codes: LUN_STRUCT list of OSD LUNs in the system
 *
 **/
LUN_STRUCT *prepare_osd_luns(char *conf_file);

/**
 *
 *    Function:    get_osd_lun
 *
 *    Description:  Searches for OSD LUN with given target and lun_no in OSD LUN list 
 * 
 *    Arguments:    @LUN_STRUCT  - List of OSD LUNs in the system
 *		    @target_name -Name of OSD target
 *		    @lun -OSD Lun
 *		    @osd_lun - OSD Lun if found for given target	
 *    Return codes: 1 if entry found , 0 otherwise 
 *
 **/

int get_osd_lun(LUN_STRUCT *lun_struct,char *target_name,int lun,OSD_LUN *osd_lun);

/**
 *
 *    Function:    export_osd_luns
 *
 *    Description:  Makes entry of each OSD LUN from list to iSCSI export file
 * 
 *    Arguments:    @osd_luns   - List of OSD LUNs in the system
 *		    @iscsi_export_file - iSCSI export file
 *
 *    Return codes: 1 if entry found , 0 otherwise 
 *
 **/
int export_osd_luns(LUN_STRUCT *osd_luns,char *iscsi_export_file);
#endif /* ifdef OSD_EXPORT_H */

