/**
 *                      ==================
 *                      IITK OSD Simulator
 *                      ==================
 *   An Implementation of Simulator for Object based storage device(OSD)
 *
 *   Copyright (C) 2007-2008 CSE IIT KANPUR
 *
 *   Contact: Department of Computer Science and Engineering
 *            Indian Institute of Technology,Kanpur(IIT Kanpur)
 *            {moona,riyaz}@cse.iitk.ac.in
 *
 *   This file is part of IITK OSD Simulator Project.
 *
 *   IITK OSD Simulator Project is free software; you can redistribute it
 *   and/or modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   IITK OSD Simulator Project is distributed in the hope that it will be
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the  GNU General
 *   Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include <stdint.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "osd_attr_manip.h"

/**
**********************************
 * File:  osd_file_manip.c
 * Functions for reading and writing from object file.
 * Presently functions are for User Mode only.
 **/

//Define debug level for this file
static int dlevel=LOWER_FUNC;
extern int DEBUG;

char k_buff[2048];
extern char osd_directory[FILENAME_SIZE]; 

//-------------------------------------------------------------------
int get_attr_filename(uint64_t part_id,uint64_t object_id, uint32_t page_no,char * filename,int object_type)
{
  	char buf[FILENAME_SIZE];
  	if(page_no == OSD_PAGE_NUM_CUR_CMD)
	{
		sprintf(buf,"//current_command_attr");
	}
	else if(object_type == ROOT_OBJECT_TYPE)
	 	sprintf(buf,"//root_attr");
	
	
	 else if(object_type == PART_OBJECT_TYPE)
	 	{
			if(part_id ==0)
			sprintf(buf,"//0x%llx_attr",part_id);
			else
	 	 	sprintf(buf,"//0x%llx//_attr",part_id);
 
		} 
	else if( object_type == UOBJ_OBJECT_TYPE)
		sprintf(buf,"//0x%llx//0x%llx_attr",part_id,object_id); 
	else if(object_type == COLL_OBJECT_TYPE)
		 sprintf(buf,"//0x%llx//0x%llx_coll_attr",part_id,object_id);
	filename[0]='\0';
	strcat(filename,osd_directory);
	strcat(filename,buf);
	print_msg(dlevel,"filename=%s",filename);
	return 0;	
}

//-------------------------------------------------------------------

int get_data_filename(uint64_t part_id,uint64_t object_id, uint32_t page_no,char * filename,int object_type)
{
        char buf[FILENAME_SIZE];
	
	if(object_type == ROOT_OBJECT_TYPE)
	 	sprintf(buf,"//root_data");
	else if(object_type == CURR_CMD_OBJECT_TYPE)
	{
	//This is error
		
	}
	else if(object_type == PART_OBJECT_TYPE)
		 sprintf(buf,"//0x%llx//_data",part_id);
	else if(object_type == UOBJ_OBJECT_TYPE)
		sprintf(buf,"//0x%llx//0x%llx_data",part_id,object_id); 
	else if(object_type == COLL_OBJECT_TYPE)
		sprintf(buf,"//0x%llx//0x%llx_coll_data",part_id,object_id);
		
	
	filename[0]='\0';
        strcat(filename,osd_directory);
        strcat(filename,buf);
	print_msg(dlevel,"filename=%s",filename);
	return 0;

}
//-------------------------------------------------------------------
    
int file_write(char *filename,char *buffer,unsigned size,long offset)
{
	int fd;

	fd=open(filename,O_RDWR|O_CREAT);
	if (lseek(fd, offset, SEEK_SET) != offset )
	return 1; 
	if (write(fd,buffer,size) != size)
	return 1;	
	close(fd);
	return 0;


}

//-------------------------------------------------------------------

int file_read(char *filename,char *buffer,unsigned size,long offset)
{
	int fd;
	fd=open(filename,O_RDWR|O_CREAT);
	if ( lseek(fd, offset, SEEK_SET) != offset )
	{
	print_msg(ERROR_MSG," lseek failed for  0x%lx ",offset);
	return 1; 
	
	}
	if (read(fd,buffer,size) != size)
	{
	print_msg(ERROR_MSG," read failed ");
	return 1;	
	
	}
	close(fd);
	return 0;


}

//-------------------------------------------------------------------

int object_exists(uint64_t part_id,uint64_t object_id,int object_type)
{
//FUTURE: Instead of stat call,,we can search in part_id data file for the object
	       char filename[FILENAME_SIZE];
	       get_data_filename(part_id,object_id,0,filename,object_type);
# ifndef _KERNEL_
	{
	 	struct stat stat_buf;
		
		if(stat(filename,&stat_buf) == -1)
			return 0;
		return 1;
	 
	
	}
#endif /** ifnef _KERNEL_ */

}

//-------------------------------------------------------------------
int create_object_data_file(uint64_t part_id,uint64_t object_id,int object_type)
{
       char filename[FILENAME_SIZE];
       get_data_filename(part_id,object_id,0,filename,object_type);
       
       if(object_exists(part_id,object_id,object_type))
       return 1;

# ifndef _KERNEL_
	
	if(object_type==PART_OBJECT_TYPE)
	{
		char part_id_str[FILENAME_SIZE];
		char path[FILENAME_SIZE];
		strcpy(path,osd_directory);
		strcat(path,"//");
		sprintf(part_id_str,"0x%llx",part_id);
		strcat(path,part_id_str);
		print_msg(dlevel,"Creating directory  %s \n",path);
		
		if(mkdir(path,DIRECTORY_MODE))
		{	
			print_msg(ERROR_MSG,"Error in creating partition directory");
		
			return 1;
		}
	
	}
	{
		int fd;
		fd=open(filename,O_RDWR|O_CREAT);
		close(fd);
	}
	return 0;
#endif /* ifndef _KERNEL_*/
}
//-------------------------------------------------------------------

uint64_t get_random_object_id(int object_type)
{
	 uint64_t no;
	 no=rand()%(1024*1024)*1024+512;
	return no;
}

//-------------------------------------------------------------------
//We will never call this for add_object_type==ROOT_OBJECT_TYPE
int add_object_to_collection_list(uint64_t coll_part_id,uint64_t coll_id,uint64_t add_obj_id,int add_object_type)
{
char coll_filename[FILENAME_SIZE];
	//FUTURE need error handling here
	//We are converting to big endian for speed up of list commands
	add_obj_id=cpu_to_be64(add_obj_id);
	
	if(add_object_type == PART_OBJECT_TYPE)
	{
		get_data_filename(0,0,0,coll_filename,ROOT_OBJECT_TYPE);	//part-id=0 obj=id=0 page-no=0
		add_object_to_file(coll_filename,add_obj_id);
		return 0;
	}
	//Add object_type is COLL or UOBJ type

	if(coll_id==0) //List object is Partition type
	{
		get_data_filename(coll_part_id,0,0,coll_filename,PART_OBJECT_TYPE);	//part-id=coll_part_id obj=id=0 page-no=0
		add_object_to_file(coll_filename,add_obj_id);
		return 0;
	
	}
	
		get_data_filename(coll_part_id,coll_id,0,coll_filename,COLL_OBJECT_TYPE);	//part-id=coll_part_id obj=id=0 page-no=0
		add_object_to_file(coll_filename,add_obj_id);
		return 0;
	
	


}
//-------------------------------------------------------------------
int remove_object_from_collection_list(uint64_t coll_part_id,uint64_t coll_id,uint64_t rem_obj_id,int rem_object_type)
{
char coll_filename[FILENAME_SIZE];
	//FUTURE need error handling here
	//We are converting to big endian for speed up of list commands
	rem_obj_id=cpu_to_be64(rem_obj_id);

	if(rem_object_type == PART_OBJECT_TYPE)
	{
		get_data_filename(0,0,0,coll_filename,ROOT_OBJECT_TYPE);	//part-id=0 obj=id=0 page-no=0
		return remove_object_from_file(coll_filename,rem_obj_id);
		
	}
	//Add object_type is COLL or UOBJ type
	
	if(coll_id==0) //List object is Partition type
	{
		get_data_filename(coll_part_id,0,0,coll_filename,PART_OBJECT_TYPE);	//part-id=coll_part_id obj=id=0 page-no=0
		return remove_object_from_file(coll_filename,rem_obj_id);
		
	
	}

	get_data_filename(coll_part_id,coll_id,0,coll_filename,COLL_OBJECT_TYPE);	//part-id=coll_part_id obj=id=0 page-no=0
	return remove_object_from_file(coll_filename,rem_obj_id);

	
}
//-------------------------------------------------------------------

uint32_t iov_file_copy(int fd,OSD_DATA_IOV *data_iov,uint32_t initial_offset,uint32_t read_len)
{
	int iov_indx=0;
	uint32_t offset=0;
	read_len+=8;
	uint32_t total_read_len=0;
	uint32_t read_size,len,iov_size;
	//initial_offset=0;
	//extra read for checking end of file occured or not
	while(offset< initial_offset)
	{
		if(offset + data_iov->iov[iov_indx].iov_len >initial_offset)
		{
			offset=initial_offset-offset;
			break;
		}  
		iov_indx++;
		offset+=data_iov->iov[iov_indx].iov_len ;
	}
	

	
	while(total_read_len<read_len)
	{
		iov_size= data_iov->iov[iov_indx].iov_len-offset;
		read_size=read_len-total_read_len;
		
		if(read_size > iov_size)
		read_size=iov_size;
			
		len=read(fd,(uint8_t *)data_iov->iov[iov_indx].iov_base+offset,read_size);

		total_read_len+=len;
		
		if(len < read_size)   
		return total_read_len;
		offset=0;
		iov_indx++;
	}
	
	return total_read_len;

}
//-------------------------------------------------------------------

int list_objects_from_collection_list(OSD_LIST_COMMAND_REPLY *list_reply)
{
 uint32_t read_len=0;
 
 char coll_filename[FILENAME_SIZE];
 

//We are converting to big endian for speed up of list commands
uint64_t continuation_id=cpu_to_be64(list_reply->continuation_id);

 
 get_data_filename(list_reply->coll_part_id,list_reply->coll_id,0,coll_filename,list_reply->object_type);	//part-id=coll_part_id obj=id=0 page-no=0
	//print("continuation_id =0x%llx \n",list_reply->continuation_id);
  if(! (list_reply->continuation_id))
  {

#ifndef _KERNEL_	
	int fd;
	fd=open(coll_filename,O_RDWR|O_CREAT);

	read_len=iov_file_copy(fd,(list_reply->data_buf),list_reply->initial_data_offset,list_reply->data_buf_size+8);

	if(read_len <= list_reply->data_buf_size)
	list_reply->final=1;
	else
	{
	list_reply->final=0;
	read_len =list_reply->data_buf_size;
	}
	
	close(fd);
	
  	
 #endif /*  ifndef _KERNEL_*/
 	print_msg(dlevel,"read_len =0x%lx \n",read_len);
 	list_reply->object_count=read_len/8; 
	return 0;
  }
  else
  {
 #ifndef _KERNEL_	 


  	uint8_t buffer[FILE_RW_BUFF_SIZE];
	int final_buf_read=0,done=0,curr=0,read_len=0,i,done_len=0;
	int fd;
	
	fd=open(coll_filename,O_RDWR|O_CREAT);
	while(!final_buf_read && !done)
	{
	
		read_len=read(fd,buffer,FILE_RW_BUFF_SIZE);
		if (read_len != FILE_RW_BUFF_SIZE) final_buf_read=1;
	
			curr=0;
			for(i=0;i< read_len/8 ;i++)
			{
	
			   if(continuation_id == *((uint64_t *)((char *)buffer +curr)))
			   {
				curr+=8;
				done_len=read_len-curr;
				//memcpy(list_reply->data_buf,(uint8_t *)buffer+curr,done_len);
				iov_bufcpy(list_reply->data_buf,(uint8_t *)buffer+curr,list_reply->initial_data_offset,done_len,TO_IOV_BUF);
				
				done=1;
				break;//break for i
			   
			   }	
			   curr += 8;
			}	
		
	}
	if(done)
	{
		read_len=0;
		if(final_buf_read)
		{
			list_reply->final=1;
		}else
		{
			int rem_data_len=list_reply->data_buf_size-done_len;	
			read_len=iov_file_copy(fd,(list_reply->data_buf),list_reply->initial_data_offset+done_len,rem_data_len+8);
	
			if(read_len <= rem_data_len )
			list_reply->final=1;
			else  {	
			list_reply->final=0;
			read_len=rem_data_len ;	
			}
				
				
		}
		list_reply->object_count=(done_len+read_len)/8;
		
		close(fd);
		
		return 0;
	}
	else //COntinuation object not found 
	{
		print_msg(ERROR_MSG,"Continuation object not found\n");
		return 1;	
	
	}
  
  #endif /** ifndef _KERNEL_ */
  }
  
}
//-------------------------------------------------------------------
int add_object_to_file(const char *filename,uint64_t add_obj_id)
{
	 
# ifndef _KERNEL_
	int fd;
	fd=open(filename,O_RDWR|O_CREAT);
	lseek(fd, 0, SEEK_END) ;
	if (write(fd,&add_obj_id,8) != 8)
	return 1;	
	close(fd);
# endif /** ifndef _KERNEL_ */
	return 0;
}

//-------------------------------------------------------------------
int remove_object_from_file(const char *filename,uint64_t rem_obj_id)
{
	uint8_t buffer[FILE_RW_BUFF_SIZE];
	
	char temp_file[FILENAME_SIZE]; 
	int final_buf_read=0,i;
	int read_len,curr=0,done=0;
	strcpy(temp_file,filename);
	strcat(temp_file,".tmp");
	int fd,fd1;
# ifndef _KERNEL_
	
	fd=open(filename,O_RDWR|O_CREAT);
	fd1=open(temp_file,O_RDWR|O_CREAT);
	while(!final_buf_read)
	{
		
		read_len=read(fd,buffer,FILE_RW_BUFF_SIZE);
		if (read_len != FILE_RW_BUFF_SIZE)
			final_buf_read=1;
		if(!done)
		{
			curr=0;
			for(i=0;i< read_len/8 ;i++)
			{
			   if(rem_obj_id == *((uint64_t *)((char *)buffer +curr)))
			   {
			   	write(fd1,buffer,curr);
			   	curr+=8;
				write(fd1,(char*)buffer+curr,read_len-curr);
			   	done=1;
				break;//break for
			   
			   }	
			   else	
				curr += 8;
			}	
			
		
		}
		if(!done)
		write(fd1,buffer,read_len);
			
	}

	close(fd);
	close(fd1);	
	rename(temp_file,filename);
# endif /** ifndef _KERNEL_ */
	
	if(done) return 0;
	
	return 1;
	
}

//-------------------------------------------------------------------
int print_objects_from_collection_list(OSD_LIST_COMMAND_REPLY *list_reply)
{	
	int i;
	uint64_t oid;
	PRINT_OSD_LIST_COMMAND_REPLY_PTR(list_reply);
	
	print_msg(dlevel,"List=");
	print_msg(ERROR_MSG,"List=");
	for(i=0;i<list_reply->object_count;i++)
	{
		if(iov_bufcpy(list_reply->data_buf,(uint8_t *) &oid,i*8+list_reply->initial_data_offset,8,FROM_IOV_BUF)!=8)
		{	
			print_msg(ERROR_MSG,"OOPS Error in copying from IOVEC to buffer");
			return 1; 
		}
		oid=be64_to_cpu(oid);
	}
	print("\n");
	return 0;

}
//-------------------------------------------------------------------

int remove_object_data_file(uint64_t part_id,uint64_t object_id,int object_type)
{
 char data_filename[FILENAME_SIZE];
	if(object_type==PART_OBJECT_TYPE)
	{
		get_data_filename(part_id,object_id,0,data_filename,object_type);	
		
#ifndef _KERNEL_
		
		if( remove(data_filename))
		{
			print_msg(ERROR_MSG,"Error in removing partition data file");
			return 1;
		}
		
		{
			char part_id_str[FILENAME_SIZE];
			strcpy(data_filename,osd_directory);
			strcat(data_filename,"//");
			sprintf(part_id_str,"0x%llx",part_id);
			strcat(data_filename,part_id_str);
			if(rmdir(data_filename))
			{
				print_msg(ERROR_MSG,"Error in removing partition directory=%s",data_filename);
				return 1;
				}
		}	
#endif /** ifndef _KERNEL_ */
		
		return 0; 	
	}
	if(object_type==UOBJ_OBJECT_TYPE ||object_type==COLL_OBJECT_TYPE )
	{
	
		get_data_filename(part_id,object_id,0,data_filename,object_type);	
	
#ifndef _KERNEL_
		
		if( remove(data_filename))
		{
			print_msg(ERROR_MSG,"Error in removing partition data file=%s",data_filename);
			return 1;
		}
#endif /** ifndef _KERNEL_ */
		return 0;
	
	}
	

	return 1;
}
//-------------------------------------------------------------------
int remove_object_attr_file(uint64_t part_id,uint64_t object_id,int object_type)
{
	//We will handle case of Extended and vendor specific attribute later
	char filename[FILENAME_SIZE];

	get_attr_filename(part_id,object_id,0,filename,object_type);

#ifndef _KERNEL_
		
		if( remove(filename))
		{
			print_msg(ERROR_MSG,"Error in removing partition data file= %s",filename);
			return 1;
		}
#endif /** ifndef _KERNEL_ */
		return 0;





}
//-------------------------------------------------------------------

uint64_t perform_rw_operation(uint64_t part_id,uint64_t object_id,OSD_DATA_IOV *data_iov,uint64_t iovec_start_offset,uint64_t obj_start_address,uint64_t length,int rw)
{
	char filename[FILENAME_SIZE];
	uint64_t offset=0;int iov_indx=0;
	uint64_t iov_size,rwrite_size,total_rwrite_len=0,len,rwrite_len=length;
	int fd;

	get_data_filename(part_id,object_id,0,filename,UOBJ_OBJECT_TYPE);//page_no=0

#ifndef _KERNEL_
	if(rw)
		fd=open(filename,O_RDONLY);
	else
		fd=open(filename,O_RDWR|O_CREAT);
	
	lseek(fd,obj_start_address, SEEK_SET);
	while(offset< iovec_start_offset)
	{
		if(offset + data_iov->iov[iov_indx].iov_len >iovec_start_offset)
		{
			offset=iovec_start_offset-offset;
			break;
		}  
		iov_indx++;
		offset+=data_iov->iov[iov_indx].iov_len ;
	}
	

	while(total_rwrite_len<rwrite_len)
	{
		iov_size= data_iov->iov[iov_indx].iov_len-offset;
		rwrite_size=rwrite_len-total_rwrite_len;
		
		if(rwrite_size > iov_size)
		rwrite_size=iov_size;
		
	
		if(rw)
		len=read(fd,(uint8_t *)(data_iov->iov[iov_indx].iov_base)+offset,rwrite_size);
		else	
		len=write(fd,(uint8_t *)(data_iov->iov[iov_indx].iov_base) + offset,rwrite_size);
		
		total_rwrite_len+=len;
		
		if(len < rwrite_size)   
		{
		close(fd);
		return total_rwrite_len;
		}
		offset=0;
		iov_indx++;
	}
	close(fd);
	return total_rwrite_len;

#endif /** ifndef _KERNEL_ */
}
//-------------------------------------------------------------------
int is_uobject_member_of_collection(uint64_t part_id,uint64_t obj_id,uint64_t coll_obj_id)
{

	char coll_filename[FILENAME_SIZE];
	
	uint8_t buffer[FILE_RW_BUFF_SIZE];
	
	int final_buf_read=0,i;
	int read_len,curr=0;
	int fd;
	
	get_data_filename(part_id,coll_obj_id,0,coll_filename,COLL_OBJECT_TYPE);	//part-id=0 obj=id=0 page-no=0
	obj_id=cpu_to_be64(obj_id);
# ifndef _KERNEL_
	fd=open(coll_filename,O_RDWR|O_CREAT);

	while(!final_buf_read)
	{
		read_len=read(fd,buffer,FILE_RW_BUFF_SIZE);
		if (read_len != FILE_RW_BUFF_SIZE)
			final_buf_read=1;
		curr=0;
		for(i=0;i< read_len/8 ;i++)
		{
		   if(obj_id == *((uint64_t *)((char *)buffer +curr)))
		   {
			return 1;			   
		   }	
		   else	
			curr += 8;
		}	
	}
	close(fd);
# endif /** ifndef _KERNEL_ */
return 0;
}
//-------------------------------------------------------------------
