/**
 *                      ==================
 *                      IITK OSD Simulator
 *                      ==================
 *   An Implementation of Simulator for Object based storage device(OSD)
 *
 *   Copyright (C) 2007-2008 CSE IIT KANPUR
 *
 *   Contact: Department of Computer Science and Engineering
 *            Indian Institute of Technology,Kanpur(IIT Kanpur)
 *            {moona,riyaz}@cse.iitk.ac.in
 *
 *   This file is part of IITK OSD Simulator Project.
 *
 *   IITK OSD Simulator Project is free software; you can redistribute it
 *   and/or modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   IITK OSD Simulator Project is distributed in the hope that it will be
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the  GNU General
 *   Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

# include "osd_req_ops.h"

/**
**********************************
 * File:  osd_req_ops.c
 * Implements top level functions for which invokes OSD module communication 
 *  And OSD command processing functions
 **/
extern int DEBUG;
static int dlevel=UPPER_FUNC;
//-----------------------------------------------------------------------------
//Protocol of communication between OSD target and iSCSI Module is explained in README file
int osd_request_receive(OSD_REQUEST *osd_req,int sockfd)
{

	int len;
	u8 hello_buf[HELLO_BUF_SIZE];	
	print_msg(dlevel,"Request accepted, processing now\n");
	OSD_COMMAND_INFO osd_command_info;
	
	OSD_CDB_STRUCT *osd_cdb;
	osd_cdb=&(osd_req->osd_cdb_struct);		
	
	//Get LUN INFO structure
	len=read(sockfd,&osd_command_info,SOCKET_MAX_READ);
	if(len != sizeof(osd_command_info))
	{
		print_msg(ERROR_MSG,"Error receiving LUN_INFO from Kernel \n");
		return 1;
	}
	print_msg(ERROR_MSG,"OSD_COMMAND_INFO received succefully\n");

	if(write(sockfd,hello_buf,HELLO_BUF_SIZE)!=HELLO_BUF_SIZE)
	{
		print_msg(ERROR_MSG,"Error Sending Hello Buf for LUN_INFO \n");
		return 1;
	} 
	print_msg(ERROR_MSG,"OSD COMMAND INFO Ack to kernel  sent successfully\n");
	
        print_msg(ERROR_MSG,"LUN_INFO Target=%s Lun=%d",osd_command_info.osd_lun_info.target_name,osd_command_info.osd_lun_info.lun_no);
        strcpy(osd_req->target_name,osd_command_info.osd_lun_info.target_name);
        osd_req->lun_no=osd_command_info.osd_lun_info.lun_no;

	//Parse OSD command header
	osd_req->data_iov_size=osd_command_info.osd_header.iov_size;

	if(handle_osd_cmnd_get_osd_cdb_struct(&(osd_command_info.osd_header),osd_cdb))
	{
	
		print_msg(ERROR_MSG,"Error  Extracing Kernel User Metadata \n");
		return 1;
	
	}
	print_msg(dlevel," handle_osd_cmnd_get_osd_cdb_struct done\n");
	
	print_msg(dlevel," osd_header.data_len =0x%x \n",osd_command_info.osd_header.data_len );

	//Receive Data IN if any 
	if(osd_command_info.osd_header.data_len )
	{
		u32 data_received=0;
		int i=0;
		OSD_DATA osd_data;
		osd_data_iov_alloc(&(osd_req->data_in),osd_command_info.osd_header.data_len,osd_req->data_iov_size)
;
		print_msg(dlevel,"Receiving data_len=0x%x\n",osd_command_info.osd_header.data_len);
	
		for(i=0;i<osd_req->data_in.iov_count;i++)
		{
			len=read(sockfd,&osd_data,SOCKET_MAX_READ);
			 
			 if(!len || len != (osd_data.data_len+2*sizeof(int))) 
			 {	
			 	print_msg(ERROR_MSG,"Error Receiving data from kernel for i=%d  recevied=0x%x sent=0x%x\n",i,len,(osd_data.data_len+2*sizeof(int)));
				return 1;
			}	
			print_msg(dlevel,"For IOVEC no=%d received data=0x%x\n",i,len);
			//osd_data.data_len
			
			memcpy(osd_req->data_in.iov[i].iov_base,osd_data.data,osd_data.data_len);
			osd_req->data_in.iov[i].iov_len=osd_data.data_len;
			data_received+=(len-2*sizeof(int));
			
			if(osd_data.final)
			{
				if(data_received != osd_command_info.osd_header.data_len )
				{
					print_msg(ERROR_MSG,"Error in receving   data from Kernel Total data Expected =0x%x and Received=0x%x \n",osd_command_info.osd_header.data_len,data_received);
					return 1;
				
				}
				break;
				
			}
			else
			{
				//As per the protocol, send ack
				if(write(sockfd,hello_buf,HELLO_BUF_SIZE)!=HELLO_BUF_SIZE)
				{
				print_msg(ERROR_MSG,"Error  Sending Hello Buf for  Kernel User Metadata \n");
				return 1;
				}			
			
			}
			
		}
	
	}

	return 0;
}
// 
//-----------------------------------------------------------------------------

int osd_request_extract_get_set_attr(OSD_REQUEST *osd_req)
{
	OSD_CDB_ATTR_LIST_HDR *osd_cdb_attr_list_hdr;
	OSD_CDB_STRUCT *osd_cdb;
	
	osd_cdb=&(osd_req->osd_cdb_struct);		
	osd_cdb_attr_list_hdr=(OSD_CDB_ATTR_LIST_HDR *)osd_cdb;
	
	//Extract Set Attr List
	if(osd_cdb_attr_list_hdr->set_list_length)
	{
		init_not_initiated_command_functions_bit(osd_req->not_initiated_command_functions,COMMAND_FUNCTIONS_SET_ATTR,COMMAND_FUNCTIONS_SET_ATTR);
	
		if(extract_set_attribute_list(osd_req))
		{
			print_msg(ERROR_MSG,"Error in extract_set_attribute_list\n");
			osd_req->osd_response_header=build_osd_response_header(SCSI_CHECK_CONDITION,SCSI_SENSE_KEY_ILLEGAL_REQUEST,SCSI_INVALID_FIELD_IN_PARAMETER_LIST_ASC,SCSI_INVALID_FIELD_IN_PARAMETER_LIST_ASCQ,0);
			return 1;
		
		}
	}
	
	//Extract Get Attr List
	if(osd_cdb_attr_list_hdr->get_list_length)
	{
		init_not_initiated_command_functions_bit(osd_req->not_initiated_command_functions,COMMAND_FUNCTIONS_GET_ATTR,COMMAND_FUNCTIONS_GET_ATTR);
		if(extract_get_attribute_list(osd_req))
		{
			print_msg(ERROR_MSG,"Error in extract_get_attribute_list\n");
			osd_req->osd_response_header=build_osd_response_header(SCSI_CHECK_CONDITION,SCSI_SENSE_KEY_ILLEGAL_REQUEST,SCSI_INVALID_FIELD_IN_PARAMETER_LIST_ASC,SCSI_INVALID_FIELD_IN_PARAMETER_LIST_ASCQ,0);
			return 1;
		}
	
	}
	return 0;	

}

//-----------------------------------------------------------------------------
	
int osd_request_process(OSD_REQUEST *osd_req)
{
	int ret=0;
	u32 dout_size=handle_osd_cmnd_get_dout_size(&(osd_req->osd_cdb_struct));
	print_msg(dlevel,"dout buffer size=0x%lx",dout_size);
	osd_data_iov_alloc(&(osd_req->data_out),dout_size,osd_req->data_iov_size);
	
	switch(cmnd_opcode(osd_req))
	{
	case OSD_CMD_FORMAT:
				ret=osd_format(osd_req);
				break;
	
	case OSD_CMD_CREATE_PARTITION :
				ret=osd_create_partition(osd_req);
				break;							
	case OSD_CMD_CREATE_COLLECTION:	
				ret=osd_create_collection(osd_req);
				break;
	case OSD_CMD_ADD_TO_COLLECTION:	
				ret=osd_add_to_collection(osd_req);
				break;
	case OSD_CMD_REMOVE_FROM_COLLECTION:	
				ret=osd_remove_from_collection(osd_req);
				break;
	case OSD_CMD_CREATE:	
				ret=osd_create(osd_req);
				break;
	case OSD_CMD_REMOVE:
				ret=osd_remove(osd_req);
				break;
	case OSD_CMD_REMOVE_PARTITION:
				ret=osd_remove_partition(osd_req);
				break;
	case OSD_CMD_REMOVE_COLLECTION:
				ret=osd_remove_collection(osd_req);
				break;
	case OSD_CMD_READ:
				ret=osd_read(osd_req);
				break;	
	case OSD_CMD_WRITE:
				ret=osd_write_append(osd_req);
				break;	
	case OSD_CMD_APPEND:			
				ret=osd_write_append(osd_req);
				break;	
	case OSD_CMD_LIST:
				ret=osd_list(osd_req);
				break;					
	case OSD_CMD_LIST_COLLECTION:
				ret=osd_list_collection(osd_req);
				break;			
	case OSD_CMD_SET_ATTR:
	case OSD_CMD_GET_ATTR:
				ret=osd_get_set_attr(osd_req);
				break;	
	case OSD_CMD_FLUSH:
	case OSD_CMD_FLUSH_PARTITION:
	case OSD_CMD_FLUSH_OSD:
				ret=osd_other_command(osd_req);
				break;
	case OSD_CMD_CREATE_AND_WRITE:
				ret=osd_create_write(osd_req);
				break;
	case OSD_CMD_FLUSH_COLLECTION:
	//case OSD_CMD_PERFORM_TASK_MGMT:
	//case OSD_CMD_PERFORM_SCSI_COMMAND:
	case OSD_CMD_INQUIRY:
				//it will done by iSCSI module
				break;
	}			
	return ret;
}
//-----------------------------------------------------------------------------
//Protocol of communication between OSD target and iSCSI Module is explained in README file
int osd_request_send_response(OSD_REQUEST *osd_req,int sockfd)
{
	int len;
	u8 hello_buf[HELLO_BUF_SIZE];		

	osd_req->osd_response_header.cmnd_status.sense_data_size=
	osd_req->osd_sense_response.sense_data_size;
	len=sizeof(osd_req->osd_response_header);
	//Send OSD Response Header	
	if(write(sockfd,&(osd_req->osd_response_header),len)!=len)
	{
		print_msg(ERROR_MSG,"Error  Sending OSD_response header to  Kernel User Metadata \n");
		return 1;
	}			
				
	print_msg(dlevel,"osd_response_header sent successfully \n");
	if(read(sockfd,hello_buf,HELLO_BUF_SIZE)!=HELLO_BUF_SIZE)
	{
		print_msg(ERROR_MSG,"Error  Receiving Hello Buf for  OSD Response header ack  \n");
		return 1;
	}			
	print_msg(dlevel,"osd_response_header ack received successfully \n");	
	
	//Send Sense data if error in OSD command processing	
	if(osd_req->osd_response_header.cmnd_status.status)
	{
		if(osd_req->osd_response_header.cmnd_status.sense_data_size)
		{
			print_msg(dlevel,"Sending sense data \n");		
			len=osd_req->osd_response_header.cmnd_status.sense_data_size;
			
			if(write(sockfd,osd_req->osd_sense_response.sense_buf,len)!=len)
			{
				print_msg(ERROR_MSG,"Error sending sense data\n");
				return 1;
			}			
			print_msg(dlevel,"sense data sent successfully \n");
	
			if(read(sockfd,hello_buf,HELLO_BUF_SIZE)!=HELLO_BUF_SIZE)
			{
				print_msg(ERROR_MSG,"Error  Receiving Hello Buf for  sense data sent ack  \n");
				return 1;
			}			
			print_msg(dlevel,"sense data ack received successfully \n");		

		}
		else
			print_msg(dlevel,"No sense data to send");		
	}//Send OSD Data OUT	
	else if(osd_req->osd_response_header.data_len)//for sending data 
	{
		OSD_DATA osd_data;
		u32 data_len=osd_req->osd_response_header.data_len;
		u32 offset=0,len;
		u16 iov_size;
		iov_size=osd_req->data_iov_size;
		print_msg(ERROR_MSG,"sending osd data to kernel");
		print_msg(ERROR_MSG,"data_len =0x%x",osd_req->osd_response_header.data_len);
		
		while(data_len > 0)
		{
			if(data_len > iov_size) 
			{	
				len=iov_bufcpy(&(osd_req->data_out),osd_data.data,offset,iov_size,FROM_IOV_BUF);
				osd_data.data_len=iov_size;
				offset+=iov_size;
				data_len-=iov_size;
				osd_data.final=0;
				
			}	
		
			else
			{	len=iov_bufcpy(&(osd_req->data_out),osd_data.data,offset,  data_len,FROM_IOV_BUF);
				osd_data.data_len=data_len;
				offset+=data_len;
				data_len-=data_len;
				osd_data.final=1;
			}
			len=osd_data.data_len+2*sizeof(int);
			print_msg(ERROR_MSG,"sending osd_data data_len =0x%x ",osd_data.data_len);

			if(write(sockfd,&osd_data,len)!= len)
			{
				print_msg(ERROR_MSG,"Error sending data to Kernel  \n");
				return 0;
			}			
			print_msg(dlevel,"osd_read_data sent successfully \n");
				
		
			if(read(sockfd,hello_buf,HELLO_BUF_SIZE)!=HELLO_BUF_SIZE)
			{
				print_msg(ERROR_MSG,"Error  Receiving Hello Buf for  OSD Read Response \n");
				return 0;
			}			
			print_msg(dlevel,"osd_read_data ack received successfully \n");
		}
	}			
	return 0;	

}
//-----------------------------------------------------------------------------
