/**
 *                      ==================
 *                      IITK OSD Simulator
 *                      ==================
 *   An Implementation of Simulator for Object based storage device(OSD)
 *
 *   Copyright (C) 2007-2008 CSE IIT KANPUR
 *
 *   Contact: Department of Computer Science and Engineering
 *            Indian Institute of Technology,Kanpur(IIT Kanpur)
 *            {moona,riyaz}@cse.iitk.ac.in
 *
 *   This file is part of IITK OSD Simulator Project.
 *
 *   IITK OSD Simulator Project is free software; you can redistribute it
 *   and/or modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   IITK OSD Simulator Project is distributed in the hope that it will be
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the  GNU General
 *   Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef OSD_REQ_OPS_H
#define OSD_REQ_OPS_H

#include<sys/types.h>
#include<sys/socket.h>
#include<netinet/in.h>
#include<arpa/inet.h>
#include<errno.h>
//# include "osd.h"
//# include "osd_file_manip.h"
# include "osd_ops.h"
# include "osd_sense.h"
# define SOCKET_MAX_READ  8192
#define SERV_HOST_ADDR "127.0.0.1"   
#define HELLO_BUF_SIZE 100

/**
**********************************
 * File:  osd_req_ops.h
 * Top level functions for which invokes OSD module communication and OSD command processing functions
 **/

/**
 *
 *    Functions:    osd_request_receive
 *
 *    Description:  Receive OSD command request from iSCSI Module on given socket channel 
 * 
 *    Arguments:    @osd_req - OSD request
 *		    @sockefd - Socket channel	
 *	
 *    Return codes: 0 for success,1 in error 
 *
 **/
int osd_request_receive(OSD_REQUEST *osd_req,int sockfd);

/**
 *
 *    Functions:    osd_request_extract_get_set_attr
 *
 *    Description:  Function which calls extract_get/set_attr_list
 * 
 *    Arguments:    @osd_req - OSD request
 *	
 *    Return codes: 0 for success,1 in error 
 *
 **/
int osd_request_extract_get_set_attr(OSD_REQUEST *osd_req);	

/**
 *
 *    Functions:    osd_request_process
 *
 *    Description:  Function to process OSD request
 * 
 *    Arguments:    @osd_req - OSD request
 *	
 *    Return codes: 0 for success,1 in error 
 *
 **/
int osd_request_process(OSD_REQUEST *osd_req);


/**
 *
 *    Functions:    osd_request_send_response
 *
 *    Description:  Function to send OSD comamnd response to iSCSI Module on given socket channel 
 * 
 *    Arguments:    @osd_req - OSD request
 *		    @sockefd - Socket channel	
 * 
 *     Return codes: 0 for success,1 in error 
 *
 **/
int osd_request_send_response(OSD_REQUEST *osd_req,int sockfd);

#endif
