/**
 *                      ==================
 *                      IITK OSD Simulator
 *                      ==================
 *   An Implementation of Simulator for Object based storage device(OSD)
 *
 *   Copyright (C) 2007-2008 CSE IIT KANPUR
 *
 *   Contact: Department of Computer Science and Engineering
 *            Indian Institute of Technology,Kanpur(IIT Kanpur)
 *            {moona,riyaz}@cse.iitk.ac.in
 *
 *   This file is part of IITK OSD Simulator Project.
 *
 *   IITK OSD Simulator Project is free software; you can redistribute it
 *   and/or modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   IITK OSD Simulator Project is distributed in the hope that it will be
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the  GNU General
 *   Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

# include "osd_ops.h"
# include "osd_sense.h"

/**
**********************************
 * File:  osd_sense.c
 * Implementation of functions for OSD sense data generation 
 **/
//------------------------------------------------------------------------

int create_osd_error_id_sense_data_desc(OSD_REQUEST *osd_req)
{
	
	OSD_CDB_ANY_COMMAND_HDR *osd_cdb_any_command;
    	ERROR_ID_SENSE_DESC *error_id_sense;
	error_id_sense=(ERROR_ID_SENSE_DESC *)((u8 *)osd_req->osd_sense_response.sense_buf+osd_req->osd_sense_response.sense_data_size);
    
    	error_id_sense->desc_type=OSD_ERROR_ID_SENSE_DATA_DESC_TYPE; //6 object_id_sense_desc//u8
    	error_id_sense->add_len=OSD_ERROR_ID_SENSE_DATA_DESC_ADD_LEN; //0x1E //u8
    	//u8 resvd[6];
	memcpy(error_id_sense->not_initiated_command_functions,osd_req->not_initiated_command_functions,4);
       	memcpy(error_id_sense->completed_command_functions,osd_req->completed_command_functions,4);     
    	osd_cdb_any_command=(OSD_CDB_ANY_COMMAND_HDR *)&(osd_req->osd_cdb_struct); 
    	error_id_sense->part_id=osd_cdb_any_command->part_id;
    	error_id_sense->obj_id=osd_cdb_any_command->object_id;
       	osd_req->osd_sense_response.sense_data_size+=sizeof(ERROR_ID_SENSE_DESC);
	return 0;
}

//------------------------------------------------------------------------
/*
int create_osd_rsp_int_chk_val_data_desc(OSD_REQUEST *osd_req);
*/
//------------------------------------------------------------------------
int create_osd_attr_id_sense_data_desc(OSD_REQUEST *osd_req,u32 page_no,u32 attr_no)
{
	ATTR_ID_SENSE_DESC *attr_id_sense;
	attr_id_sense=(ATTR_ID_SENSE_DESC *)((u8 *)osd_req->osd_sense_response.sense_buf+osd_req->osd_sense_response.sense_data_size);    
	attr_id_sense->key=OSD_ATTR_ID_SENSE_DATA_DESC_TYPE;
	attr_id_sense->add_len=sizeof(ATTR_ID_SENSE_DESC)-2;
	attr_id_sense->page_no=page_no;
	attr_id_sense->attr_no=attr_no;
	osd_req->osd_sense_response.sense_data_size+=sizeof(ATTR_ID_SENSE_DESC);
	return 0;		
}
//------------------------------------------------------------------------
int create_osd_key_specific_sense_data_desc(OSD_REQUEST *osd_req,u16 offset,u8 flags)
{
	KEY_SPECIFIC_SENSE_DESC *key_specific_sense;
	key_specific_sense=(KEY_SPECIFIC_SENSE_DESC *)((u8 *)osd_req->osd_sense_response.sense_buf+osd_req->osd_sense_response.sense_data_size);    
	key_specific_sense->key=OSD_KEY_SPECIFIC_SENSE_DATA_DESC_KEY;
	key_specific_sense->add_len=OSD_KEY_SPECIFIC_SENSE_DATA_DESC_ADD_LEN;
	//    u8 resvd[2];
	key_specific_sense->flags=flags;
	key_specific_sense->field_offset=offset;
	//u8 resvd
	osd_req->osd_sense_response.sense_data_size+=sizeof(KEY_SPECIFIC_SENSE_DESC);
	return 0;
}
//------------------------------------------------------------------------
int create_osd_cmnd_info_sense_data_desc(OSD_REQUEST *osd_req,u64 value)
{
	CMD_INFO_SENSE_DESC *cmd_info_sense;
	cmd_info_sense=(CMD_INFO_SENSE_DESC *)((u8 *)osd_req->osd_sense_response.sense_buf+osd_req->osd_sense_response.sense_data_size);     
	cmd_info_sense->key=OSD_CMD_INFO_SENSE_DATA_DESC_KEY;
	cmd_info_sense->add_len=OSD_CMD_INFO_SENSE_DATA_DESC_ADD_LEN;
	cmd_info_sense->value=value;
	osd_req->osd_sense_response.sense_data_size+=sizeof(CMD_INFO_SENSE_DESC);
	return 0;
}
//------------------------------------------------------------------------/----------------------------------------------------------------------
//**************    COMMAND_FUNCTIONS BIT SETTING FUNCS ******************
//----------------------------------------------------------------------

inline void set_command_functions_bit(OSD_REQUEST *osd_req,u8 command_byte,u8 bit_val)
{
	osd_req->completed_command_functions[command_byte]|=bit_val;	
	bit_val=255-bit_val;
	osd_req->not_initiated_command_functions[command_byte]&=bit_val;	
	/*
	set_completed_command_functions_bit(osd_req->completed_command_functions,command_byte,bit_val);
	set_not_initiated_command_functions_bit(osd_req->not_initiated_command_functions,command_byte,bit_val);
	*/
}
//----------------------------------------------------------------------
void init_not_initiated_command_functions_bit(u8 *command_functions,u8 command_byte,u8 bit_val)
{
	command_functions[command_byte]|=bit_val;	
}
//----------------------------------------------------------------------
 //----------------------------------------------------------------------
inline OSD_RESPONSE_HEADER build_osd_response_header(u8 status,u8 sense_key,u8  add_sense_code,u8  add_sense_code_qual,u32 data_len)
{
	OSD_RESPONSE_HEADER osd_response_hdr;
	osd_response_hdr.cmnd_status.status=status;
	osd_response_hdr.cmnd_status.sense_key=sense_key;
	osd_response_hdr.cmnd_status.add_sense_code=add_sense_code;
	osd_response_hdr.cmnd_status.add_sense_code_qual=add_sense_code_qual;
	osd_response_hdr.cmnd_status.sense_data_size=0;
	osd_response_hdr.data_len=data_len;
	return osd_response_hdr;
}
//----------------------------------------------------------------------
