/**
 *                      ==================
 *                      IITK OSD Simulator
 *                      ==================
 *   An Implementation of Simulator for Object based storage device(OSD)
 *
 *   Copyright (C) 2007-2008 CSE IIT KANPUR
 *
 *   Contact: Department of Computer Science and Engineering
 *            Indian Institute of Technology,Kanpur(IIT Kanpur)
 *            {moona,riyaz}@cse.iitk.ac.in
 *
 *   This file is part of IITK OSD Simulator Project.
 *
 *   IITK OSD Simulator Project is free software; you can redistribute it
 *   and/or modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   IITK OSD Simulator Project is distributed in the hope that it will be
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the  GNU General
 *   Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef OSD_SENSE_H
#define OSD_SENSE_H
# include "osd_socket.h"

/**
**********************************
 * File:  osd_sense.h
 * OSD sense data generation related macros and functions
 **/


/** defines macros for Sense values */
# define MAX_SENSE_BUF_SIZE 255
# define ISCSI_TARGET_NAME_LEN 256
//SCSI STATUS VALUES

/** defines macros for SCSI Sense values and Keys */
# define SCSI_GOOD 0
# define SCSI_CHECK_CONDITION 0x2
# define SCSI_SENSE_KEY_ILLEGAL_REQUEST 0x05
	# define SCSI_INVALID_FIELD_IN_CDB_ASC 0x24
	# define SCSI_INVALID_FIELD_IN_CDB_ASCQ 0
	# define SCSI_INVALID_FIELD_IN_PARAMETER_LIST_ASC 0x26
	# define SCSI_INVALID_FIELD_IN_PARAMETER_LIST_ASCQ 0
	# define SCSI_INVALID_OPCODE_ASC 0x20
	# define SCSI_INVALID_OPCODE_ASCQ 0
	# define SCSI_PARTITION_NOT_EMPTY_ASC 0x2C
	# define SCSI_PARTITION_NOT_EMPTY_ASCQ 0x0A
# define SCSI_SENSE_KEY_DATA_PROTECT 0x07
	# define SCSI_QUOTA_ERROR_ASC 0x55
	# define SCSI_QUOTA_ERROR_ASCQ 0x7
# define SCSI_RECOVERED_ERROR 0x01
	# define SCSI_READ_PAST_END_OF_USER_OBJECT_ASC 0x3B
	# define SCSI_READ_PAST_END_OF_USER_OBJECT_ASCQ	0x17
# define SCSI_NO_SENSE 0x0


# define SCSI_SENSE_KEY_MEDIUM_ERROR 0x03
# define SCSI_SENSE_KEY_HARDWARE_ERROR 0x04

//SCSI ADD SENSE CODE (ASC) and ADD SENSE CODE QUAL (ASCQ)
# define SCSI_INTERNAL_TARGET_FAILURE_ASC 0x44
# define SCSI_INTERNAL_TARGET_FAILURE_ASCQ 0


/** Macros for OSD specific sense descriptor according to T10 OSD-2 specification 4.12.2 and SPC-3 */
#define OSD_ERROR_ID_SENSE_DATA_DESC_TYPE 0x06
#define OSD_ERROR_ID_SENSE_DATA_DESC_ADD_LEN 0x1E
# define OSD_ATTR_ID_SENSE_DATA_DESC_TYPE 0x08
# define OSD_KEY_SPECIFIC_SENSE_DATA_DESC_KEY 0x02
# define OSD_KEY_SPECIFIC_SENSE_DATA_DESC_ADD_LEN 0x06
# define OSD_CMD_INFO_SENSE_DATA_DESC_KEY 0x01
# define OSD_CMD_INFO_SENSE_DATA_DESC_ADD_LEN  0x0A

/** Macros for OSD specific sense command processing status */
# define COMMAND_FUNCTIONS_COMMAND_BYTE 0
	# define COMMAND_FUNCTIONS_VALIDATION_BIT 0x80
	# define COMMAND_FUNCTIONS_CMD_CAP_V_BIT 0x20
	# define COMMAND_FUNCTIONS_COMMAND_BIT 0x10
# define COMMAND_FUNCTIONS_COMMAND_RESULT_SET_ATTR 1
	# define COMMAND_FUNCTIONS_IMP_ST_ATT_BIT 0x10
# define COMMAND_FUNCTIONS_SET_ATTR_BYTE 2
	# define COMMAND_FUNCTIONS_SA_CAP_V_BIT 0x20
	# define COMMAND_FUNCTIONS_SET_ATTR 0x10
# define COMMAND_FUNCTIONS_GET_ATTR_BYTE 3
	# define COMMAND_FUNCTIONS_GA_CAP_V_BIT 0x20
	# define COMMAND_FUNCTIONS_GET_ATTR 0x10


 
/**
 *
 *    Functions:    init_not_initiated_command_functions_bit
  *
 *    Description:  According to T10 OSD-2 standard 4.14.2.1.
 * 		    Sets bits for non initiated command functions
 * 
 *    Arguments:    @command_functions - Command function bits
 *		    @command_byte - Operation bit to set
 *		    @bit_val - Operation bit value
 *	
 *    Return codes: nothing 
 *
 **/

void init_not_initiated_command_functions_bit(u8 *command_functions,u8 command_byte,u8 bit_val);


/** OSD Error ID sense data descriptor strucure according to T10 OSD-2 standard 4.14.2.1 */
typedef struct 
{
    u8 desc_type; //6 object_id_sense_desc
    u8 add_len; //0x1E
    u8 resvd[6];
    u8 not_initiated_command_functions[4];
    u8 completed_command_functions[4];
    u64 part_id;
    u64 obj_id;
}ERROR_ID_SENSE_DESC;

/** OSD attribute ID  sense data descriptor strucure according to T10 OSD-2 standard 4.14.2.3 */

typedef struct {
    u8 key;     //8 attr_id_sense_data_desc
    u8 add_len; //=no attrs*8 - 2
    u8 resvd[2];
    u32 page_no;
    u32 attr_no;
}ATTR_ID_SENSE_DESC;


/** SCSI Command Info Sense descriptor according to spc3r16 4.5.2.2*/
typedef struct {
    u8 key;     //1
    u8 add_len; //0x0A
    u8 resvd[2];
    u64 value;
}CMD_INFO_SENSE_DESC;

/** SCSI Key Specific Sense descriptor according to spc3r16 4.5.2.4  */
typedef struct {
    u8 key;     //=2
    u8 add_len; //=6
    u8 resvd[2];
    u8 flags; //use OSD_SENSE_DESC_KEY_SPF_FLAGS_.. defines
    u16 field_offset;
    u8 resvd1;
} KEY_SPECIFIC_SENSE_DESC;

# define OSD_SENSE_KEY_SPF_CDB_FIELD_FLAG 0xC0
# define OSD_SENSE_KEY_SPF_DATA_FIELD_FLAG 0x80


/**
 *
 *    Functions:    create_osd_error_id_sense_data_desc
 *
 *    Description:  Creates Error ID sense data descriptor for OSD request 
 * 
 *    Arguments:    @osd_req - OSD request
 *	
 *    Return codes: 0 for success,1 in error 
 *
 **/

int create_osd_error_id_sense_data_desc(OSD_REQUEST *osd_req);



/**
 *
 *    Functions:    create_osd_attr_id_sense_data_desc
 *
 *    Description:  Creates Attr ID sense data descriptor for OSD request 
 * 
 *    Arguments:    @osd_req - OSD request
 *		    @page_no- Page no of Error attribute	
 *		    @attr_no- Attribute no of error attribute	
 *    Return codes: 0 for success,1 in error 
 *
 **/
int create_osd_attr_id_sense_data_desc(OSD_REQUEST *osd_req,u32 page_no,u32 attr_no);

/**
 *
 *    Functions:    create_osd_key_specific_sense_data_desc
 *
 *    Description:  Creates SCSI Key specific sense data descriptor  for OSD request 
 * 
 *    Arguments:    @osd_req - OSD request
		    @offset- offset in data buffer/cdb depending on flag value
 *		    @flag- can be 
 *			OSD_SENSE_KEY_SPF_CDB_FIELD_FLAG - for CDB field error
 *			OSD_SENSE_KEY_SPF_DATA_FIELD_FLAG - for data buffer field error
 *    Return codes: 0 for success,1 in error 
 *
 **/

int create_osd_key_specific_sense_data_desc(OSD_REQUEST *osd_req,u16 offset,u8 flag);


/**
 *
 *    Functions:    create_osd_cmnd_info_sense_data_desc
 *
 *    Description:  Creates SCSI Command info sense data descriptor  for OSD request 
 * 
 *    Arguments:    @osd_req - OSD request
		    @value - Value for command info
 *    Return codes: 0 for success,1 in error 
 *
 **/

int create_osd_cmnd_info_sense_data_desc(OSD_REQUEST *osd_req,u64 value);



/**
 *
 *    Functions:    build_osd_response_header
 *
 *    Description:  Builds OSD response header OSD_RESPONSE_HEADER
 * 
 *    Arguments:    @status  - OSD SCSI request status can be SCSI_GOOD or SCSI_CHECK_CONDITION
 *		    @sense_key - Value of SCSI sense key
 *		    @add_sense_code - Additional sense code SCSI ASC
 *		    @add_sense_code_qual - Value of SCSI ASCQ
 *		    @data_len - Data out buffere length
 *	
 *    Return codes: OSD_RESPONSE_HEADER with given values set
 *
 **/
OSD_RESPONSE_HEADER build_osd_response_header(u8 status,u8 sense_key,u8  add_sense_code,
						u8  add_sense_code_qual,u32 data_len);

//not implemented
int create_osd_rsp_int_chk_val_data_desc(OSD_REQUEST *osd_req);

#endif /* ifdef OSD_SENSE_H */



