/**
 *                      ==================
 *                      IITK OSD Simulator
 *                      ==================
 *   An Implementation of Simulator for Object based storage device(OSD)
 *
 *   Copyright (C) 2007-2008 CSE IIT KANPUR
 *
 *   Contact: Department of Computer Science and Engineering
 *            Indian Institute of Technology,Kanpur(IIT Kanpur)
 *            {moona,riyaz}@cse.iitk.ac.in
 *
 *   This file is part of IITK OSD Simulator Project.
 *
 *   IITK OSD Simulator Project is free software; you can redistribute it
 *   and/or modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   IITK OSD Simulator Project is distributed in the hope that it will be
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the  GNU General
 *   Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef OSD_SOCKET_H
#define OSD_SOCKET_H
#define SOCKET_MAX_READ 8192
#define ISCSI_TARGET_NAME_LEN 256
/**
**********************************
 * File:  osd_socket.h
 * Functions and strucutre related to socket communication with iSCSI module 
 **/


/** Structure for command status **/
typedef struct 
{
        u8 status; //SCSI_GOOD or SCSI_CHECK_CONDITION
	u8 sense_key; // SCSI Sense value
        u8 add_sense_code; // SCSI ASC
	u8 add_sense_code_qual; //SCSI ASCQ
	u16 sense_data_size; // SCSI data size
}COMMAND_STATUS;

/** structure for OSD response header send as response to OSD command*/

typedef struct 
{
COMMAND_STATUS cmnd_status;
u32 data_len;
}OSD_RESPONSE_HEADER;

/** Structure send by iSCSI module for OSD command processing request */
typedef struct 
{
        u8 scb[16];
        int ahs_len;
        u8 ahs_data[250];
        u32 data_len;
	u16 iov_size;
}OSD_HEADER;

/** OSD Lun information struct */
typedef struct {
char target_name[ISCSI_TARGET_NAME_LEN];
u32 lun_no;
}OSD_LUN_INFO;


/** Structure for receiving LUN info  and osd header  from OSD-iSCSI Module*/
typedef struct {
OSD_LUN_INFO osd_lun_info;
OSD_HEADER osd_header;
}OSD_COMMAND_INFO;

/** Structure used to transfer data IO vectors between OSD simulator and iSCSI modules */
typedef struct 
{
        int final;
        int data_len;
        u8 data[SOCKET_MAX_READ-16];
}OSD_DATA;

#endif /* ifdef OSD_SOCKET_H */
