/**
 *                      ==================
 *                      IITK OSD Simulator
 *                      ==================
 *   An Implementation of Simulator for Object based storage device(OSD)
 *
 *   Copyright (C) 2007-2008 CSE IIT KANPUR
 *
 *   Contact: Department of Computer Science and Engineering
 *            Indian Institute of Technology,Kanpur(IIT Kanpur)
 *            {moona,riyaz}@cse.iitk.ac.in
 *
 *   This file is part of IITK OSD Simulator Project.
 *
 *   IITK OSD Simulator Project is free software; you can redistribute it
 *   and/or modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   IITK OSD Simulator Project is distributed in the hope that it will be
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the  GNU General
 *   Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
# include "osd_types.h"
/**
**********************************
 * File:  osd_types.c
 * Implements Functions for conversion of 56/48 bit to 64 bits
 **/
inline uint64_t convert_48_64(uint48_t arr)
{	uint8_t i;
 	uint64_t a=0;
	uint8_t *src,*dest;
	src=arr.bytes;
	dest=(uint8_t *)&a;
	for(i=0;i<6;i++) dest[i]=src[i];
	dest[i]=0;
	dest[i+1]=0;
	return a; 
} 

inline uint48_t convert_64_48(uint64_t a)
{
 	uint8_t i;
	uint48_t arr;
	uint8_t *src,*dest;
	src=(uint8_t *)&a;
	dest=arr.bytes;
	for(i=0;i<6;i++) dest[i]=src[i];
	return arr; 
 
}
inline uint64_t convert_56_64(uint56_t arr) 
{
	uint8_t i;
 	uint64_t a=0;
	uint8_t *src,*dest;
	src=arr.bytes;
	dest=(uint8_t *)&a;
	for(i=0;i<7;i++) dest[i]=src[i];
	dest[i]=0;
	return a; 
}

inline uint56_t convert_64_56(uint64_t a )
{
	uint8_t i;
	uint56_t arr;
	uint8_t *src,*dest;
	src=(uint8_t *)&a;
	dest=(uint8_t *)arr.bytes;
	for(i=0;i<7;i++) dest[i]=src[i];
	return arr; 
 
}
 
inline uint48_t cpu_to_be48(uint48_t src)
{
#ifndef _BIG_ENDIAN	

	uint48_t dest;
	uint8_t *a,*arr;
	int i;
	a=(uint8_t *)dest.bytes;arr=(uint8_t *)src.bytes;
	
	for(i=0;i<6;i++)
	{
		a[i]=arr[5-i];
	}
	return dest;
#else
	return src;
#endif /** ifndef _BIG_ENDIAN */
}

inline uint48_t be48_to_cpu(uint48_t src)
{
#ifndef _BIG_ENDIAN	
	uint48_t dest;
	uint8_t *a,*arr;
	int i;
	a=(uint8_t *)dest.bytes;arr=(uint8_t *)src.bytes;
	for(i=0;i<6;i++)
	{
		a[i]=arr[5-i];
	}
	return dest;
#else
	return src;
#endif /**ifndef _BIG_ENDIAN	*/

}
