/**
 *                      ==================
 *                      IITK OSD Simulator
 *                      ==================
 *   An Implementation of Simulator for Object based storage device(OSD)
 *
 *   Copyright (C) 2007-2008 CSE IIT KANPUR
 *
 *   Contact: Department of Computer Science and Engineering
 *            Indian Institute of Technology,Kanpur(IIT Kanpur)
 *            {moona,riyaz}@cse.iitk.ac.in
 *
 *   This file is part of IITK OSD Simulator Project.
 *
 *   IITK OSD Simulator Project is free software; you can redistribute it
 *   and/or modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   IITK OSD Simulator Project is distributed in the hope that it will be
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the  GNU General
 *   Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#ifndef OSD_FILE_TYPES_H
#define OSD_FILE_TYPES_H

# include <stdint.h>
/**
**********************************
 * File:  osd_types.h
 * Declaration of 56 bit and 48 bit data types
 * Functions for conversion of 56/48 bit to 64 bits
 **/

/** 56 bit data type as array of 7 bytes */
typedef struct 
{
  uint8_t bytes[7];
}uint56_t; 

/** 56 bit data type as array of 6 bytes */
typedef struct 
{
 uint8_t bytes[6];
}uint48_t; 

 //Assume everything in Little endian
/**Functions to convert between 48 and 64 */
inline uint64_t convert_48_64(uint48_t arr);
inline uint48_t convert_64_48(uint64_t a);

/**Functions to convert between 56 and 64 */
inline uint64_t convert_56_64(uint56_t arr);
inline uint56_t convert_64_56(uint64_t a );

/** Network to host conversion  of 48 bit */
inline uint48_t cpu_to_be48(uint48_t src);
inline uint48_t be48_to_cpu(uint48_t src);

/** Network to host conversion  of 56 bit */
inline uint56_t cpu_to_be56(uint56_t src);
inline uint56_t be56_to_cpu(uint56_t src);

# endif

